//============================================================================
// FILE: CoreSimTime.h 
//
// TYPE: C++TYPEDEF
// DESCRIPTION: définition des types et fonctions "moteur" liées 
//              à la gestion du temps
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _CORESIMTIME_H
#define _CORESIMTIME_H

// SYSTEM INCLUDES
//

#include <iostream>

// PROJECT INCLUDES
//
#include "CoreTime.h"   // pour Date
#include "File.h"
  
/**@name Types et fonctions pour la gestion du temps simulé  
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include CoreSimTime.h
[/P] 
\end{itemize} 
On désignera par {\em temps simulé}\index{temps simulé} celui qui est représenté par le simulateur, par opposition au {\em temps réel}\index{temps réel} qui est celui qui s'écoule pendant que la simulation s'exécute. Pour illustration, une simulation qui dure une minute (en temps réel) peut représenter la vie d'un système pendant trois mois de temps simulé. La gestion du temps simulé (dates et durées) repose sur les points suivants. \\

\begin{enumerate}
\item L'utilisateur choisit l'unité dans laquelle il souhaite exprimer lui-même les {\em durées} (et dans laquelle il souhaite que les fonctions de manipulation du temps lui retournent les valeurs de durées). Cette unité est un certain nombre d'unités de temps "naturelles" : par exemple dix jours, une heure, quinze minutes, une seconde. Ainsi, avec l'unité "quinze minutes", une valeur de 4 aura le sens d'une heure. 
\noLF
Une fois l'unité choisie, elle doit être utilisée pour exprimer toutes durées relatives à toutes les classes d'entités et de façon permanente tout au long de la simulation. On peut cependant en changer d'une simulation à l'autre. L'utilisateur spécifie l'unité de temps simulé à l'aide des méthodes [Y]ClockTimeUnit[/Y] et [Y]ClockTimeUnitQuantity[/Y] de la classe [C]Simulation[/C]. L'horloge du simulateur mesure et fait avancer le temps simulé selon l'unité choisie. La valeur de l'horloge est toujours un nombre entier.\label{defTpsSimule}
 
\item Dans les fichiers de données en entrée, l'utilisateur exprime les {\em dates} en jours dans le mois, mois et millésime (et éventuellement en heure du jour, minute dans l'heure et seconde dans la minute). L'interpréteur des fichiers transforme ces dates soit en la structure prédéfinie [C]{tm[/C] de [F]time.h[/F], soit en la structure [C]Date[/C] de DIESE. Le code du simulateur peut alors utiliser normalement les opérateurs "[Y].[/Y]" ou "[Y]->[/Y]" pour accéder aux champs de ces structures (voir la page 'Constantes, structures et fonctions pour les dates et le temps en général'). 
 
\item L'utilisateur doit indiquer une {\em date de début} de la simulation, c'est-à-dire le point zéro du temps simulé. Pour cet instant, DIESE donne la valeur 0 à l'horloge de la simulation. A tout instant du déroulement de la simulation, la valeur de l'horloge représente donc la date courante relative, en nombre entier d'unités de temps simulé depuis la date de début de la période simulée. L'utilisateur dispose de plusieurs fonctions qui traduisent cette valeur d'horloge dans un format de date naturel, ou en une structure [C]Date[/C] ou [C]tm[/C] (voir la page 'Fonctions de manipulation du temps simulé').
\end{enumerate}\\

*/

//@{

         /**@name Unités de temps simulé.
DAY=0, HOUR, MINUTE, SECOND=3\\

Enumération exhaustive des unités de temps "naturelles" qui peuvent être utilisées pour définir l'unité de temps simulé.\index{TimeUnit}\label{defTimeUnit} \\

ATTENTION !\\
\begin{enumerate}
\item ne pas exprimer (avec des dates calendaires) un intervalle dont la taille réelle n'est pas la taille en secondes d'un nombre entier (n) d'unités d'horloge. e.g. avec unité = '2 HOUR' :
\begin{enumerate}
\itemvalide   : [30/MRS/08/0/0/0 -> 30/MRS/08/3/0/0], taille =  7200 (n=1)
\iteminvalide : [30/MRS/08/0/0/0 -> 30/MRS/08/1/0/0], taille =  3600 (n=?)
\iteminvalide : [30/MRS/08/0/0/0 -> 30/MRS/08/4/0/0], taille = 10800 (n=?) 
\end{enumerate}
\item avec DAY, les valeurs éventuellement produites d'heures, minutes, secondes sont non significatives et généralement erronées (utiliser le mode DATE_DMY ou ISO-8601) 
\item avec HOUR, une unité d'horloge '24 HOUR' n'est pas équivalente à une unité d'horloge '1 DAY' (parce qu'il y a deux jours dans l'année, ceux des passages entre heure d'été et heure d'hiver, qui n'ont pas une durée de 24 heures)  
\item avec HOUR, une progression de '2 HOUR' peut faire passer de 0h à 3h (le jour du passage hiver->été) et peut faire passer de 1h à 2h (le jour du passage été->hiver)
\item quelle que soit l'unité d'horloge, un instant situé à l'intérieur d'un intervalle de temps unitaire ne peut pas être correctement restitué (notamment : affiché). {\em e.g.} avec l'unité = '2 HOUR' et un début simulation en '30 MRS 08', l'instant '30 MRS 08 1 0 0' sera restitué comme '30 MRS 08'
\end{enumerate}
*/
//@{
enum TimeUnit {
DAY, HOUR, MINUTE, SECOND
};
//@}

         /**@name Les formats de date.
CLOCK=0, DATE\_MHJ, DATE\_JHM, DATE\_jHM, DATE\_HDM, DATE\_DMY, ISO\_8601\\

Ces symboles d'énumération ci-dessus sont utilisés par les fonctions [Y]PrintDateFromClock[/Y], [Y]CoutDateFromClock[/Y] et [Y]OutputDateFromClock[/Y] pour imprimer et afficher sur la sortie standard ou dans un flux quelconque (respectivement) une date passée en nombre d'unités de temps simulé.\\
Dans ces fonctions, un autre argument est donc le symbole, qui provoque le formattage suivant :

\begin{description}
\item[CLOCK :] pas de formattage (simple valeur entière)\\

\item[DATE\_MHJ :] minutes\_dans\_heure/heure\_dans\_jour/jour\_julien)\\

\item[DATE\_JHM :] jour\_julien/heure\_dans\_jour/minutes\_dans\_heure)\\

\item[DATE\_jHM :] jours\_depuis\_début/heure\_dans\_jour/minutes\_dans\_heure)\\

\item[DATE\_HDM :] heure\_dans\_jour/jour\_dans\_mois/mois)\\

\item[DATE\_DMY :] jour\_dans\_mois/mois/année)\\

\item[ISO\_8601 :] année-mois-jour hh:mm:ss
\end{description} 
\index{TimeType} 
*/  
//@{
enum TimeType {
CLOCK, DATE_MHJ, DATE_JHM, DATE_jHM, DATE_HDM, DATE_DMY, ISO_8601
};
//@}

/**@name Structure d'une durée.
Une durée simulée est un certain nombre (entier) d'une certaine unité de temps naturelle (par exemple "15 minutes"). Le nombre ([Y]mQuantity[/Y]) et l'unité naturelle ([Y]mTimeUnit[/Y]) sont les deux données membres de cette structure.\\

Un usage particulier de cette structure est fait par le moteur de simulation : c'est par elle qu'on code l'unité de temps simulé (c'est-à-dire la durée de la plus petite période indivisible de la simulation), attribut de la classe [C]Simulation[/C].\\

Le code de l'application peut utiliser cette structure de durée pour tout autre motif :\\

[P]
\begin{verbatim}
{
  ...
  TimeDuration myDuration;
  myDuration.mTimeUnit = MINUTE;
  myDuration.mQuantity = 15;
  printf("\nLa durée de myDuration est %d %s.", 
         myDuration.mQuantity, TimeUnitToChar(myDuration.mTimeUnit));
  ...
}
\end{verbatim}
[/P]
\begin{verbatim}
>>
La durée de myDuration est 15 MINUTE.
\end{verbatim}
\index{TimeDuration} 
*/        
//@{
         /// 
struct TimeDuration
{
         /// L'unité de temps naturelle. 
  TimeUnit        mTimeUnit;
         /// Valeur de la durée en nombre de cette unité naturelle. 
  int             mQuantity; 
};         
//@} 

// TIME FUNCTIONS
/**@name Fonctions de manipulation du temps simulé */
//@{

/**@name Pour imprimer ou afficher une valeur d'horloge ... */
//@{
/** Imprime la date passée en nombre d'unités de temps simulé depuis le début de la simulation. La destination est le fichier {\small {\tt file}}. Le format est passé en troisième argument.\index{fonctions!PrintDateFromClock} */
void PrintDateFromClock(FILE* file, int, TimeType);
void PrintDate(FILE* file, int, TimeType);  // pour compatibilité 4.4

/** Affiche sur la sortie standard la date passée en nombre d'unités de temps simulé depuis le début de la simulation. Le format est passé en deuxième argument.\index{fonctions!TraceDateFromClock} */
void TraceDateFromClock(int, TimeType);  // + 090817

/* * Affiche sur la sortie standard la date passée en nombre d'unités de temps simulé depuis le début de la simulation. Le format est passé en deuxième argument.\index{fonctions!CoutDateFromClock} */
void CoutDateFromClock(int, TimeType);
void CoutDate(int, TimeType);  // pour compatibilité 4.4

/* * Affiche sur un flux quelconque la date passée en nombre d'unités de temps simulé depuis le début de la simulation. La destination est {\small {\tt stream}}. Le format est passé en deuxième argument.\index{fonctions!OutputDateFromClock} */
void OutputDateFromClock(int, TimeType, std::ostream& stream);

/** Renvoie une chaîne de caractères évoquant le mois de l'année dont le numéro d'ordre est passé en argument (avec JAN : 1).*/
char* MonthToChar(int m);
//@}

/**@name Sinon, lorsqu'une valeur d'horloge est en argument ou bien renvoyée ... */
//@{
/** Renvoie le nombre d'unités de temps simulé entre le début de la période simulée et la date fournie en 6 arguments. 
Les arguments sont, dans l'ordre, le jour, le mois (ou le {\em ShortMonthName}), le millésime (par exemple 2001), l'heure, la minute et la seconde. Renvoie -9 si la date est antérieure au début de la simulation.\index{fonctions!DateToClockValue} */
int DateToClockValue(int, int, int, int, int, int);
int DateToClock(int, int, int, int, int, int);  // pour compatibilité 4.4

/** Renvoie le nombre d'unités de temps simulé entre le début de la période simulée et la date fournie en 3 arguments. 
Les arguments sont, dans l'ordre, le jour, le mois (ou le {\em ShortMonthName}), le millésime (par exemple 2001). L'heure, la minute et la seconde sont considérées nulles. Renvoie -9 si la date est antérieure au début de la simulation.*/
int DateToClockValue(int, int, int);

/** Renvoie le nombre d'unités de temps simulé entre le début de la période simulée et la date fournie en un argument (structure [C]tm[/C]). Renvoie-9 si la date est antérieure au début de la simulation.\index{fonctions!StructTmToClockValue} */ 
int StructTmToClockValue(tm* pTm);  // + JPR 5.0 090803

/** Renvoie une date (pointeur sur une structure [C]tm[/C]) à partir de la valeur de temps simulé fourni en argument, interprétée comme le temps écoulé depuis le début de la simulation. 
Si l'argument fourni est la valeur courante de l'horloge, la valeur retournée représente la date simulée courante.\index{fonctions!ClockValueToTm} */
tm*    ClockValueToTm(int t);
tm*    ClockToTm(int t);  // pour compatibilité 4.4

/** Renvoie une date (structure [C]Date[/C]) à partir de la valeur de temps simulé fourni en argument, interprétée comme le temps écoulé depuis le début de la simulation. 
Si l'argument fourni est la valeur courante de l'horloge, la valeur retournée représente la date simulée courante.\index{fonctions!ClockValueToDate} */
Date    ClockValueToDate(int t);
Date    ClockToDate(int t);  // pour compatibilité 4.4

/** Traduit en nombre de jours la quantité d'unités d'horloge passée en argument. 
Si l'argument fourni est la valeur courante de l'horloge, la valeur retournée représente le nombre de jours écoulés depuis le début de la période de simulation. \index{fonctions!ClockValueToDays} */
int    ClockValueToDays(int t);
int    ClockToDays(int t);  // pour compatibilité 4.4

/** Renvoie une date en format ISO à partir de la valeur de temps simulé fournie en argument, interprétée comme le temps écoulé depuis le début de la simulation. 
Si l'argument fourni est la valeur courante de l'horloge, la valeur retournée représente la date simulée courante. La valeur retournée doit être désallouée par le code client.\index{fonctions!ClockValueToStringISO} */
char*  ClockValueToStringISO(int t);
//@}

/**@name Lorsque un intervalle de temps est en argument ou bien renvoyé ... */
//@{

/** Traduit un nombre de secondes en un nombre d'unités de temps simulé. 
Par exemple, si l'unité de temps simulé est "quinze minutes", l'appel [P]TimeIntervalToClockNumber(3600)[/P] renverra {\small {\tt 4}}.\index{fonctions!TimeIntervalToClockNumber} */
int TimeIntervalToClockNumber(time_t);
int TimeIntervalToClockInterval(time_t);  // pour compatibilité 4.4

/** Traduit une durée exprimée en unité de temps simulé en un nombre de secondes. 
Par exemple, si l'unité de temps simulé est "quinze minutes", l'appel [P]ClockIntervalToTimeInterval(4)[/P] renverra {\small {\tt 3600}}. Attention ! Si l'unité de temps est en DAY, n'utiliser cette fonction que si les deux bornes de l'intervalle se situent dans la même période d'horaire (été ou hiver).\index{fonctions!ClockNumberToTimeInterval} */
time_t ClockNumberToTimeInterval(int);
time_t ClockIntervalToTimeInterval(int);  // pour compatibilité 5.7

/** Traduit en un nombre de secondes la durée (exprimée en unité de temps simulé) d'un intervalle dont les bornes sont positionnées dans les période d'été et d'hiver. Les deux arguments spécifient la période d'horaire des deux bornes (1 si été, 0 si hiver).*/
time_t ClockNumberToTimeInterval(int n, int isdst_beg, int isdst_end);
time_t ClockIntervalToTimeInterval(int n, int isdst_beg, int isdst_end);  // pour compatibilité 5.7

/** Renvoie le nombre de secondes entre le début de la période simulée et la date fournie en argument. 
 Les arguments sont, dans l'ordre, le jour, le mois (ou le {\em ShortMonthName}), le millésime (par exemple 2001), l'heure, la minute et la seconde. Renvoie -9 si la date est antérieure au début de la simulation.\index{fonctions!DateToRelativeTime} */
time_t DateToRelativeTime(int, int, int, int, int, int);
//@}


//@}  // fin FUNCTIONS
   
//X///////////////X/////////////////////////////X////////////////////////////
//
// fonctions de parsing
//
//X///////////////X/////////////////////////////X////////////////////////////

         //
int IndexMonth (char* m);
         //
TimeUnit IndexUnite (char* m);
         //
char* TimeUnitToChar(TimeUnit tu);
     
//@}   // fin

//@}  // ouverture dans CoreSimTime.h



// EXTERNAL REFERENCES
  

#endif  // _CORESIMTIME_H
