//============================================================================
// FILE: CoreTime.h 
//
// TYPE: C++TYPEDEF
// DESCRIPTION: définition des types et fonctions "moteur" liées 
//              à la gestion du temps
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _CORETIME_H
#define _CORETIME_H

// SYSTEM INCLUDES
//
#include <sys/time.h> 
#include <time.h> // /opt/gnu/lib/gcc-lib/sparc-sun-solaris2.5/2.7.2.1/include

// PROJECT INCLUDES
// 
#include "CoreTypes.h"
 
// !!! attention sous solaris, clock() renvoie toujours un nombre de la forme
//     x.x000 microsecondes, c'est-à-dire x.x millisecondes
//     la précision en dessous de la milliseconde n'est pas disponible


extern long gSecDeb;
extern long gUSecDeb;
extern long gMicrocumul;
extern struct timeval* mytp;

extern char gTimeString[50];


/**@name CDPFORTEXLa gestion du temps 

Les événements discrets sur lesquels repose la simulation entreprise avec la bibliothèque sont, par conception, positionnés dans le temps. Il sont ainsi conçus pour représenter le contrôle d'un système dont on veut étudier la dynamique. Dès lors, il est nécessaire de représenter le temps comme une grandeur, de savoir associer à des instants des valeurs de cette grandeur, de mesurer et comparer des intervalles entre deux instants. \\

\D fournit deux ensembles de structures et de services pour représenter et gérer le temps :
\begin{itemize}
\item Pour la gestion de la dynamique du système, la notion de base est la {\em période de simulation}\index{période de simulation}, située entre le moment de la vie réelle d'un système à partir duquel on décrit sa dynamique et le moment après lequel on ne la décrit plus. Par rapport au moment on l'utilisateur réalise la simulation, la période de simulation peut naturellement être antérieure (on décrit ce qui s'est passé) ou postérieure (on décrit ce qui se passera). On appelle {\em temps simulé}\index{temps simulé} le temps sur lequel s'inscrit la période de simulation. Un ensemble de services sert à se gestion. A titre d'exemple, on peut calculer la durée entre le début de la période de simulation et l'instant présent dans la dynamique du système. \\

Dans le temps simulé, les durées (longueur des intervelles entre deux instants de la période de simulation) sont un certain nombre d'{\em unités de temps simulé}\index{temps simulé - unité}. Cette unité repose nécessairement sur une unité de temps "naturelle" parmi, {\em a priori} et entre autres, la seconde, la minute, l'heure\ldots Dans \D, l'{\em unité de temps simulé} est un certain nombre d'unités de temps "naturelles" : par exemple dix jours, une heure, quinze minutes, une seconde. Parmi les services dans cet ensemble, figurent donc les traductions entre le temps naturel et le temps simulé. Par exemple, si l'{\em unité de temps simulé} est 'quinze minutes', une durée de 8 est traduite en deux heures, et {\em vice versa}.

\item Un autre ensemble de services sert à exprimer et à manipuler le temps en général, c'est-à-dire sans référence au {\em temps simulé}. Par exemples, \D offre une représentation particulière pour une date calendaire, des outils de traduction (entre différents formats de date, ou entre heure locale et heure UTC) ou encore des services de chronométrage des durées dans l'espace-temps de l'utilisateur (typiquement pour mesurer la durée des simulations).
\end{itemize}
*/
//@{  // fermeture dans CoreSimTime.h

/**@name Constantes, structures et fonctions pour les dates et le temps en général.
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include CoreTime.h
[/P]
\end{itemize} 
 Ces éléments sont définis hors le contexte d'une simulation, c'est-à-dire que les dates, durées, \ldots ne portent pas nécessairement sur une période sujette à simulation.
*/

//@{
         /**@name Représentation symbolique des mois de l'année.
JAN=1, [FEV ou FEB]=2, MRS, [AVR ou APR]=4, MAI, JUN, \\
JUL=7, [AOU ou AUG]=8, SEP, OCT, NOV, DEC=12\\

Enumération de symboles pour les mois de l'année. \\

En entrée (notamment dans les fichiers externes) ces symboles (ou de façon indifférente les entiers correspondants) sont utilisables comme valeur entière d'un mois. \\

En sortie, les éléments de cette énumération sont par défaut exprimés par leur valeur entière. L'utilisation de la variable globale [Y]ShortMonthNameToCharTab[/Y] (indicée par le symbole du mois) permet une visualisation en chaîne de caractères.\\

Noter que le mois de janvier porte le numéro 1 (en tenir compte lors de l'utilisation de la structure [C]tm[/C], dans laquelle c'est la valeur 0 qui correspond au mois de janvier). L'utilisateur peut écrire une fonction utilisant ce numéro en argument et renvoyant le nom du mois en chaîne de caractères.
\index{ShortMonthName}\label{defShortMonthName}
*/
//@{
enum ShortMonthName {JAN=1, FEV, MRS, AVR, MAI, JUN, JUL, AOU, SEP, OCT, NOV, DEC};
enum ShortMonthName_e {     FEB=2,    APR=4,              AUG=8};
//@}

         /**@name Représentation symbolique des jours de la semaine.
[DIM ou SUN]=0,\\ 
[LUN ou MON], \\
[MAR ou THU], \\
[MER ou WED], \\
[JEU ou TUE], \\
[VEN ou FRI], \\
[SAM ou SAT}=6\\

Enumération de symboles pour les jours de la semaine. \\

En entrée (notamment dans les fichiers externes) ces symboles (ou de façon indifférente les entiers correspondants) sont utilisables comme valeur entière d'un jour de la semaine.\\

En sortie, les éléments de cette énumération sont par défaut exprimés par leur valeur entière. L'utilisation de la variable globale [Y]ShortWeekDayNameToCharTab[/Y] (indicée par le symbole du jour) permet une visualisation en chaîne de caractères.\\

Noter que le dimanche porte le numéro 0, comme dans la structure [C]tm[/C].
\index{ShortWeekDayName} 
*/
//@{
enum ShortWeekDayName {DIM=0, LUN=1, MAR=2, MER=3, JEU=4, VEN=5, SAM=6};
enum ShortWeekDayName_e {SUN=0, MON=1, THU=2, WED=3, TUE=4, FRI=5, SAT=6};
//@}

         /**@name Structure d'une date.
Cette classe est une des formes utilisables pour exprimer une date, comme argument des fonctions de manipulation du temps. Les données membres sont publiques, donc accessibles directement, comme dans l'exemple ci-dessous.\\

[P]
\begin{verbatim}
  Date* myDate = new Date();
  myDate.year = 2001;
  printf("\nle millésime pour myDate est %d.", myDate.year);
\end{verbatim}
[/P]
\begin{verbatim}
>>
Le millésime pour myDate est 2001.
\end{verbatim}
\index{Date} 
	 */
//@{
         ///
class Date {  // _no swig_
 public:
/* *@name LIFECYCLE */
// @{
         /// Constructeur sans argument. La date n'est pas précisée. On value et on accède aux composantes de la date par référence directe. Par exemple [P]myDate.year = 2005;[/P].
  Date();
  ~Date(); // + 091113 to scratch
         /// Constructeur par copie.
  Date(const Date&);
// @} 

/* *@name OPERATOR */
// @{

          /// Opérateur de test d'égalité. Les deux dates sont égales (et l'opérateur renvoie 1) si tous les composants sont égaux\index{Date!operator!==}
      int            operator == (const Date& source);
// @} 

/**@name Les composantes de la date */
//@{
         /// Le millésime (par exemple 2001).
  int        year;
         /// Le mois (1->12).
  int        month;
         /// Le jour du mois (1->31).
  int        mday;
         /// L'heure du jour (0->23).
  int        hour;
         /// La minute (0->59).
  int        minute;
         /// La seconde (0->59).
  int        second;
//@}
}; 
//@} // fin Structure d'une date

    
         /**@name Une structure C prédéfinie : tm.
Autre forme utilisable pour exprimer une date. On rappelle que la structure [C]tm[/C] est ainsi prédéfinie (dans [F]time.h[/F]) :\\

[P]
\begin{verbatim}
struct tm {

  int tm_sec;     // secondes (0->61)

  int tm_min;     // minutes (0->59)

  int tm_hour;    // heures (0->23

  int tm_mday;    // jour du mois (1->31

  int tm_mon;     // mois depuis janvier (0->11)

  int tm_year;    // années depuis 1900 (100, pour 2000)

  int tm_wday;    // jours depuis le dimanche (0->6)

  int tm_yday;    // jours depuis le 1er janvier (0->365)

  int tm_isdst;   // indicateur d'horaire : été=1/hiver=0
}
\end{verbatim}
[/P]

... exploitable, pour illustration, de la manière suivante :

[P]
\begin{verbatim}
{
  ...
  tm* myTm = new tm();
  myTm->tm_year = 2001;
  printf("\nLe millésime pour myTm est %d.", myTm->tm_year);
  ...
}
\end{verbatim}
[/P]
\begin{verbatim}
>>
Le millésime pour myTm est 2001.
\end{verbatim}
\index{tm} 
*/
//@{
struct tm;
//@}                             

// MISCELLANEOUS
/**@name D\'efinitions de types et variables globales
\begin{itemize}
\item Le type des variables "internes" mesurant le temps en secondes : {\em time\_t} (entier long).\index{time\_t}
\item [Y]gTimeString[/Y] : variable globale (de type {\em char*}) recevant une valeur en effet de bord de certaines fonctions générales de manipulation du temps (voir page suivante).\index{gTimeString}
\end{itemize}
 
*/  
//@{      
typedef long time_t;
//@}        

// TIME FUNCTIONS
/**@name Fonctions g\'en\'erales de manipulation du temps*/
//@{ 

/**@name Chronométrage

Le chronométrage repose sur l'usage de la fonction C [Y]gettimeofday(timeval* T, NULL)[/Y], qui établit la valeur de {\em T} comme la date de l'instant d'appel, avec une précision d'un centième de seconde. Il suffit alors de faire la différence entre les deux valeurs de {\em T} établies par deux appels en deux instants pour avoir la durée entre les deux appels. \\

Le premier appel est réalisé par [Y]StartChrono()[/Y] et le second par [Y]PauseChrono()[/Y]. \\
Préalablement, le chronomètre a été rendu prêt à fonctionner par [Y]InitChrono()[/Y], et il sera déactivé par [Y]CloseChrono()[/Y].\\
Après un [Y]CloseChrono()[/Y], un [Y]InitChrono()[/Y] est à nouveau nécessaire avant un nouveau [Y]StartChrono()[/Y]. \\
On peut enchaîner plusieurs couples [Y]StartChrono()[/Y] / [Y]PauseChrono()[/Y] entre un  [Y]InitChrono()[/Y] et un [Y]CloseChrono()[/Y], notamment pour cumuler les durées d'exécution d'un bloc de calcul invoqué de manière répétée (par exemple itérativement). \\

Enfin, les fonctions de la famille [Y]PrintChrono()[/Y] rangent la durée mesurée dans la variable globale [Y]gTimeString[/Y] (type {\em char*}), qu'on peut alors afficher ou imprimer. Une séquence typique est donc : 
[P]
\begin{verbatim}
void f() {
  StartChrono();
  ...
  PauseChrono();
}
int main(...) {
  InitChrono();
  for(int k=0;k<1000++) {
    f();
  }
  CloseChrono();
  PrintChrono();
  printf("\nduree cumulee d'execution de 'f' : %s\n", gTimeString);
\end {verbatim}
[/P]
>>
\begin{verbatim}
duree cumulee d'execution de 'f' : 0h01'18
\end {verbatim}
*/
//@{ 

/** Crée un chronomètre pour mesurer le temps réel et le met à zéro. 
Un seul chronomètre peut être en vie à un moment donné.\index{fonctions!InitChrono} */
void InitChrono();
/** Déclenche le chronomètre créé par [Y]InitChrono[/Y], ou le remet en marche s'il a été arrêté par [Y]PauseChrono[/Y].\index{fonctions!StartChrono}   */
void StartChrono();
/** Arrête le chronomètre déclenché par [Y]StartChrono[/Y]. 
Ne remet pas le compteur à zéro.\index{fonctions!PauseChrono}    */
void PauseChrono();
/** Ferme le chronomètre créé par [Y]InitChrono[/Y] et arrêté par [Y]PauseChrono[/Y].\index{fonctions!CloseChrono}  */
void CloseChrono();
/** Place dans la chaîne de caractères [Y]gTimeString[/Y] l'indication du chronomètre mis en pause, avec secondes, millisecondes et microsecondes.\index{fonctions!MicroPrintChrono}  */
void MicroPrintChrono();
/** Place dans la chaîne de caractères [Y]gTimeString[/Y] l'indication du chronomètre mis en pause, avec secondes et millisecondes seulement.\index{fonctions!MilliPrintChrono}  */
void MilliPrintChrono();
/** Place dans la chaîne de caractères [Y]gTimeString[/Y] l'indication du chronomètre mis en pause, avec heures, minutes et secondes.\index{fonctions!PrintChrono}  */
void PrintChrono();
//@}

/**@name Affichage du temps en chaînes de caractères
*/
//@{ 
/** Traduit une date exprimée selon la structure [C]tm[/C] (incluant une heure supposée locale) en une chaîne de caractères (par exemple "30/3/97 13h0'0'' (heure d'été)"). 
La mention "heure d'été" est affichée, le cas échéant, si le second argument est TRUE. La fonction [Y]asctime[/Y] de [F]time.h[/F]} rend un service semblable. Le résultat est placé dans la variable globale [Y]gTimeString[/Y].\index{fonctions!StructTmToChar} */
void   StructTmToChar(tm*, Boolean);

/** Même effet que la fonction précédente, mais la mention "heure d'été" n'est jamais affichée.*/
void   StructTmToChar(tm*);

/** Traduit une date exprimée selon la structure [C]Date[/C] en une date sous forme de chaîne de caractères (par exemple "30/3/97 13h0'0''"). 
Que l'heure traduite soit ou non une heure d'été, il n'en n'est pas fait mention. Le résultat est placé dans la variable globale [Y]gTimeString[/Y].\index{fonctions!StructDateToChar} */
void   StructDateToChar(Date*);
//@}


/**@name Fonctions renvoyant un temps absolu (depuis le 1er janvier 1970)
*/
//@{ 
/** Renvoie le nombre de secondes entre le 1er janvier 1970 (0h) et la date dont les composantes (jour, mois, \ldots) sont passées en arguments, et exprimée en heure supposée locale. 
Les arguments sont, dans l'ordre, le jour, le mois, le millésime (par exemple 2001), l'heure, la minute et la seconde.\index{fonctions!DateToAbsoluteTime} */
time_t DateToAbsoluteTime(int, int, int, int, int, int);

/**  Même effet que la fonction précédente, avec une structure [C]Date[/C] en argument.\index{fonctions!StructDateToAbsoluteTime} */
time_t StructDateToAbsoluteTime(Date*);
time_t DateToAbsoluteTime2(Date*);  // pour compatibilité 4.4

/** Renvoie le nombre de secondes entre le 1er janvier 1970 (0h) et la date dont les composantes (jour, mois, \ldots) sont passées en arguments, et exprimée en heure UTC (heure de référence internationale).
Les arguments sont, dans l'ordre, le jour, le mois, le millésime (par exemple 2001), l'heure, la minute et la seconde.\index{fonctions!UTCDateToAbsoluteTime} */
time_t UTCDateToAbsoluteTime(int, int, int, int, int, int);

/**  Même effet que la fonction précédente, avec une structure [C]Date[/C] en argument.\index{fonctions!StructDateToAbsoluteTime} */
time_t StructUTCDateToAbsoluteTime(Date*);
//@}


/**@name Fonctions renvoyant une structure tm
*/
//@{ 
/** Renvoie une date (pointeur sur une structure [C]tm[/C]) à partir de la date dont les composantes (jour, mois, \ldots) sont passées en arguments, et exprimée en heure supposée locale. 
Les arguments sont, dans l'ordre, le jour, le mois, le millésime (par exemple 2001), l'heure, la minute et la seconde. Les champs de la structure tm renvoyée reflètent l'heure locale.\index{fonctions!DateToTm} */
tm*    DateToTm(int, int, int, int, int, int);

/** Même effet que la fonction précédente, avec une structure [C]Date[/C] en argument.\index{fonctions!StructDateToTm} */
tm*    StructDateToTm(Date*); // _no swig_
tm*    DateToTm2(Date*); // _no swig_  // pour compatibilité 4.4   DateToTm(int, int, int, int, int, int);

/** Renvoie une date (pointeur sur une structure [C]tm[/C]) à partir de la date dont les composantes (jour, mois, \ldots) sont passées en arguments, et exprimée en heure UTC (heure de référence internationale). 
Les arguments sont, dans l'ordre, le jour, le mois, le millésime (par exemple 2001), l'heure, la minute et la seconde. Les champs de la structure tm renvoyée reflètent l'heure locale.\index{fonctions!DateToTm} */
tm*    UTCDateToTm(int, int, int, int, int, int);

/** Renvoie une date (pointeur sur une structure [C]tm[/C]) à partir de la date fournie en argument, incluant une heure UTC (heure de référence internationale). 
L' argument est une structure [C]Date[/C]. Les champs de la structure tm renvoyée reflètent l'heure locale.\index{fonctions!DateToTm} */
tm*    StructUTCDateToTm(Date*);

/** Renvoie une date (pointeur sur une structure [C]tm[/C]) à partir de la date en format ISO fournie en argument. Le format en entrée est le suivant : "aaaa-mm-qqThh:mi:sszzzzzz" (ex : 1997-07-16T19:20:30+01:00). Si le décalage horaire n'est pas '+00:00', cette date comprend une heure locale. Si le décalage horaire est '+00:00', cette date comprend une heure locale transformée en heure UTC ou bien une heure locale sur le méridien de longitude zéro.\index{fonctions!StringISOToTm} */
tm*    StringISOToTm(char*);

/** Traduit la date courante réelle (celle au moment où la fonction est appelée), exprimée en heure supposée locale, en une structure [C]tm[/C].\index{fonctions!CurrentRealDateToTm} */
tm*    CurrentRealDateToTm();
//@}


/**@name Fonction renvoyant une structure Date
*/
//@{ 

/** Renvoie une date (pointeur sur une structure [C]Date[/C]) à partir de la date (structure [C]tm[/C]) fournie en argument. L'argument et la structure renvoyée reflètent l'heure locale. \index{fonctions!StructTmToDate} */
Date*   StructTmToDate(tm* pTm);
//@}

/**@name Fonction générant une date en format ISO ({\em char*})
*/
//@{ 
/** Forme une date en format ISO à partir de la date (pointeur sur une structure [C]tm[/C]) fournie en argument, sans hypothèse sur l'heure (locale ou UTC) que véhicule la structure. Le format renvoyé est le suivant : "aaaa-mm-qqThh:mi:sszzzzzz" (ex : 1997-07-16T19:20:30+01:00). Le résultat est placé dans la variable globale [Y]gTimeString[/Y].\index{fonctions!StructTmToStringISO} */
void   StructTmToStringISO(tm*);

/** Même effet que la fonction précédente, mais renvoie la chaîne résultat. La valeur retournée doit être désallouée par le code client.\index{fonctions!StringISOFromStructTm} */
char*  StringISOFromStructTm(tm*);

/** Forme une date (ajustée en heure locale) en format ISO à part<ir de la date dont les composantes (jour, mois, \ldots) sont passées en arguments et considérée en heure UTC. Les arguments sont, dans l'ordre, le jour, le mois, le millésime (par exemple 2001), l'heure, la minute et la seconde. Le format construit est le suivant : "aaaa-mm-qqThh:mi:sszzzzzz" (ex : 1997-07-16T19:20:30+01:00). Les deux dates en entrée et en sortie correspondent au même temps absolu. Le résultat est placé dans la variable globale [Y]gTimeString[/Y].\index{fonctions!StructTmToStringISO} */
void   UTCDateToLocalStringISO(int, int, int, int, int, int);

/** Forme une date UTC en format ISO à partir de la date (pointeur sur une structure [C]tm[/C]) fournie en argument, et incluant une heure locale. Les deux dates en entrée et en sortie correspondent au même temps absolu. Le format renvoyé est le suivant : "aaaa-mm-qqThh:mi:sszzzzzz" (ex : 1997-07-16T19:20:30+01:00). Le résultat est placé dans la variable globale [Y]gTimeString[/Y].\index{fonctions!StructTmToStringISO} */
void   StructLocalTmToUTCStringISO(tm*);
//@}

/* * Retarde l'instruction suivante de {\em ms} millièmes de seconde environ.\index{fonctions!Diese_millisleep} 
*/
void Diese_millisleep(int ms);

//@}  // fin FUNCTIONS
     
//@}   // fin



// EXTERNAL REFERENCES
  

#endif  // _CORETIME_H
