//============================================================================
// FILE: Entity.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des entités
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _ENTITY_HH
#define _ENTITY_HH

// SYSTEM INCLUDES
//
//#include <>                                   //

// PROJECT INCLUDES
//
#include "DescriptedEntity.h"

/**@name CDPFORTEXLa classe des entit\'es avec descripteurs et m\'ethodes
\begin{itemize} \item Directive d'inclusion :
[P]
\#include Entity.h
[/P]
\end{itemize}
Les classes héritant de la classe de base [C]Entity[/C] sont équipées de descripteurs fonctionnels, en plus des variables d'état et des descripteurs constants héritées de la classes [C]DescriptedEntity[/C]. Si une classe d'entité dans le domaine n'est pas caractérisée par des descripteurs fonctionnels, il suffit d'en faire une spécialisation de [C]DescriptedEntity[/C] ou, si elle ne possède ni variables d'état ni descripteurs constants, de [C]BasicEntity[/C].
*/
//@{
           ///
class Entity : public DescriptedEntity
{
public:

  /**@name LIFECYCLE */
//@{
          /// Constructeur de base, invoquant d'abord le constructeur de base de [C]DescriptedEntity[/C]. L'instance créée possède le type O\_ENTITY, valeur renvoyée par [Y]EntityType[/Y] et qui surcharge celle donnée par le constructeur de [C]DescriptedEntity[/C].
   Entity(); // _no swig_

          /// Constructeur par copie profonde, invoquant d'abord le constructeur par copie de [C]DescriptedEntity[/C]. Les instances de [C]Method[/C] sont copiées, et leur 'objet décrit' devient la nouvelle entité. Ajoute l'entité en cours de création par copie à la liste des instances (indirectes) de [C]Entity[/C]. 
   Entity(const Entity&); // _no swig_    
   Entity(const Entity&, bool triggerMonitor); // _no swig_   
          /// Même effet que le constructeur ci-dessus, mais les éléments ou les composants ne sont pas nécessairement copiés. Si l'argument 'partsCopied' est FALSE, ce rôle est laissé aux constructeurs par copie des sous-classes de l'application. 
   Entity(const Entity&, int partsCopied); // _no swig_    
   Entity(const Entity&, bool triggerMonitor, int partsCopied); // _no swig_   

          /// Destructeur superficiel (les composants et les éléments ne sont pas détruits ; les descripteurs et les méthodes sont détruits ; le démon attaché n'est pas détruit). Enlève l'entité en cours de destruction de la liste des instances (directes) de sa propre classe, puis de la liste des instances des classes parentes, jusqu'à la classe [C]Entity[/C] (méthode [Y]RemoveInstanceFromClass[/Y]).
   ~Entity(); // _no swig_
//@} 

/**@name [SC]Opérations sur la liste de méthodes[/SC] */
//@{
  
         /// Ajoute la méthode à la liste des méthodes du récepteur, s'il n'y est pas déjà. Lève l'exception [Y]BadMethodType[/Y].\index{méthodes!AddMethod - Entity}
   void            AddMethod(Method*); // _no swig_

         /// Renvoie la méthode dont le symbole de classe est passé en argument. Lève l'exception [Y]InexistantMethod[/Y].\index{méthodes!GetMethod - Entity}
   Method*             GetMethod(UserMethodId); // _no swig_ 

         /// Renvoie la méthode dont le nom de classe est passé en argument. Lève l'exception [Y]InexistantMethod[/Y].
   Method*             GetMethod(char*); // _no swig_

         /// Renvoie la méthode dont le symbole de classe est passé en argument, et NULL si la méthode n'existe pas ou si elle ne possède pas de corps.\index{méthodes!GetMethodWithBody - Entity}
   Method*             GetMethodWithBody(UserMethodId); // _no swig_

         /// Renvoie {\em vrai} si la méthode dont le symbole de classe est passé en argument existe et si elle possède un corps, et {\em faux} dans les autres cas.\index{méthodes!HasBody - Entity}
   bool                HasBody(UserMethodId); // _no swig_

         // Service 'moteur'. Renvoie la table des méthodes.
   pMethodTab          MethodTab() const; // _no swig_

//@}


/**@name [SC]Affectation d'un corps aux méthodes[/SC]
L'appel [Y]AssignBody[/Y]\index{méthodes!AssignBody - Entity} affecte un corps à une méthode (au sens d'une instance de [C]Method[/C]) dont le symbole de classe est donné en premier argument.\\ 

Ces appels lèvent les exceptions [Y]BadMethodType[/Y] et [Y]InexistantMethod[/Y].
*/
//@{
          
         /// 
   void            AssignBody(UserMethodId, void (*f)(EntityMethod*)); // _no swig_     
         /// 
   void            AssignBody(UserMethodId, bool (*f)(EntityMethod*)); // _no swig_     
         /// 
   void            AssignBody(UserMethodId, int (*f)(EntityMethod*)); // _no swig_     
         /// 
   void            AssignBody(UserMethodId, float (*f)(EntityMethod*)); // _no swig_     
         /// 
   void            AssignBody(UserMethodId, double (*f)(EntityMethod*)); // _no swig_     
         /// 
   void            AssignBody(UserMethodId, BasicEntity* (*f)(EntityMethod*)); // _no swig_
         /// 
   void            AssignBody(UserMethodId, char* (*f)(EntityMethod*)); // _no swig_     
         /// 
   void            AssignBody(UserMethodId, pEntityTab* (*f)(EntityMethod*)); // _no swig_
//@}


/**@name [SC]Affectation d'une valeur aux arguments des méthodes[/SC]
L'appel [Y]Set..ArgValue[/Y]\index{méthodes!Set..ArgValue - Entity} affecte, pour la méthode (au sens d'une instance de [C]Method[/C]) dont le symbole de classe est donné en premier paramètre, la valeur passée en troisième paramètre à l'argument (au sens d'une instance d'[C]Argument[/C]) dont le symbole de classe est passé en deuxième paramètre. \\ 

Ces appels lèvent l'exception [Y]InexistantMethod[/Y].
*/ 
//@{
          
         /// 
   void            SetIntArgValue(UserMethodId, UserArgumentId, int);
         /// 
   void            SetFloatArgValue(UserMethodId, UserArgumentId, float);
         /// 
   void            SetDoubleArgValue(UserMethodId, UserArgumentId, double);
         /// 
   void            SetEntityArgValue(UserMethodId, UserArgumentId, BasicEntity*);
         /// 
   void            SetEntityTabArgValue(UserMethodId, UserArgumentId, pEntityTab*);
         /// 
   void            SetStringArgValue(UserMethodId, UserArgumentId, char*);
         /// 
   void            SetIntTabArgValue(UserMethodId, UserArgumentId, TableauInt*);
         /// 
   void            SetFloatTabArgValue(UserMethodId, UserArgumentId, TableauFloat*);
         /// 
   void            SetDoubleTabArgValue(UserMethodId, UserArgumentId, TableauDouble*);
//@} 


/**@name [SC]Invocation des méthodes[/SC] 
Les appels [Y]Exec..Method[/Y]\index{méthodes!Exec..Method - Entity} exécutent le corps de la méthode (au sens d'une instance de [C]Method[/C]) dont le symbole de classe est donné en paramètre, et renvoient éventuellement la valeur renvoyée par ce corps. \\ 

Ces appels lèvent les exceptions [Y]BadMethodType[/Y] et [Y]InexistantMethod[/Y].
*/
//@{
          
         /// 
   void            ExecVoidMethod(UserMethodId);
         /// 
   bool            ExecPredicate(UserMethodId);
         /// 
   int             ExecIntMethod(UserMethodId);
         /// 
   float           ExecFloatMethod(UserMethodId);
         /// 
   double          ExecDoubleMethod(UserMethodId);
         /// 
   BasicEntity*    ExecEntityMethod(UserMethodId);
         /// 
   char*           ExecStringMethod(UserMethodId);
         /// 
   pEntityTab*     ExecEntityTabMethod(UserMethodId);
         /// 
   pFloatTab*      ExecFloatTabMethod(UserMethodId); // JPR 051114
         /// 
   pIntTab*        ExecIntTabMethod(UserMethodId); // JPR 091207
//@} 
   
// INQUIRY

protected:

   pMethodTab             mpMethodTab;            // + JPR 0204
   TableauInt             mpDualMethodTab;

  // début code gestion OutputSpec
  // fin code gestion OutputSpec

protected:

// PRIVATE OPERATIONS

// PRIVATE ACCESS

};  // _end class_

//@} // fin classe

//@}         // ouverture dans BasicEntity.h

#endif  // _ENTITY_HH
