//============================================================================
// FILE: Method.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des méthodes
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _METHOD_HH
#define _METHOD_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Argument.h"

//-X///////////////X/////////////////////////////X////////////////////////////


/**@name CDPFORTEXAttributs fonctionnels : les m\'ethodes
Le système simulé est composé d'entités entre lesquelles existent des relations structurales (de composition, ensembliste), gérées par des services de la classe [C]BasicEntity[/C]. Ces services, par exemple [Y]GetElement(int k)[/Y], sont des méthodes (au sens du C++ natif) de la classe [C]BasicEntity[/C]. Dans le code de la classe, ce sont de simples fonctions-membres :
\begin{verbatim}
[P]Entity* BasicEntity::GetElement(int k) {
  Entity* result = NULL;
  ...  // recherche de l'élément de rang k
  return result;
}[/P]
\end{verbatim}
Une telle méthode est spécifiée au niveau de la classe d'objets et est commune à tous les objets instances de la classe.\\

Les "méthodes" présentées dans cette page ne sont pas des services de la même nature : ce sont elles-mêmes des objets (complexes) qui permettent de spécifier des attributs fonctionnels d'autres objets. Un attribut essentiel de ces objets  "méthodes" est leur {\em corps} qui, lui, est une simple fonction. Par exemple, si le service [Y]GetElement(int k)[/Y] n'était pas prédéfini, on doterait la classe [C]Entity[/C] d'une "méthode" [C]GetElementMethod[/C], héritant de [C]EntityMethod[/C], dont le {\em corps} serait le suivant :
\begin{verbatim}
[P]Entity* getElementMethod_Body(EntityMethod* pM) {
  int rank = pM->GetIntArgValue(RANK);
  Entity* result = NULL;
  ...  // recherche de l'élément de rang 'rank'
  return result;
}[/P]
\end{verbatim}
On note ici, et on précisera plus loin, que :
\begin{itemize}
\item le corps de la méthode comporte un et un seul argument : un pointeur (ici {\small {\tt pM}}, sur la méthode dont il est le {\em corps} ;
\item une méthode est dotée d'arguments, eux-même objets de la classe [C]Argument[/C], et que leurs valeurs sont accessibles à l'intérieur du {\em corps}.
\end{itemize}

{\large {\tt Les différentes spécialisations de méthodes}}

\begin{itemize}
\item Les entités (classe [C]Entity[/C]) sont dotées de méthodes (classe [C]EntityMethod[/C]) qui pécifient un comportement de l'entité ou calculent la valeur d'une fonction prenant l'entité en paramètre (fonction sur son état ou sur ses relations avec d'autres entités).
\item Les processus ([C]Process[/C]) sont dotés de méthodes (classe [C]ProcessMethod[/C]) qui spécifient l'état du système lors de leur initialisation, l'effet qu'ils ont sur le système lors de leur poursuite, et une procédure accompagnant éventuellement leur arrêt.
\item Les méthodes (classe [C]EventMethod[/C]) des événements ([C]Event[/C]) spécifient leur post-conséquences et la manière dont ils sont autogénérés.
\item Les démons ([C]DescValueMonitor[/C] et [C]StructureMonitor[/C]) ont des méthodes (classe [C]MonitorMethod[/C]) qui sont déclenchées réactivement.
\item Enfin, les spécifications d'ensembles d'entités (classe [C]EntitySpec[/C]) sont dotées de méthodes (classe [C]EntitySpecMethod[/C]) qui spécifient les critères de sélection des entités ou la manière d'en construire la liste désirée.
\end{itemize}

{\large {\tt Les attributs des méthodes}}

Une méthode en général (instance de [C]Method[/C]) est caractérisée essentiellement par les propriétés suivantes :
\begin{itemize}
\item un {\em corps}, qui est un morceau de programme qui effectue le traitement caractéristique de la méthode et qui renvoie éventuellement une valeur au module à l'intérieur duquel la méthode a été invoquée ;
\item un type, qui est celui de la valeur renvoyée par la méthode (plus précisément par le {\em corps} de la méthode), ou bien {\em void} si le corps de la méthode ne renvoie pas de valeur ;
\item la liste des classes des arguments : les arguments de la méthode sont des valeurs, évaluées dynamiquement au moment de l'invocation de la méthode, qui sont exploitées par le {\em corps} de la méthode pour adapter le traitement à ces valeurs courantes ;
\item la liste des arguments eux-mêmes, qui sont des instances indirectes de [C]Argument[/C] (voir page 'Les classes de base pour représenter les arguments') ; 
\item enfin l'objet duquel la méthode est une connaissance fonctionnelle, par exemple une instance d'[C]Entity[/C] pour les [C]EntityMethod[/C]. Il est important de remarquer que l'adresse de cet objet, en tant que donnée membre de la classe, est accessible dans le corps de la méthode (pour suivre l'exemple, par [Y]DescribedEntity()[/Y]), dont elle est alors de fait une sorte de paramètre constant.
\end{itemize}

A chaque type de valeur renvoyée par le {\em corps} correspond une sous classe de [C]EntityMethod[/C], [C]ProcessMethod[/C], [C]EventMethod[/C], \ldots, selon le cas. Dans \D, le type d'une méthode peut être l'un des suivants : 
\begin{itemize}
\item un des types de descripteurs (voir page 'La classe de base pour tous les descripteurs'), à savoir un entier ou un réel (les types {\em int}, {\em float} et {\em double} de C), une chaîne de caractères (le type {\em char*} de C), 
\item une instance d'[C]Entity[/C] ou encore un tableau de pointeurs sur des [C]Entity[/C] ({\em pEntityTab}) ; 
\item les types {\em void} et {\em bool} de C.
\end{itemize}

Accessoirement, une méthode est caractérisée par les propriétés suivantes :
\begin{itemize}
\item son symbole de classe, utilement choisi de telle sorte qu'il rappelle la nature de l'information qu'elle renvoie ou le contenu du traitement qu'elle effectue. Par exemple, GET_ELEMENT_METHOD ;  
\item son nom de classe, qui a le même rôle que le symbole, mais sous la forme d'une chaîne de caractères. Par exemple, {\small {\tt "getElementMethod"}}.
\end{itemize}

{\large {\tt L'invocation des méthodes}}

L'invocation d'une méthode, instance de [C]Method[/C], attachée par exemple à une entité, n'est pas très différente de l'invocation de la même méthode qui serait installée comme une fonction membre classique [Y]GetElement(int)[/Y] de l'entité. L'appel classique :
\begin{verbatim}
[P]  int k = 1;
  Entity* pElem_k = pObj->GetElement(k);[/P]
\end{verbatim}

devient :
\begin{verbatim}
[P]  pObj->SetIntArgValue(GET_ELEMENT_METHOD, RANK, 1);
  Entity* pElem_k = pObj->ExecIntMethod(GET_ELEMENT_METHOD);[/P]
\end{verbatim}

ou encore :
\begin{verbatim}
[P]  EntityEntityMethod* pM = pObj->GetMethod(GET_ELEMENT_METHOD);
  pM->SetArgValue(RANK, 1);
  Entity* pElem_k = pM->Body(pM);[/P]
\end{verbatim}

où {\small {\tt GET_ELEMENT_METHOD}} est le symbole de la sous-classe de [C]Method[/C] dont le corps a le même code que [Y]GetElement(int)[/Y], et dont {\small {\tt pM}} est l'instance attachée à {\small {\tt pObj}}.\\

{\large {\tt Pourquoi les méthodes sont-elles des classes ?}}

L'utilisateur de \Db peut trouver les avantages suivants au codage d'une connaissance fonctionnelle par une instance de [C]Method[/C] :
\begin{itemize}

\item l'unification de la structure des entités du domaine : une série de fonctions membres est remplacée dans tous les cas par un unique attribut de type 'tableau de pointeurs sur des instances de [C]Method[/C]' (la même remarque peut être faite sur l'unification de la représentation de variables d'état et des constantes, sous la forme de tableaux de pointeurs sur des instances de [C]Descriptor[/C]) ;

\item une connaissance fonctionnelle est séparée de l'entité (ou processus ou événement) sur laquelle elle porte, parce qu'implantée dans un objet [C]Method[/C], lequel n'est que référencé par l'entité ; cela augmente la déclarativité et la granularité de la représentation, et autorise l'attachement dynamique et temporaire de connaissances fonctionnelles à une entité ;

\item conséquence de l'item précédent, la possibilité de distinguer deux instances de la même sous-classe d'[C]Entity[/C] (ou de [C]Process[/C] ou d'[C]Event[/C], etc.) par deux corps distincts de la même méthode ;

\item enfin, les corps des instances de [C]Method[/C] ont une signature commune unique (le seul argument est justement l'instance de [C]Method[/C] dont le corps est attribut) : cette unicité permet de proposer un mécanisme de génération automatique de code (dans l'outil Solfege, associé à la bibliothèque \D).
\end{itemize}

{\large {\tt Remarques diverses et générales}}

\begin{itemize}

\item Les constructeurs de base des sous-classes directes de [C]Method[/C] établissent le type de l'objet décrit, précisé dans la documentation de chaque sous-classe.

\item Les constructeurs par copie des sous-classes directes copient l'objet décrit (entité, processus ou événement, etc.) et invoquent le constructeur par copie de la classe de base [C]Method[/C].

\item Les opérateurs d'affectation des sous-classes indirectes invoquent les opérateurs d'affectation des arguments, et la méthode à gauche pointe sur le même corps que celle à droite.

\item Les services tels que [Y]DescribedEntity(pObj)[/Y], [Y]DescribedProcess(pObj)[/Y], \ldots), désignent l'entité (ou le processus, ou l'événement, \ldots) {\small {\tt pObj}} comme l'objet dont la méthode est un attribut fonctionnel.\label{defDescribedEntity}\label{defDescribedProcess}

\item Les services tels que [Y]DescribedEntity()[/Y], [Y]DescribedProcess()[/Y], \ldots), renvoient l'entité (ou le processus, ou l'événement, \ldots) dont la méthode est un attribut fonctionnel.

\item Chaque sous-classe terminale de méthode possède un membre [Y]Body[/Y], le corps de la méthode, dont la valeur est un pointeur sur une fonction écrite par l'utilisateur. A l'image de l'exemple codé plus haut, l'invocation du corps peut se faire de deux manières :
\begin{enumerate}

\item [un message à l'objet décrit : ] la séquence suivante demande à l'entité {\small {\tt pObj}} d'exécuter sa méthode (à valeur entière) dont le symbole de classe est F, après lui avoir demander de valuer l'argument à valeur entière dont le symbole de classe est DATE :
\begin{verbatim}
[P]  pObj->SetIntArgValue(F, DATE, 12); 
  int x = pObj->ExecIntMethod(F);[/P]
\end{verbatim}

\item [un message à la méthode : ] la séquence suivante identifie d'abord la méthode, puis adresse des messages à cette méthode (valuation de son argument, puis invocation du corps) :
\begin{verbatim}
[P]  IntEntityMethod pM = pObj->GetMethod(F);
  pM->SetArgValue(DATE, 12);
  int x = pM->Body(pM);[/P]
\end{verbatim}
\end{enumerate}

\item Les services [Y]AssignBody[/Y] affectent un corps à la méthode.
\end{itemize}
*/
//@{  // Method + CoreMethod

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Les classes de base pour représenter les m\'ethodes
\begin{itemize} \item Directive d'inclusion :
[P]
\#include Method.h
[/P]
\end{itemize}\label{defMethod}
 */
//@{  // classe de base + ss-classes par types d'objet

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name La classe de base */
//@{
           /// 
class Method
{
public:

/**@name LIFECYCLE */
//@{
          // Constructeur de base.
  Method();  // _no swig_
          /// Constructeur par copie. Les arguments sont copiés.
  Method(const Method&);  // _no swig_
          /// Destructeur.                
  ~Method(void);  // _no swig_
//@}

/* *@name OPERATORS */
// @{

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const Method&) const {return TRUE;};
  Boolean         operator > (const Method&) const {return TRUE;};

// @}

/**@name Acc\`es g\'en\'eral */
//@{
         /// Renvoie le type d'objet dont la méthode est un attribut fonctionnel ([C]Entity[/C], [C]Process[/C], [C]Event[/C]).\index{méthodes!ObjectClass - Method}
   ObjectClassId      ObjectClass();  // _no swig_
         /// Affecte un type (entier, flottant, \ldots) à la méthode qui reçoit ce message.\index{méthodes!ClassType - Method}
   void              ClassType(ValueTypeId);  // _no swig_
         /// Renvoie le type de la méthode.
   ValueTypeId       ClassType();  // _no swig_
         /// Affecte un symbole de classe à la méthode qui reçoit ce message.\index{méthodes!ClassSymbol - Method}
   void              ClassSymbol(UserMethodId);  // _no swig_
         /// Renvoie le symbole de classe de la méthode.
   UserMethodId      ClassSymbol();  // _no swig_
         /// Affecte un nom de classe à la méthode qui reçoit ce message.\index{méthodes!ClassName - Method}
   void              ClassName(char*);  // _no swig_
         /// Renvoie le nom de classe de la méthode.
   char*             ClassName();  // _no swig_
         /// Affecte un commentaire court à la méthode qui reçoit ce message.\index{méthodes!ClassShortComment - Method}
   void              ClassShortComment(char*);  // _no swig_
         /// Renvoie le commentaire court de la méthode.
   char*             ClassShortComment();  // _no swig_
         /// Renvoie l'argument dont le symbole de classe est passé en paramètre. Lève l'exception [Y]InexistantArgument[/Y].\index{méthodes!GetArg} 
   Argument*         GetArg(UserArgumentId);  // _no swig_
         /// Renvoie le tableau des arguments.\index{méthodes!GetArg} 
   pArgumentTab         ArgumentTab() const;  // _no swig_
         /// Renvoie {\em vrai} si un corps a été assigné à la méthode (par un appel à [Y]AssignBody[/Y].\index{méthodes!HasBodyAssigned} 
   bool              HasBodyAssigned();  // _no swig_
//@}

/**@name Op\'erations sur les arguments */ 
//@{ 
         /// Ajoute un argument à la liste des arguments.\index{méthodes!AddArg}
   void              AddArg(Argument*);  // _no swig_ 

//@}

   /**@name Affectation d'une valeur à un argument
Ces appels affectent une valeur à l'argument dont le symbole de classe est passé en premier paramètre. Ils lèvent l'exception [Y]BadArgumentType[/Y].\index{méthodes!SetArgValue}
*/
   //@{  
         /// 
   void              SetArgValue(UserArgumentId, int);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, float);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, double);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, BasicEntity*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, pEntityTab*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, char*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId id, pIntIntervalTab*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, DescValueSpec*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, pDescValueSpecTab*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, pIntTab*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, pFloatTab*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, pDoubleTab*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, Event*);  // _no swig_
         /// 
   void              SetArgValue(UserArgumentId, Descriptor*);  // _no swig_
   //@}

   /**@name Acc\`es aux valeurs des arguments
Ces appels renvoient la valeur de l'argument dont le symbole de classe est passé en paramètre. Ils lèvent l'exception [Y]BadArgumentType[/Y].\index{méthodes!Get..ArgValue}
 */ 
   //@{
         /// 
   int               GetIntArgValue(UserArgumentId);
         /// 
   float             GetFloatArgValue(UserArgumentId);
         /// 
   double            GetDoubleArgValue(UserArgumentId);
         /// 
   BasicEntity*      GetBasicEntityArgValue(UserArgumentId);
         /// 
   Entity*           GetEntityArgValue(UserArgumentId);
         /// 
   pEntityTab*       GetEntityTabArgValue(UserArgumentId);
         /// 
   char*             GetStringArgValue(UserArgumentId);
         /// 
   pIntIntervalTab*  GetIntDomainArgValue(UserArgumentId);
         /// 
   DescValueSpec*    GetDescValueSpecArgValue(UserArgumentId);
         /// 
   pDescValueSpecTab*  GetDescValueSpecTabArgValue(UserArgumentId);
         /// 
   pIntTab*        GetIntTabArgValue(UserArgumentId);
         /// 
   pFloatTab*        GetFloatTabArgValue(UserArgumentId);
         /// 
   pDoubleTab*        GetDoubleTabArgValue(UserArgumentId);
         /// 
   Event*            GetEventArgValue(UserArgumentId);
         /// 
   Descriptor*       GetDescriptorArgValue(UserArgumentId);
   //@}


    // comptage
   //static pMethodTab  mpInstanceTab;           // + JPR 060316  // - JPR 071109
   static int          mInstanceNumber;   
         // 
   void             RemoveInstance(Method*);  // _no swig_     

   //static pMethodTab  InstanceTab();  
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;} 

// INQUIRY

protected:

   int static   _nbConst;
   int static   _nbDest;

   ObjectClassId     mObjectClassId;
   ValueTypeId       mMethodType;
   UserMethodId      mMethodClassId;
   char*             mMethodClassName;
   char*             mMethodShortComment;

   // -----------------
   // | 0 | 1 | 2 | 3 |  <- indice
   // -----------------
   // | 7 | 2 | 8 | 1 |  <- symbole de l'argument
   // -----------------
   pArgumentTab      mpArgTab;

   // ce tableau contient en position k l'indice du tableau 
   // mpArgTab correspondant à l'argument de symbole k
   // une valeur -9 indique que l'argument de symbole k n'existe pas
   // -------------------------------------
   // | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 |  <- indice
   // -------------------------------------
   // |-9 | 3 | 1 |-9 |-9 |-9 |-9 | 0 | 2 |  <- indice dans mpArgTab
   // -------------------------------------
   TableauInt        mpDualArgTab;

   bool              mHasBodyAssigned;
   

private:
}; // _end class_

//@} // fin classe de base

//-X///////////////X/////////////////////////////X///////////////////////////
// 
// classes des Méthodes d'ENTITES
//
//-X///////////////X/////////////////////////////X///////////////////////////

/**@name Les m\'ethodes d'entit\'es
L'objet dont la méthode est un attribut fonctionnel est une instance d'[C]Entity[/C].\\

De façon générale :
\begin{itemize}
\item Les constructeurs de base des sous-classes d'[C]EntityMethod[/C] affectent un symbole et un nom de classe, ainsi qu'un type, précisés dans la documentation de chaque sous-classe.
\item Les constructeurs par copie des sous-classes d'[C]EntityMethod[/C] copient le corps, et font appel au constructeur par copie de la classe [C]EntityMethod[/C].
\end{itemize}
*/
//@{

/**@name Classe de base pour les m\'ethodes d'entit\'es */
//@{
          /// 
class EntityMethod : public Method
{
public:
          /// Constructeur de base. Affecte le type d'objet O\_ENTITY. 
  EntityMethod();  // _no swig_
          /// Constructeur par copie. 
  EntityMethod(const EntityMethod& source);  // _no swig_ 

          /// .\index{méthodes!DescribedEntity - EntityMethod}
void DescribedEntity(Entity* pObj);  // _no swig_
          /// 
Entity* DescribedEntity();

protected:
   Entity*           mpDescribedObject;
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es sans valeur renvoy\'ee */
//@{
          /// 
class VoidEntityMethod : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  VoidEntityMethod();  // _no swig_
          /// Constructeur de base. Affecte le type de méthode VOID. 
  VoidEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  VoidEntityMethod(const VoidEntityMethod& source);
         // Opérateur d'affectation.
   const VoidEntityMethod&   operator=(const VoidEntityMethod&);

          /// \index{méthodes!AssignBody - VoidEntityMethod}
void           AssignBody(void (*f)(EntityMethod*));
          /// \index{méthodes!Body - VoidEntityMethod}
void          (*Body)(EntityMethod*);
}; // _end class_   

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es à valeur boole\`enne */
//@{
          /// 
class EntityPredicate : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  EntityPredicate();
          /// Constructeur de base. Affecte le type de méthode BOOLEAN. 
  EntityPredicate(UserMethodId, char*);
          /// Constructeur par copie.
  EntityPredicate(const EntityPredicate& source);
         // Opérateur d'affectation. 
   const EntityPredicate&      operator=(const EntityPredicate&);

          /// \index{méthodes!AssignBody - EntityPredicate}
void           AssignBody(bool (*f)(EntityMethod*));
          /// \index{méthodes!Body - EntityPredicate}
bool       (*Body)(EntityMethod*);
};  // _end class_  

//@} // fin classe

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es à valeur enti\`ere */
//@{
          /// 
class IntEntityMethod : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  IntEntityMethod();
          /// Constructeur de base. Affecte le type de méthode INT. 
  IntEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  IntEntityMethod(const IntEntityMethod& source);
         // Opérateur d'affectation. 
   const IntEntityMethod&      operator=(const IntEntityMethod&);

          /// \index{méthodes!AssignBody - IntEntityMethod}
void           AssignBody(int (*f)(EntityMethod*));
          /// \index{méthodes!Body - IntEntityMethod}
 int           (*Body)(EntityMethod*);
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es à valeur flottante */
//@{
          /// 
class FloatEntityMethod : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  FloatEntityMethod();
          /// Constructeur de base. Affecte le type de méthode FLOAT. 
  FloatEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  FloatEntityMethod(const FloatEntityMethod& source);
         // Opérateur d'affectation. 
   const FloatEntityMethod&      operator=(const FloatEntityMethod&);

          /// \index{méthodes!AssignBody - FloatEntityMethod}
void           AssignBody(float (*f)(EntityMethod*));
          /// \index{méthodes!Body - FloatEntityMethod}
 float         (*Body)(EntityMethod*);
}; // _end class_  

//@} // fin classe

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es à valeur flottante en double précision */
//@{
          /// 
class DoubleEntityMethod : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  DoubleEntityMethod();
          /// Constructeur de base. Affecte le type de méthode DOUBLE. 
  DoubleEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  DoubleEntityMethod(const DoubleEntityMethod& source);
         // Opérateur d'affectation. 
   const DoubleEntityMethod&      operator=(const DoubleEntityMethod&);

          /// \index{méthodes!AssignBody - DoubleEntityMethod}
void           AssignBody(double (*f)(EntityMethod*));
          /// \index{méthodes!Body - DoubleEntityMethod}
 double        (*Body)(EntityMethod*);
}; // _end class_  

//@} // fin classe

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es à valeur 'pointeur sur une entit\'e' */
//@{
           ///
class EntityEntityMethod : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  EntityEntityMethod();
          /// Constructeur de base. Affecte le type de méthode P\_ENTITY. 
  EntityEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  EntityEntityMethod(const EntityEntityMethod& source);
         // Opérateur d'affectation. 
   const EntityEntityMethod&      operator=(const EntityEntityMethod&);

          /// \index{méthodes!AssignBody - EntityEntityMethod}
void           AssignBody(BasicEntity* (*f)(EntityMethod*));
          /// \index{méthodes!Body - EntityEntityMethod}
 BasicEntity*       (*Body)(EntityMethod*);
}; // _end class_ 

//@} // fin classe

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es à valeur 'chaine de caract\`eres' */
//@{
          /// 
class StringEntityMethod : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  StringEntityMethod();
          /// Constructeur de base. Affecte le type de méthode STRING. 
  StringEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  StringEntityMethod(const StringEntityMethod& source);
         // Opérateur d'affectation. 
   const StringEntityMethod&      operator=(const StringEntityMethod&);

          /// \index{méthodes!AssignBody - StringEntityMethod}
void           AssignBody(char* (*f)(EntityMethod*));
          /// \index{méthodes!Body - StringEntityMethod}
 char*         (*Body)(EntityMethod*);
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'entit\'es à valeur 'tableau de pointeurs sur des entit\'es' */
//@{
          /// 
class EntityTabEntityMethod : public EntityMethod  // _no swig_
{
public:
          // Constructeur de base.
  EntityTabEntityMethod();
          /// Constructeur de base. Affecte le type de méthode ENTITY\_TAB. 
  EntityTabEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  EntityTabEntityMethod(const EntityTabEntityMethod& source);
         // Opérateur d'affectation. 
   const EntityTabEntityMethod&      operator=(const EntityTabEntityMethod&);

          /// \index{méthodes!AssignBody - EntityTabEntityMethod}
   void           AssignBody(pEntityTab* (*f)(EntityMethod*));
          /// \index{méthodes!Body - EntityTabEntityMethod}
   pEntityTab*     (*Body)(EntityMethod*);
}; // _end class_ 

//@} // fin classe   

//-X///////////////X/////////////////////////////X///////////////////////////
 
/**@name M\'ethodes d'entit\'es à valeur 'tableau d'entiers' */
//@{
          /// 
class IntTabEntityMethod : public EntityMethod   // JPR 051114  // _no swig_
{
public:
          // Constructeur de base.
  IntTabEntityMethod();
          /// Constructeur de base. Affecte le type de méthode INT\_TAB. 
  IntTabEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  IntTabEntityMethod(const IntTabEntityMethod& source);
         // Opérateur d'affectation. 
   const IntTabEntityMethod&      operator=(const IntTabEntityMethod&);

          /// \index{méthodes!AssignBody - IntTabEntityMethod}
   void           AssignBody(pIntTab* (*f)(EntityMethod*));
          /// \index{méthodes!Body - IntTabEntityMethod}
   pIntTab*     (*Body)(EntityMethod*);
}; // _end class_
//@} // fin classe  

//-X///////////////X/////////////////////////////X///////////////////////////
 
/**@name M\'ethodes d'entit\'es à valeur 'tableau de flottants' */
//@{
          /// 
class FloatTabEntityMethod : public EntityMethod   // JPR 051114  // _no swig_
{
public:
          // Constructeur de base.
  FloatTabEntityMethod();
          /// Constructeur de base. Affecte le type de méthode FLOAT\_TAB. 
  FloatTabEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  FloatTabEntityMethod(const FloatTabEntityMethod& source);
         // Opérateur d'affectation. 
   const FloatTabEntityMethod&      operator=(const FloatTabEntityMethod&);

          /// \index{méthodes!AssignBody - FloatTabEntityMethod}
   void           AssignBody(pFloatTab* (*f)(EntityMethod*));
          /// \index{méthodes!Body - FloatTabEntityMethod}
   pFloatTab*     (*Body)(EntityMethod*);
}; // _end class_
//@} // fin classe  

//-X///////////////X/////////////////////////////X///////////////////////////
 
/**@name M\'ethodes d'entit\'es à valeur 'tableau de flottants DP' */
//@{
          /// 
class DoubleTabEntityMethod : public EntityMethod   // JPR 051114  // _no swig_
{
public:
          // Constructeur de base.
  DoubleTabEntityMethod();
          /// Constructeur de base. Affecte le type de méthode DOUBLE\_TAB. 
  DoubleTabEntityMethod(UserMethodId, char*);
          /// Constructeur par copie.
  DoubleTabEntityMethod(const DoubleTabEntityMethod& source);
         // Opérateur d'affectation. 
   const DoubleTabEntityMethod&      operator=(const DoubleTabEntityMethod&);

          /// \index{méthodes!AssignBody - DoubleTabEntityMethod}
void           AssignBody(pDoubleTab* (*f)(EntityMethod*));
          /// \index{méthodes!Body - DoubleTabEntityMethod}
pDoubleTab*     (*Body)(EntityMethod*);
}; // _end class_
//@} // fin classe 

//@}  // fin méthodes d'entités

//-X///////////////X/////////////////////////////X////////////////////////////// 
// classes des Méthodes de PROCESS
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Les m\'ethodes de processus
/L'objet dont la méthode est un attribut fonctionnel est une instance indirecte de[C]Process[/C].\\

De façon générale :
\begin{itemize}
\item Les constructeurs de base des sous-classes de [C]ProcessMethod[/C] affectent un symbole et un nom de classe, ainsi qu'un type, précisés dans la documentation de chaque sous-classe.
\item Les constructeurs par copie des sous-classes de [C]ProcessMethod[/C] copient le corps, et font appel au constructeur par copie de la classe [C]ProcessMethod[/C].
\end{itemize}
*/
//@{

/**@name Classe de base des m\'ethodes de processus */
//@{
          /// 
class ProcessMethod : public Method
{
public:

          /// Constructeur de base. Affecte le type O\_PROCESS. 
  ProcessMethod();  // _no swig_
          /// Constructeur par copie. 
  ProcessMethod(const ProcessMethod& source);  // _no swig_
 
          /// \index{méthodes!DescribedProcess - ProcessMethod}
void DescribedProcess(Process* pObj);  // _no swig_
          /// \index{méthodes!DescribedProcess - ProcessMethod}
Process* DescribedProcess();
          /// \index{méthodes!DescribedContinuousProcess - ProcessMethod}
ContinuousProcess* DescribedContinuousProcess();
// lancer une exception si DP
          /// \index{méthodes!DescribedDiscretProcess - ProcessMethod}
DiscreteProcess*   DescribedDiscreteProcess();
// lancer une exception si CP

// INQUIRY

protected:
   Process*           mpDescribedObject;
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de processus sans valeur renvoy\'ee */
//@{
          /// 
class VoidProcessMethod : public ProcessMethod  // _no swig_
{
public:
          // Constructeur de base. 
  VoidProcessMethod();
          /// Constructeur de base. Affecte le type de méthode VOID. 
  VoidProcessMethod(UserMethodId, char*);
          /// Constructeur par copie.
  VoidProcessMethod(const VoidProcessMethod& source); 
         // Opérateur d'affectation. 
  const VoidProcessMethod&      operator=(const VoidProcessMethod&);

          /// \index{méthodes!AssignBody - VoidProcessMethod}
void           AssignBody(void (*f)(ProcessMethod*));
          /// \index{méthodes!Body - VoidProcessMethod}
 void          (*Body)(ProcessMethod*);
};  // _end class_ 

//@} // fin classe  

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de processus à valeur enti\`ere */
//@{
          /// 
class IntProcessMethod : public ProcessMethod  // _no swig_
{
public:
          // Constructeur de base. 
  IntProcessMethod();
          /// Constructeur de base. Affecte le type de méthode INT. 
  IntProcessMethod(UserMethodId, char*);
          /// Constructeur par copie.
  IntProcessMethod(const IntProcessMethod& source); 
         // Opérateur d'affectation. 
  const IntProcessMethod&      operator=(const IntProcessMethod&);

          /// \index{méthodes!AssignBody - IntProcessMethod}
void           AssignBody(int (*f)(ProcessMethod*));
          /// \index{méthodes!Body - IntProcessMethod}
 int           (*Body)(ProcessMethod*);
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de processus à valeur flottante simple précision */
//@{
          /// 
class FloatProcessMethod : public ProcessMethod  // _no swig_
{
public:
          // Constructeur de base. 
  FloatProcessMethod();
          /// Constructeur de base. Affecte le type de méthode INT. 
  FloatProcessMethod(UserMethodId, char*);
          /// Constructeur par copie.
  FloatProcessMethod(const FloatProcessMethod& source); 
         // Opérateur d'affectation. 
  const FloatProcessMethod&      operator=(const FloatProcessMethod&);

          /// \index{méthodes!AssignBody - FloatProcessMethod}
void           AssignBody(float (*f)(ProcessMethod*));
          /// \index{méthodes!Body - FloatProcessMethod}
 float           (*Body)(ProcessMethod*);
}; // _end class_ 

//@} // fin classe  

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de processus à valeur flottante double précision */
//@{
          /// 
class DoubleProcessMethod : public ProcessMethod  // _no swig_
{
public:
          // Constructeur de base. 
  DoubleProcessMethod();
          /// Constructeur de base. Affecte le type de méthode INT. 
  DoubleProcessMethod(UserMethodId, char*);
          /// Constructeur par copie.
  DoubleProcessMethod(const DoubleProcessMethod& source); 
         // Opérateur d'affectation. 
  const DoubleProcessMethod&      operator=(const DoubleProcessMethod&);

          /// \index{méthodes!AssignBody - DoubleProcessMethod}
void           AssignBody(double (*f)(ProcessMethod*));
          /// \index{méthodes!Body - DoubleProcessMethod}
 double           (*Body)(ProcessMethod*);
}; // _end class_ 

//@} // fin classe   

//@}  // fin méthodes de processus


//-X///////////////X/////////////////////////////X////////////////////////////// 
// classes des Méthodes d'EVENEMENTS
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Les m\'ethodes d'\'ev\'enements
L'objet dont la méthode est un attribut fonctionnel est une instance d'[C]Event[/C]. De façon générale :
\begin{itemize}
\item Les constructeurs de base des sous-classes d'[C]EventMethod[/C] affectent un symbole et un nom de classe, ainsi qu'un type, précisés dans la documentation de chaque sous-classe.
\item Les constructeurs par copie des sous-classes d'[C]EventMethod[/C] copient le corps, et font appel au constructeur par copie de la classe [C]EventMethod[/C].
\end{itemize}
*/
//@{

/**@name Classe de base des m\'ethodes d'\'ev\'enements */
//@{
          ///
class EventMethod : public Method
{
public:
          /// Constructeur de base. Affecte le type O\_EVENT.
  EventMethod();  // _no swig_
          /// Constructeur par copie. 
  EventMethod(const EventMethod& source);  // _no swig_ 

          /// \index{méthodes!DescribedEvent - EventMethod}
void DescribedEvent(Event* pObj);  // _no swig_
          ///  
Event* DescribedEvent();

protected:
   Event*           mpDescribedObject;
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'\'ev\'enements sans valeur renvoy\'ee */
//@{
          /// 
class VoidEventMethod : public EventMethod  // _no swig_
{
public:
          // Constructeur de base. 
  VoidEventMethod();
          /// Constructeur de base. Affecte le type de méthode VOID. 
  VoidEventMethod(UserMethodId, char*);
          /// Constructeur par copie.
  VoidEventMethod(const VoidEventMethod& source);
         // Opérateur d'affectation. 
  const VoidEventMethod&      operator=(const VoidEventMethod&);

          /// \index{méthodes!AssignBody - VoidEventMethod}
void           AssignBody(void (*f)(EventMethod*));
          /// \index{méthodes!Body - VoidEventMethod}
 void          (*Body)(EventMethod*);
};    // _end class_ 

//@} // fin classe

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'\'ev\'enements à valeur boole\'enne */
//@{
          /// 
class EventPredicate : public EventMethod  // _no swig_
{
public:
          // Constructeur de base. 
  EventPredicate();
          /// Constructeur de base. Affecte le type de méthode BOOLEAN. 
  EventPredicate(UserMethodId, char*);
          /// Constructeur par copie.
  EventPredicate(const EventPredicate& source);
         // Opérateur d'affectation. 
  const EventPredicate&      operator=(const EventPredicate&);

          /// \index{méthodes!AssignBody - IntEventMethod}
void           AssignBody(bool (*f)(EventMethod*));
          /// \index{méthodes!Body - EventPredicate}
 bool       (*Body)(EventMethod*);
}; // _end class_ 

//@} // fin classe   

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes d'\'ev\'enements à valeur enti\`ere */
//@{
          /// 
class IntEventMethod : public EventMethod  // _no swig_
{
public:
          // Constructeur de base. 
  IntEventMethod();
          /// Constructeur de base. Affecte le type de méthode INT. 
  IntEventMethod(UserMethodId, char*);
          /// Constructeur par copie.
  IntEventMethod(const IntEventMethod& source);
         // Opérateur d'affectation. 
  const IntEventMethod&      operator=(const IntEventMethod&);

          /// \index{méthodes!AssignBody - IntEventMethod}
void           AssignBody(int (*f)(EventMethod*));
          /// \index{méthodes!Body - IntEventMethod}
 int           (*Body)(EventMethod*);
}; // _end class_ 

//@} // fin classe

//@}  // fin méthodes d'événements


//-X///////////////X/////////////////////////////X////////////////////////////// 
// classes des Méthodes de DEMONS
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Les m\'ethodes de d\'emons (moniteurs)
L'objet dont la méthode est un attribut fonctionnel est une instance de [C]DescValueMonitor[/C] ou de [C]StructureMonitor[/C]. De façon générale :
\begin{itemize}
\item Les constructeurs de base des sous-classes de [C]MonitorMethod[/C] affectent un symbole et un nom de classe, ainsi qu'un type, précisés dans la documentation de chaque sous-classe.
\item Les constructeurs par copie des sous-classes de [C]MonitorMethod[/C] copient le corps, et font appel au constructeur par copie de la classe [C]MonitorMethod[/C].
\end{itemize}
*/
//@{

/**@name Classe de base des m\'ethodes de d\'emons */
//@{
          /// 
class MonitorMethod : public Method
{
public:

          /// Constructeur de base. Affecte le type O\_DV\_MONITOR.
  MonitorMethod();  // _no swig_
          /// Constructeur par copie. 
  MonitorMethod(const MonitorMethod& source);  // _no swig_ 

          /// \index{méthodes!DescribedMonitor - MonitorMethod}
void DescribedMonitor(DescValueMonitor* pObj);  // _no swig_
          /// 
void DescribedMonitor(StructureMonitor* pObj);  // _no swig_
          ///  
DescValueMonitor* DescribedDescValueMonitor();
          /// 
StructureMonitor* DescribedStructureMonitor();

protected:
   DescValueMonitor*           mpDescribedDVM;
   StructureMonitor*           mpDescribedSM;
}; // _end class_ 

//@} // fin classe   

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons sans valeur renvoy\'ee */
//@{
          /// 
class VoidMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  VoidMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode VOID. 
  VoidMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  VoidMonitorMethod(const VoidMonitorMethod& source);
         // Opérateur d'affectation. 
  const VoidMonitorMethod&      operator=(const VoidMonitorMethod&);

          /// \index{méthodes!AssignBody - VoidMonitorMethod}
void           AssignBody(void (*f)(MonitorMethod*));
          /// \index{méthodes!Body - VoidMonitorMethod}
 void          (*Body)(MonitorMethod*);
}; // _end class_ 

//@} // fin classe   

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name Méthodes de démons à valeur entière */
//@{
          /// 
class IntMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  IntMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode INT. 
  IntMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  IntMonitorMethod(const IntMonitorMethod& source);
         // Opérateur d'affectation. 
  const IntMonitorMethod&      operator=(const IntMonitorMethod&);

          /// \index{méthodes!AssignBody - IntMonitorMethod}
void           AssignBody(int (*f)(MonitorMethod*));
          /// \index{méthodes!Body - IntMonitorMethod}
 int           (*Body)(MonitorMethod*);
}; // _end class_ 

//@} // fin classe   

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons à valeur flottante */
//@{
          /// 
class FloatMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  FloatMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode FLOAT. 
  FloatMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  FloatMonitorMethod(const FloatMonitorMethod& source);
         // Opérateur d'affectation. 
   const FloatMonitorMethod&      operator=(const FloatMonitorMethod&);

          /// \index{méthodes!AssignBody - FloatMonitorMethod}
void           AssignBody(float (*f)(MonitorMethod*));
          /// \index{méthodes!Body - FloatMonitorMethod}
 float         (*Body)(MonitorMethod*);
}; // _end class_ 

//@} // fin classe  

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons à valeur flottante en double précision */
//@{
          /// 
class DoubleMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  DoubleMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode DOUBLE. 
  DoubleMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  DoubleMonitorMethod(const DoubleMonitorMethod& source);
         // Opérateur d'affectation. 
   const DoubleMonitorMethod&      operator=(const DoubleMonitorMethod&);

          /// \index{méthodes!AssignBody - DoubleMonitorMethod}
void           AssignBody(double (*f)(MonitorMethod*));
          /// \index{méthodes!Body - DoubleMonitorMethod}
 double        (*Body)(MonitorMethod*);
};  // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons à valeur 'pointeur sur une entit\'e' */
//@{
          /// 
class EntityMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  EntityMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode P\_ENTITY. 
  EntityMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  EntityMonitorMethod(const EntityMonitorMethod& source);
         // Opérateur d'affectation. 
   const EntityMonitorMethod&      operator=(const EntityMonitorMethod&);

          /// \index{méthodes!AssignBody - EntityMonitorMethod}
void           AssignBody(BasicEntity* (*f)(MonitorMethod*));
          /// \index{méthodes!Body - EntityMonitorMethod}
 BasicEntity*       (*Body)(MonitorMethod*);
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons à valeur 'chaine de caract\`eres' */
//@{
          /// 
class StringMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  StringMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode STRING. 
  StringMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  StringMonitorMethod(const StringMonitorMethod& source);
         // Opérateur d'affectation. 
   const StringMonitorMethod&      operator=(const StringMonitorMethod&);

          /// \index{méthodes!AssignBody - StringMonitorMethod}
void           AssignBody(char* (*f)(MonitorMethod*));
          /// \index{méthodes!Body - StringMonitorMethod}
 char*         (*Body)(MonitorMethod*);
}; // _end class_ 

//@} // fin classe
 
//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons à valeur 'tableau de pointeurs sur des entit\'es' */
//@{
          ///
class EntityTabMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  EntityTabMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode ENTITY\_TAB. 
  EntityTabMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  EntityTabMonitorMethod(const EntityTabMonitorMethod& source);
         // Opérateur d'affectation. 
   const EntityTabMonitorMethod&      operator=(const EntityTabMonitorMethod&);

          /// \index{méthodes!AssignBody - EntityTabMonitorMethod}
void           AssignBody(pEntityTab* (*f)(MonitorMethod*));
          /// \index{méthodes!Body - EntityTabMonitorMethod}
pEntityTab*     (*Body)(MonitorMethod*);
};  // _end class_ 

//@} // fin classe
 
//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons à valeur 'sp\'ecification de domaine de valeur' */
//@{
          /// 
class DescValueSpecMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  DescValueSpecMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode P\_DESCVALUESPEC. 
  DescValueSpecMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  DescValueSpecMonitorMethod(const DescValueSpecMonitorMethod& source);
         // Opérateur d'affectation. 
   const DescValueSpecMonitorMethod&      operator=(const DescValueSpecMonitorMethod&);

          /// \index{méthodes!AssignBody - DescValueSpecMonitorMethod}
void           AssignBody(DescValueSpec* (*f)(MonitorMethod*));
          /// \index{méthodes!Body - DescValueSpecMonitorMethod}
DescValueSpec*     (*Body)(MonitorMethod*);
};  // _end class_ 

//@} // fin classe
 
//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de d\'emons à valeur 'tableau de sp\'ecifications de domaine de valeur' */
//@{
          /// 
class DescValueSpecTabMonitorMethod : public MonitorMethod  // _no swig_
{
public:
          // Constructeur de base. 
  DescValueSpecTabMonitorMethod();
          /// Constructeur de base. Affecte le type de méthode DESCVALUESPEC\_TAB. 
  DescValueSpecTabMonitorMethod(UserMethodId, char*);
          /// Constructeur par copie.
  DescValueSpecTabMonitorMethod(const DescValueSpecTabMonitorMethod& source);
         // Opérateur d'affectation. 
   const DescValueSpecTabMonitorMethod&      operator=(const DescValueSpecTabMonitorMethod&);

          /// \index{méthodes!AssignBody - DescValueSpecTabMonitorMethod}
void           AssignBody(pDescValueSpecTab* (*f)(MonitorMethod*));
          /// \index{méthodes!Body - DescValueSpecTabMonitorMethod}
pDescValueSpecTab*     (*Body)(MonitorMethod*);
};  // _end class_ 

//@} // fin classe

//@}  // fin méthodes de moniteurs


//-X///////////////X/////////////////////////////X////////////////////////////// 
// classes des Méthodes d'ENTITY_SPEC
//
//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Les m\'ethodes de sp\'ecifications d'ensembles d'entit\'es
L'objet dont la méthode est un attribut fonctionnel est une instance d'[C]EntitySpec[/C]. De façon générale :
\begin{itemize}
\item Les constructeurs de base des sous-classes d'[C]EntitySpecMethod[/C] affectent un symbole et un nom de classe, ainsi qu'un type, précisés dans la documentation de chaque sous-classe.
\item Les constructeurs par copie des sous-classes d'[C]EntitySpecMethod[/C] copient le corps, et font appel au constructeur par copie de la classe [C]EntitySpecMethod[/C].
\end{itemize}
*/
//@{

/**@name Classe de base des m\'ethodes de sp\'ecifications d'ensembles d'entit\'es */
//@{
          /// 
class EntitySpecMethod : public Method
{
public:

          /// Constructeur de base. Affecte le type O\_ENTITYSPEC.
  EntitySpecMethod();  // _no swig_
          /// Constructeur par copie. 
  EntitySpecMethod(const EntitySpecMethod& source);   // _no swig_

          /// \index{méthodes!DescribedEntitySpec - EntitySpecMethod}
void DescribedEntitySpec(EntitySpec* pObj);  // _no swig_
          ///  
EntitySpec* DescribedEntitySpec();

protected:
   EntitySpec*           mpDescribedObject;
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de sp\'ecifications d'ensembles d'entit\'es à valeur 'spécification d'ensemble d'entit\'es' */
//@{
          /// 
class EntitySpecPredicate : public EntitySpecMethod  // _no swig_
{
public:
          // Constructeur de base. 
  EntitySpecPredicate();
          /// Constructeur de base. Affecte le type de méthode BOOLEAN. 
  EntitySpecPredicate(UserMethodId, char*);
          /// Constructeur par copie.
  EntitySpecPredicate(const EntitySpecPredicate& source);
         // Opérateur d'affectation. 
  const EntitySpecPredicate&      operator=(const EntitySpecPredicate&);

          /// \index{méthodes!AssignBody - IntEntitySpecMethod}
void           AssignBody(bool (*f)(EntitySpecMethod*));
          /// \index{méthodes!Body - EntitySpecPredicate}
 bool       (*Body)(EntitySpecMethod*);
};  // _end class_ 

//@} // fin classe  

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de sp\'ecifications d'ensembles d'entit\'es à valeur enti\`ere */
//@{
          /// 
class IntEntitySpecMethod : public EntitySpecMethod  // _no swig_
{
public:
          /// Constructeur de base. 
  IntEntitySpecMethod();
          /// Constructeur de base. Affecte le type de méthode INT. 
  IntEntitySpecMethod(UserMethodId, char*);
          /// Constructeur par copie.
  IntEntitySpecMethod(const IntEntitySpecMethod& source);
         // Opérateur d'affectation. 
  const IntEntitySpecMethod&      operator=(const IntEntitySpecMethod&);

          /// \index{méthodes!AssignBody - IntEntitySpecMethod}
void           AssignBody(int (*f)(EntitySpecMethod*));
          /// \index{méthodes!Body - IntEntitySpecMethod}
 int           (*Body)(EntitySpecMethod*);
}; // _end class_ 

//@} // fin classe 

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de sp\'ecifications d'ensembles d'entit\'es à valeur 'pointeur sur une entit\'e' */
//@{
          /// 
class EntityEntitySpecMethod : public EntitySpecMethod  // _no swig_
{
public:
          /// Constructeur de base. 
  EntityEntitySpecMethod();
          /// Constructeur de base. Affecte le type de méthode P\_ENTITY. 
  EntityEntitySpecMethod(UserMethodId, char*);
          /// Constructeur par copie.
  EntityEntitySpecMethod(const EntityEntitySpecMethod& source);
         // Opérateur d'affectation. 
   const EntityEntitySpecMethod&      operator=(const EntityEntitySpecMethod&);

          /// \index{méthodes!AssignBody - EntityEntitySpecMethod}
void           AssignBody(BasicEntity* (*f)(EntitySpecMethod*));
          /// \index{méthodes!Body - EntityEntitySpecMethod}
 BasicEntity*       (*Body)(EntitySpecMethod*);
};  // _end class_ 

//@} // fin classe

//-X///////////////X/////////////////////////////X///////////////////////////

/**@name M\'ethodes de sp\'ecifications d'ensembles d'entit\'es à valeur 'tableau de pointeurs sur des entit\'es' */
//@{
          /// 
class EntityTabEntitySpecMethod : public EntitySpecMethod  // _no swig_
{
public:
          /// Constructeur de base. 
  EntityTabEntitySpecMethod();
          /// Constructeur de base. Affecte le type de méthode ENTITY\_TAB. 
  EntityTabEntitySpecMethod(UserMethodId, char*);
          /// Constructeur par copie.
  EntityTabEntitySpecMethod(const EntityTabEntitySpecMethod& source);
         // Opérateur d'affectation. 
   const EntityTabEntitySpecMethod&      operator=(const EntityTabEntitySpecMethod&);

          /// \index{méthodes!AssignBody - EntityTabEntitySpecMethod}
void           AssignBody(pEntityTab* (*f)(EntitySpecMethod*));
          /// \index{méthodes!Body - EntityTabEntitySpecMethod}
pEntityTab*     (*Body)(EntitySpecMethod*);
}; // _end class_ 

//@} // fin classe 

//@}  // fin méthodes d'EntitySpec

//@}  // fin classe de base + par types d'objet

#endif  // _METHOD_HH
