//============================================================================
// FILE: VariableDescriptor.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des descripteurs variables
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _VARIABLEDESCRIPTOR_HH
#define _VARIABLEDESCRIPTOR_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "Descriptor.h"

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name CDPFORTEXLes descripteurs variables
\begin{itemize} \item Directive d'inclusion :
[P]
\#include VariableDescriptor.h
[/P]
\end{itemize}

Un descripteur d'entité est déclaré variable, c'est-à-dire installé comme une instance de la présente classe, lorsque sa valeur est censée évoluer au cours de la simulation. Sont activés les services tels que ceux rendus par la classe [C]DescValueMonitor[/C], ceux attachés à l'utilisation interactive du simulateur (notamment l'affichage de la dynamique des changements de valeur) et enfin la possibilité de sauvegarder sur fichier externe la chronique des changements de valeurs..\\

Lors de la création d'un descripteur variable, les mêmes valeurs par défaut (celles qui seront renvoyées par [Y]GetDefaultValue[/Y]) que pour les descripteurs constants sont attribuées aux descripteurs variables (voir le tableau ci-dessous). Dans les constructeurs propres à son application, l'utilisateur peut définir et affecter une nouvelle valeur par défaut avec les méthodes [Y]SetDefaultValue[/Y] et [Y]SetToDefaultValue[/Y]. Aucune valeur (celle renvoyée par [Y]Get..Value[/Y]) n'est attribuée automatiquement au descripteur.

Noter que dans la présente version de \D, les types 'tableau numérique' ({\em pIntTab*}, {\em pFloatTab*} {\em pDoubleTab*}) ne sont pas supportés :  

\begin{tabular}{cccl}
{\tiny classe} & {\tiny valeur par défaut} & {\tiny valeur} & {\tiny type}\\
\hline
{\tiny [C]IntVariableDescriptor[/C]} & {\tiny 0} & {\tiny 0} & {\tiny valeur entière ({\em int})} \\
{\tiny [C]FloatVariableDescriptor[/C]} & {\tiny 0.0} & {\tiny 0.0} & {\tiny valeur flottante ({\em float})} \\
{\tiny [C]DoubleVariableDescriptor[/C]} & {\tiny 0.0} & {\tiny 0.0} & {\tiny valeur flottante double précision ({\em double})} \\
{\tiny [C]StringVariableDescriptor[/C]} & {\tiny ""} & {\tiny ""} & {\tiny une chaÃ®ne de caractères ({\em char*})} \\
{\tiny [C]EntityVariableDescriptor[/C]} & {\tiny NULL} & {\tiny NULL} & {\tiny {\em BasicEntity*}} \\
{\tiny [C]EntitySpecVariableDescriptor[/C]} & {\tiny NULL} & {\tiny NULL} & {\tiny {\em EntitySpec*}} \\
{\tiny [C]StringTabVariableDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em TableauString*}} \\
{\tiny [C]EntityTabVariableDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pEntityTab*}} \\
{\tiny [C]DescValueSpecTabVariableDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny NULL} & {\tiny {\em pDescValueSpecTab*}} \\
{\tiny [C]IntDomainVariableDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pIntIntervalTab*}} \\
{\tiny [C]FloatDomainVariableDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pFloatIntervalTab*}} \\
{\tiny [C]DoubleDomainVariableDescriptor[/C]} & {\tiny un tableau de taille nulle} & {\tiny un tableau de taille nulle} & {\tiny {\em pDoubleIntervalTab*}} \\
\end{tabular}
*/
//@{ // classe de base + ss_classes typées


/**@name La classe de base des descripteurs variables
\\

Les services de manipulation de la valeur, entre autres, sont détenus par les sous-classes typées ([C]IntVariableDescriptor[/C], etc.).\\

Par contre, c'est dans cette classe que sont fournis les services de manipulation des démons (ou 'moniteurs', instances de la classe [C]DescValueMonitor[/C]).\\
La manière de programmer l'installation d'un tel démon sur un descripteur variable est présentée à la page 'Les démons'.\\

On rappelle ici que l'action d'un démon placé sur un descripteur variable peut être débrayée par l'adjonction d'un argument complémentaire, de type {\em Boolean} et à valeur {\em faux}, aux appels [Y]SetValue[/Y] et [Y]Get..Value[/Y] (voir la documentation de la classe [C]DescriptedEntity[/C], page 'Affectation et accès aux valeurs des descripteurs').
 */
//@{
           /// 
class VariableDescriptor : public Descriptor 
{
public:

// LIFECYCLE
          // Constructeur de base. N'est normalement appelé que par les sous-classes prédéfinies, et non dans les classes définies par l'utilisateur.
   VariableDescriptor();   // _no swig_
          // Destructeur (pour le tableau date/valeur)
   ~VariableDescriptor();   // _no swig_

// OPERATORS
         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const VariableDescriptor&) const {return TRUE;};
  Boolean         operator > (const VariableDescriptor&) const {return TRUE;};

// OPERATIONS 
         //
   void      ValueToString(char str[50]);

/**@name [SC]Manipulation des démons (ou moniteurs)[/SC] */
//@{
          /// Renvoie le démon attaché à la valeur de ce descripteur. \index{méthodes!GetMonitor - Var.Descriptor}
   DescValueMonitor* GetMonitor() const;   // _no swig_
          /// Attache le démon passé en argument à la valeur de ce descripteur. Utilisé par la méthode [Y]AttachToDescriptor[/Y] de [C]DescValueMonitor[/C]. Lorsqu'un démon est attaché à un descripteur, une bonne pratique est de lui spécifier une valeur par défaut afin d'éviter, lors de la première affectation de valeur, un accès à une mémoire non encore allouée.\index{méthodes!SetMonitor - Var.Descriptor}
   void              SetMonitor(DescValueMonitor*);    
          /// Renvoie TRUE si un démon est attaché à la valeur de ce descripteur.
   Boolean           HasMonitor() const;   
          // Détermine si un démon est attaché (TRUE) ou non (FALSE) à la valeur de ce descripteur.
   void              HasMonitor(Boolean);   // _no swig_
//@}                            

  // début code gestion OutputSpec
          // Détermine si les changements de valeurs du descripteur doivent être envoyés à l'interface pour affichage en temps réel.
   void              IsPlotted(Boolean);   // _no swig_   
          // Renvoie TRUE s'il a été spécifié que les changements de valeurs du descripteur devaient être envoyés à l'interface pour affichage en temps réel. \index{méthodes!IsPlotted - VariableDescriptor}
   Boolean           IsPlotted() const;  // _no swig_  
          // Etablit le nombre de décimales à garder dans le cas d'un affichage en temps réel par l'interface utilisateur. La valeur par défaut est 0, et elle n'est utilisée que s'il s'agit d'une variable à valeur entière ou flottante. \index{méthodes!SetPrecisionForPlotting - VariableDescriptor}
   void              SetPrecisionForPlotting(int);  // _no swig_  
          // Renvoie le nombre de décimales à garder dans le cas d'un affichage en temps réel par l'interface utilisateur. \index{méthodes!GetPrecisionForPlotting - VariableDescriptor}
   int               GetPrecisionForPlotting() const;  // _no swig_
          // Détermine le facteur d'échantillonnage {\em n}, dont la valeur par défaut est 1. Un appel à [Y]Set..Value[/Y] chaque {\em n} appels occasionne un enregistrement au titre des spécifications de sortie, et un message envoyé à l'interface utilisateur. \index{méthodes!SetSamplingStep - VariableDescriptor}
  void               SetSamplingStep(int val);   // _no swig_ 
          // Renvoie le facteur d'échantillonnage. \index{méthodes!GetSamplingStep - VariableDescriptor}
  int                GetSamplingStep();   // _no swig_ 
   // fin code gestion OutputSpec

// INQUIRY

protected:
  
   DescValueMonitor*    mpDescValueMonitorAttached;
   Boolean              mHasDescValueMonitorAttached;
       
   // début code gestion OutputSpec
   Boolean              mIsPlotted;
   int                  mPrecisionForPlotting;

   int                  mSamplingStep; // + JPR 000921 pour IHM
   int                  mNbSetValue;   // + JPR 000921 pour IHM
   // fincode gestion OutputSpec

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Sous-classes typ\'ees de descripteurs variables. 
Toutes les sous-classes typées de descripteurs variables possèdent les mêmes services, qui ne se différencient que par le type de la valeur traitée. On les définit de façon commune de la manière suivante :\\

\begin{itemize}
\item Les constructeurs tels que [Y]IntVariableDescriptor()[/Y] construisent le descripteur en lui affectant un type (e.g. INT) et une catégorie (e.g. VARIABLE) pour la sous-classe [C]IntVariableDescriptor[/C].
\item Les constructeurs tels que [Y]IntVariableDescriptor(UserDescriptorId)[/Y] construisent le descripteur en lui affectant un symbole de classe. 
\item Les constructeurs tels que [Y]IntVariableDescriptor(UserDescriptorId, char*)[/Y] construisent le descripteur en lui affectant un symbole et d'un nom de classe. Le nom est affiché lors de la trace de la création (voir page 'Fonctions globales/RÃŽle de ParseMainArguments'). 
\item Les constructeurs tels que [Y]IntVariableDescriptor(const IntVariableDescriptor\&)[/Y] sont les constructeurs par copie. L'entité décrite n'est pas copiée dans ce constructeur. La copie de l'entité décrite et son attachement à la copie du descripteur doivent être gérés par ailleurs (voir le constructeur par copie de [C]Entity[/C]). Si un démon est attaché au descripteur copié, c'est lui-même (et non une copie du démon) qui est attaché à la copie du descripteur. {\em idem} pour les autres types.
\item Les destructeurs tels que [Y]~IntVariableDescriptor[/Y] détruisent le domaine de valeurs admissibles éventuellement affecté. Le démon éventuellement attaché n'est pas détruit.
\item Les opérateurs tels que [Y]operator = (const IntVariableDescriptor\&)[/Y] sont les opérateurs d'affectation. La valeur, et seulement elle, du descripteur à droite du signe = est affectée au descripteur à gauche du signe =.

\item void [Y]SetDomain(DescIntValueSpec*)[/Y] dote le descripteur d'un domaine de valeurs entières admissibles, celui passé en argument. {\em idem} pour les autres types. \index{méthodes!SetDomain - Var.Descriptor}
\item void [Y]SetSubDomain(int)[/Y] dote le descripteur d'un domaine vide de valeurs admissibles, puis lui ajoute une valeur (ici entière). {\em idem} pour les autres types. \index{méthodes!SetSubDomain - Var.Descriptor}
\item void [Y]SetSubDomain(int, int)[/Y] dote le descripteur d'un domaine vide de valeurs admissibles, puis lui ajoute un intervalle de valeurs (ici une suite de nombres entiers). {\em idem} pour les autres types. \index{méthodes!SetSubDomain - Var.Descriptor}
\item void [Y]AddSubDomain(int)[/Y] ajoute une valeur (ici entière) au domaine de valeurs admissibles. {\em idem} pour les autres types. Les valeurs numériques doivent être ajoutées en respectant l'ordre croissant. \index{méthodes!AddSubDomain - Var.Descriptor}
\item void [Y]AddSubDomain(float, float)[/Y] ajoute un intervalle de valeurs (ici flottantes) au domaine de valeurs admissibles. Si les intervalles sont ajoutés sans nécessairement respecter l'ordre croissant de leurs bornes inférieures, il faut utiliser la méthode [Y]SortOnLowBound[/Y] des sous-classes de [C]DescValueSpec[/C]. {\em idem} pour les autres types numériques.
\item [Y]GetIntDomain()[/Y] renvoie un pointeur sur l'instance de [C]DescIntValueSpec[/C] qui représente le domaine de valeurs admissibles. {\em idem} pour les autres types. \index{méthodes!Get..Domain - Var.Descriptor}
\item bool [Y]IsInDomain(int)[/Y] renvoie {\em vrai} si et seulement si un domaine de valeurs admissibles a été spécifié et que la valeur passée en argument est dans ce domaine. \index{méthodes!IsInDomain -Var.Descriptor}
\item bool [Y]IsOutOfDomain(int)[/Y] renvoie {\em vrai} si et seulement si un domaine de valeurs admissibles a été spécifié et que la valeur passée en argument n'est pas dans ce domaine. \index{méthodes!IsOutOfDomain -Var.Descriptor}

\item int [Y]GetIntValue()[/Y] renvoie la valeur entière du descripteur. {\em idem} pour les autres types. \index{méthodes!Get..Value - Var.Descriptor}
\item int [Y]GetDefaultValue()[/Y] renvoie la valeur entière par défaut du descripteur. {\em idem} pour les autres types. \index{méthodes!GetDefaultValue - Var.Descriptor}

\item void [Y]SetValue(int)[/Y] affecte une valeur entière au descripteur. Lève l'exception [Y]OutOfIntDomain[/Y] si la valeur n'est pas admissible. {\em idem} pour les autres types. \index{méthodes!SetValue - Var.Descriptor}
\item void [Y]SetValue(char*)[/Y] fait une copie de la chaîne passée en argument, puis l'affecte au descripteur.
\item void [Y]SetDefaultValue(int)[/Y] établit que la valeur entière donnée en argument est la valeur par défaut du descripteur. {\em idem} pour les autres types. \index{méthodes!SetDefaultValue - Var.Descriptor}
\item void [Y]SetToDefaultValue()[/Y] établit que la valeur est la valeur qui a été attribuée par défaut. \index{méthodes!SetToDefaultValue - Var.Descriptor}
\item void [Y]SetToDefaultValue(int)[/Y] établit que la valeur entière donnée en argument est la valeur par défaut du descripteur, et l'affecte à la valeur courante du descripteur. {\em idem} pour les autres types. \index{méthodes!SetToDefaultValue({\em valeur}) - Var.Descriptor}
\item Pour le type de valeurs entières seulement, la méthode [Y]IncrValue[/Y] augmente de 1 et [Y]DecrValue[/Y] diminue de 1 la valeur du descripteur, et la méthode [Y]IncreaseValue[/Y] augmente et [Y]DecreaseValue[/Y] diminue d'une valeur entière la valeur du descripteur. Lèvent l'exception [Y]OutOfIntDomain[/Y] si la valeur n'est pas admissible. \index{méthodes!IncrValue - Var.Descriptor}\index{méthodes!DecrValue - Var.Descriptor}\index{méthodes!IncreaseValue - Var.Descriptor}\index{méthodes!DecreaseValue - Var.Descriptor} 
\end{itemize}
Pour tous les groupes de méthodes, si un démon est attaché au descripteur, il sera déclenché, sauf si un argument de type {\em Boolean} est fourni et que sa valeur est FALSE.
*/
//@{       // ss_classes typées

//-X///////////////X/////////////////////////////X////////////////////////////


/**@name Descripteurs constants à valeur numérique monidimensionnelle 
Par rapport aux autres, ces descripteurs sont simplement caractérisés par une unité.*/
//@{
          ///
class MonoDimensionalNumericalVariableDescriptor : public VariableDescriptor
{
public:

// @Man: LIFECYCLE
// @{
          // Constructeur de base. N'est normalement appelé que par les sous-classes prédéfinies, et non dans les classes définies par l'utilisateur.
   MonoDimensionalNumericalVariableDescriptor();  // _no swig_    
          // Destructeur.
   ~MonoDimensionalNumericalVariableDescriptor();   // _no swig_  
// @}

// @Man: OPERATORS
// @{
         // Opération  d'affectation. Cette opération n'est en fait pas gérée par DIESE : l'exception [Y]UnAuthorizedAssignement[/Y] est levée.

   const MonoDimensionalNumericalVariableDescriptor&     operator=(const MonoDimensionalNumericalVariableDescriptor&);  // assignment operator
// @}

// OPERATIONS
                  

/**@name ACCESS */
//@{
         /// Affecte une unité au descripteur qui reçoit ce message.\index{méthodes!ClassUnity - MonoDimensionalNumericalVariableDescriptor}
   void              ClassUnity(char*);  // _no swig_
         /// Renvoie le unityaire associé au récepteur.
   char*             ClassUnity() const;  // _no swig_
//@}

protected:
   char*                 mDescriptorUnity;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name Descripteurs variables à valeur entière */
//@{
           /// 
class IntVariableDescriptor : public MonoDimensionalNumericalVariableDescriptor
{
public:

/**@name LIFECYCLE */
//@{
         /// Affecte le type INT et la catégorie VARIABLE.
   IntVariableDescriptor();   // _no swig_
         /// 
   IntVariableDescriptor(UserDescriptorId);   // _no swig_  
         /// 
   IntVariableDescriptor(UserDescriptorId, char*);   // _no swig_      
         /// 
   IntVariableDescriptor(const IntVariableDescriptor&);    // _no swig_   
         ///
   ~IntVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   IntVariableDescriptor&      operator=(const IntVariableDescriptor&); 
//@}

/**@name [SC]Opérations sur le domaine[/SC] */
//@{
          /// 
   void SetDomain(DescIntValueSpec*);   
          /// 
   void SetSubDomain(int);    // _no swig_    
          /// 
   void SetSubDomain(int, int);
          /// 
   void AddSubDomain(int);    // _no swig_    
          /// 
   void AddSubDomain(int, int); 
//@}                          

/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          // 
   void              RecordValue(int);
          ///        
   void              SetToDefaultValue();    // _no swig_    
          /// 
   void              SetDefaultValue(int); 
          /// 
   void              SetToDefaultValue(int);
          ///
   void              SetValue(int);    // _no swig_ 
          /// 
   void              SetValue(int, Boolean);
          /// 
   int               GetDefaultValue() const;
          /// 
   DescIntValueSpec* GetIntDomain() const;
          /// 
   bool              IsInDomain(int) const;
          /// 
   bool              IsOutOfDomain(int) const;   

          ///
   int               GetIntValue();    // _no swig_   
                     // plus const car passage de this à SetArgvalue
          /// 
   int               GetIntValue(Boolean);   

          /// 
   void              IncrValue();    // _no swig_
          /// 
   void              IncrValue(Boolean);   

          ///
   void              DecrValue();    // _no swig_
          /// 
   void              DecrValue(Boolean);   

          /// 
   void              IncreaseValue(int);    // _no swig_
          /// 
   void              IncreaseValue(int, Boolean);   

          ///
   void              DecreaseValue(int);    // _no swig_
          /// 
   void              DecreaseValue(int, Boolean);
         /// Affecte une fonction de conversion vers une chaîne de caractères. Ce service est utile lorsque les entiers valeurs de ce descripteur peuvent être associés à des noms symboliques, et quand on préfère afficher les noms symboliques que les valeurs entières (notamment dans les fichiers de sortie de la simulation).\index{méthodes!AssignConverse}
   void              AssignConverse(char* (*f)(int));  // _no swig_

         // Renvoie {\em vrai} si un corps a été assigné à la méthode de conversion en chaîne de caractères(par un appel à [Y]AssignConverse[/Y].\index{méthodes!HasConverseAssigned}
   bool              HasConverseAssigned();  // _no swig_
//@}

// INQUIRY

protected:

   int             mValue;
   int             mDefaultValue;
   DescIntValueSpec* mDVSDomain;

   char*           (*ConverseToString)(int); 
   bool            mHasConverseAssigned;  

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots à valeur flottante */
//@{
           /// 
class FloatVariableDescriptor : public MonoDimensionalNumericalVariableDescriptor 
{
public:

/**@name LIFECYCLE */
//@{
         /// Affecte le type FLOAT et la catégorie VARIABLE.
   FloatVariableDescriptor();   // _no swig_
         /// 
   FloatVariableDescriptor(UserDescriptorId);     // _no swig_   
         /// 
   FloatVariableDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// 
   FloatVariableDescriptor(const FloatVariableDescriptor&);    // _no swig_   
         ///
   ~FloatVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   FloatVariableDescriptor&      operator=(const FloatVariableDescriptor&); 
//@}

/**@name [SC]Opérations sur le domaine[/SC] */
//@{  
          /// 
   void SetDomain(DescFloatValueSpec*);   
          ///  
   void SetSubDomain(float, float);
          /// 
   void AddSubDomain(float, float); 
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          // 
   void              RecordValue(float);
          /// 
   void              SetToDefaultValue();     // _no swig_   
          /// 
   void              SetDefaultValue(float);  
          /// 
   void              SetToDefaultValue(float);
          /// 
   void              SetValue(float);    // _no swig_ 
          /// 
   void              SetValue(float, Boolean);
          /// 
   float             GetDefaultValue() const; 
          /// 
   DescFloatValueSpec* GetFloatDomain() const;
          /// 
   bool              IsInDomain(float) const; 
          /// 
   bool              IsOutOfDomain(float) const;   

          /// 
   float             GetFloatValue();    // _no swig_
          /// 
   float             GetFloatValue(Boolean);
//@}

// INQUIRY

protected:

   float           mValue;
   float           mDefaultValue;
   DescFloatValueSpec* mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots à valeur flottante en double précision */
//@{
           ///
class DoubleVariableDescriptor : public MonoDimensionalNumericalVariableDescriptor 
{
public:

/**@name LIFECYCLE */
//@{
         /// Affecte le type DOUBLE et la catégorie VARIABLE.
   DoubleVariableDescriptor();   // _no swig_
         /// 
   DoubleVariableDescriptor(UserDescriptorId);    // _no swig_   
         /// 
   DoubleVariableDescriptor(UserDescriptorId, char*);    // _no swig_   
         /// 
   DoubleVariableDescriptor(const DoubleVariableDescriptor&);   // _no swig_ 
         ///
   ~DoubleVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   DoubleVariableDescriptor&      operator=(const DoubleVariableDescriptor&);
//@} 

/**@name [SC]Opérations sur le domaine[/SC] */
//@{  
          /// 
   void SetDomain(DescDoubleValueSpec*);   
          ///  
   void SetSubDomain(double, double); 
          /// 
   void AddSubDomain(double, double); 
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          // 
   void              RecordValue(double);
          /// 
   void              SetToDefaultValue();    // _no swig_    
          /// 
   void              SetDefaultValue(double);  
          /// 
   void              SetToDefaultValue(double);
          /// 
   void              SetValue(double);    // _no swig_ 
          /// 
   void              SetValue(double, Boolean);
          /// 
   double            GetDefaultValue() const;
          /// 
   DescDoubleValueSpec* GetDoubleDomain() const;
          /// 
   bool              IsInDomain(double) const;
          /// 
   bool              IsOutOfDomain(double) const;    
          /// 
   double            GetDoubleValue();    // _no swig_
          /// 
   double            GetDoubleValue(Boolean);
//@}

// INQUIRY

protected:

   double           mValue;
   double           mDefaultValue;
   DescDoubleValueSpec* mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est une entité */
//@{
           ///
class EntityVariableDescriptor : public VariableDescriptor
{
public:
/**@name LIFECYCLE */
//@{
         /// Affecte le type P\_ENTITY et la catégorie VARIABLE.
   EntityVariableDescriptor();   // _no swig_
         /// 
   EntityVariableDescriptor(UserDescriptorId);   // _no swig_    
         /// 
   EntityVariableDescriptor(UserDescriptorId, char*);    // _no swig_   
         /// {\bf Important :} la valeur de la copie est la même que la valeur du descripteur copié (pointeurs sur la même entité).
   EntityVariableDescriptor(const EntityVariableDescriptor&);   // _no swig_  
         ///
   ~EntityVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   EntityVariableDescriptor&      operator=(const EntityVariableDescriptor&);
//@}

/**@name [SC]Opérations sur le domaine[/SC] */
//@{
          /// 
   void SetDomain(DescEntityValueSpec*);   
          ///  
   void SetSubDomain(BasicEntity*); 
          /// 
   void AddSubDomain(BasicEntity*);  
          /// Spécifie que la valeur du descripteur doit être une instance de la classe dont le symbole est passé en argument. Par défaut d'une telle spécification, la classe de valeur sera O\_BASIC\_ENTITY. \index{méthodes!ValueClassId - Var.Descriptor}        
   void ValueClassId(UserClassId id);

          /// Ajoute un symbole aux symboles des classes qui peuvent être celles des valeurs de l'entité. \index{méthodes!AddValueClassId - Var.Descriptor}
   void AddValueClassId(UserClassId id);
 
          /// Renvoie le symbole de la classe qui est la classe de valeur du descripteur. \index{méthodes!GetValueClassId - Var.Descriptor}
   UserClassId GetValueClassId();

          /// Renvoie la table des symboles des classes de valeur du descripteur.                 
   UserClassIdTab ValueClassIdTab();
//@}

/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          // 
   void              RecordValue(char*);  // + JPR 050805
          // 
   void              SetToDefaultValue();     // _no swig_   
          // 
   void              SetDefaultValue(BasicEntity*);  
          // 
   void              SetToDefaultValue(BasicEntity*);
          /// 
   void              SetValue(BasicEntity*);   // _no swig_  
          /// 
   void              SetValue(BasicEntity*, Boolean);
          // 
   BasicEntity*           GetDefaultValue() const;
          /// 
   DescEntityValueSpec* GetEntityDomain() const;
          /// 
   bool              IsInDomain(BasicEntity*) const;
          /// 
   bool              IsOutOfDomain(BasicEntity*) const;   
          /// 
   BasicEntity*           GetEntityValue();    // _no swig_
          /// 
   BasicEntity*           GetEntityValue(Boolean);
//@}

// INQUIRY

protected:

   BasicEntity*             mValue;
   BasicEntity*             mDefaultValue;
   UserClassIdTab           mValueClassIdTab;
   DescEntityValueSpec*     mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est un tableau d'entités */
//@{
           ///
class EntityTabVariableDescriptor : public VariableDescriptor
{
public:
/**@name LIFECYCLE */
//@{
         /// Affecte le type ENTITY\_TAB et la catégorie VARIABLE.
   EntityTabVariableDescriptor();   // _no swig_
         /// 
   EntityTabVariableDescriptor(UserDescriptorId);   // _no swig_    
         /// 
   EntityTabVariableDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// {\bf Important :} la valeur de la copie est la même que la valeur du descripteur copié (pointeurs sur le même tableau d'entités).
   EntityTabVariableDescriptor(const EntityTabVariableDescriptor&);// _no swig_
         ///
   ~EntityTabVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   EntityTabVariableDescriptor&      operator=(const EntityTabVariableDescriptor&);             
//@}              

/**@name [SC]Opérations sur le domaine[/SC] */
//@{
          /// Spécifie que les instances d'entités dans le tableau qui est la valeur du descripteur doivent être des instances de la classe dont le symbole est passé en argument. \index{méthodes!ValueClassId - Var.Descriptor}        
   void ValueClassId(UserClassId id);

          /// Ajoute un symbole de classe parmi ceux des classes qui peuvent être mère de l'entité. \index{méthodes!AddValueClassId - Var.Descriptor}        
   void AddValueClassId(UserClassId id);
 
          /// Renvoie le symbole de la classe qui est la classe des instances d'entités dans le tableau valeur. \index{méthodes!GetValueClassId - Var.Descriptor}                 
   UserClassId GetValueClassId();

          /// Renvoie la table des symboles des classes de valeur du descripteur.                 
   UserClassIdTab ValueClassIdTab();
//@}

/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          //
   void              RecordValue(char*);  // + JPR 050805
          /// 
   bool              IsInDomain(BasicEntity*) const;
          /// 
   bool              IsOutOfDomain(BasicEntity*) const;
          
   void              SetToDefaultValue();  // _no swig_  
          // 
   void              SetDefaultValue(pEntityTab*);  
          // 
   void              SetToDefaultValue(pEntityTab*);
          /// 
   void              SetValue(pEntityTab*);   // _no swig_ 
          /// 
   void              SetValue(pEntityTab*, Boolean);  
          /// 
   void              SetValueElement(BasicEntity*, int k);
          /// 
   void              SetValueElement(BasicEntity*, int k, Boolean); 
          /// Ajoute {\em pObj} à la liste de valeurs, seulement si {\em pObj} n'y est pas déjà présent.
   void              AddValueElement(BasicEntity* pObj);   // _no swig_ 
          /// Ajoute {\em pObj} à la liste de valeurs, seulement si {\em pObj} n'y est pas déjà présent. 
   void              AddValueElement(BasicEntity* pObj, Boolean);  
          /// 
   void              AddValueElements(pEntityTab*);   // _no swig_ 
          /// 
   void              AddValueElements(pEntityTab* pObj, Boolean);
          /// 
   void              RemoveValue(BasicEntity*);   // _no swig_ 
          /// 
   void              RemoveValue(BasicEntity*, Boolean); 
          /// 
   void              ClearValues();

          // 
   pEntityTab*       GetDefaultValue() const;   
          /// 
   pEntityTab*       GetEntityTabValue();    // _no swig_
          /// 
   pEntityTab*       GetEntityTabValue(Boolean);   
          /// 
   BasicEntity*      GetValueElement(int k);    // _no swig_   
          /// 
   BasicEntity*      GetValueElement(int k, Boolean);    // _no swig_
//@}

// INQUIRY

protected:

   pEntityTab*         mValue;
   pEntityTab*         mDefaultValue;
   UserClassIdTab      mValueClassIdTab;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est une spécification d'ensemble d'entités */
//@{
           ///
class EntitySpecVariableDescriptor : public VariableDescriptor
{
public:

/**@name LIFECYCLE */
//@{
         /// Affecte le type P\_ENTITYSPEC et la catégorie VARIABLE. 
   EntitySpecVariableDescriptor();    // _no swig_
         /// 
   EntitySpecVariableDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   EntitySpecVariableDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// 
   EntitySpecVariableDescriptor(const EntitySpecVariableDescriptor&);    // _no swig_
         ///
   ~EntitySpecVariableDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   EntitySpecVariableDescriptor&      operator=(const EntitySpecVariableDescriptor&); 
//@}
                           
/**@name  [SC]Opérations sur la valeur[/SC] */
//@{
          /// 
   EntitySpec*      GetEntitySpecValue() const;
          // 
   EntitySpec*      GetDefaultValue() const;
 
          /// 
   void         SetValue(EntitySpec*);
          // 
   void     SetToDefaultValue();  // _no swig_
          // 
   void     SetDefaultValue(EntitySpec*); 
          // 
   void     SetToDefaultValue(EntitySpec*); 
//@}

// INQUIRY

protected:

   EntitySpec*             mValue;
   EntitySpec*             mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est une chaÃ®ne de caractères */
//@{
           ///
class StringVariableDescriptor : public VariableDescriptor
{
public:
/**@name LIFECYCLE */
//@{
         /// Affecte le type STRING et la catégorie VARIABLE. 
   StringVariableDescriptor();   // _no swig_
         /// 
   StringVariableDescriptor(UserDescriptorId);    // _no swig_   
         /// 
   StringVariableDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// 
   StringVariableDescriptor(const StringVariableDescriptor&);    // _no swig_  
         ///
   ~StringVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   StringVariableDescriptor&      operator=(const StringVariableDescriptor&);
//@}

/**@name [SC]Opérations sur le domaine[/SC] */
//@{   
          /// 
   void SetDomain(DescStringValueSpec*);   
          ///  
   void SetSubDomain(char*); 
          /// 
   void AddSubDomain(char*);          
//@}                    

/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          // 
   void              RecordValue(char*); 
          /// 
   void              SetToDefaultValue();  // _no swig_ 
          /// 
   void              SetDefaultValue(char*);  
          /// 
   void              SetToDefaultValue(char*);
          /// 
   void              SetValue(char*);   // _no swig_ 
          /// 
   void              SetValue(char*, Boolean);
          /// 
   char*             GetDefaultValue() const; 
          /// 
   DescStringValueSpec* GetStringDomain() const;
          /// 
   bool              IsInDomain(char*) const;
          /// 
   bool              IsOutOfDomain(char*) const;  
          /// 
   char*             GetStringValue();    // _no swig_
          /// 
   char*             GetStringValue(Boolean);
//@}

// INQUIRY

protected:

   char*             mValue;
   char*             mDefaultValue;
   DescStringValueSpec* mDVSDomain;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est un tableau d'intervalles d'entiers */
//@{
           ///
class IntDomainVariableDescriptor : public MonoDimensionalNumericalVariableDescriptor 
{
public:

/**@name LIFECYCLE */
//@{

         /// Affecte le type INTDOMAIN et la catégorie VARIABLE.
   IntDomainVariableDescriptor();   // _no swig_
         /// 
   IntDomainVariableDescriptor(UserDescriptorId);    // _no swig_   
         /// 
   IntDomainVariableDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// 
   IntDomainVariableDescriptor(const IntDomainVariableDescriptor&);// _no swig_
         /// 
   ~IntDomainVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   IntDomainVariableDescriptor&      operator=(const IntDomainVariableDescriptor&); 
//@}
                           
/**@name  [SC]Opérations sur la valeur[/SC] */
//@{
          /// 
   pIntIntervalTab*      GetIntDomainValue() const;    // _no swig_
          /// 
   pIntIntervalTab*      GetIntDomainValue(Boolean);
          /// 
   pIntIntervalTab*      GetDefaultValue() const;

          /// 
   void     SetToDefaultValue();   // _no swig_
          /// 
   void     SetValue(pIntIntervalTab*);   // _no swig_ 
          /// 
   void     AddValueElement(int);   // _no swig_
          /// 
   void     AddValueElement(int, int);
          /// 
   void     InsertValueElement(int, int);
          /// 
   void     SetValue(pIntIntervalTab*, Boolean);
          /// 
   void     SetDefaultValue(pIntIntervalTab*); 
          /// 
   void     SetToDefaultValue(pIntIntervalTab*); 
          /// Renvoie {\em vrai} si et seulement si {\em val} est une des valeurs du descripteur, c'est-à-dire si {\em val} est dans un des intervalles qui définissent la valeur du descripteur. \index{méthodes!IsInValue - IntDomainVariableDescriptor}
   bool     IsInValue(int val) const;
//@}

// INQUIRY

protected:

   pIntIntervalTab*           mValue;
   pIntIntervalTab*           mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est un tableau d'intervalles de valeurs flottantes */
//@{
           ///
class FloatDomainVariableDescriptor : public MonoDimensionalNumericalVariableDescriptor 
{
public:

/**@name LIFECYCLE */
//@{

         /// Affecte le type FLOATDOMAIN et la catégorie VARIABLE.
   FloatDomainVariableDescriptor();   // _no swig_
         /// 
   FloatDomainVariableDescriptor(UserDescriptorId);    // _no swig_   
         /// 
   FloatDomainVariableDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// 
   FloatDomainVariableDescriptor(const FloatDomainVariableDescriptor&);// _no swig_
         /// 
   ~FloatDomainVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   FloatDomainVariableDescriptor&      operator=(const FloatDomainVariableDescriptor&);
//@}
                           
/**@name [SC]Opérations sur la valeur[/SC] */
//@{
          /// 
   pFloatIntervalTab*      GetFloatDomainValue() const;    // _no swig_
          /// 
   pFloatIntervalTab*      GetFloatDomainValue(Boolean);
          /// 
   pFloatIntervalTab*      GetDefaultValue() const;

          /// 
   void     SetToDefaultValue();   // _no swig_
          /// 
   void     SetValue(pFloatIntervalTab*);   // _no swig_ 
          /// 
   void     AddValueElement(float, float);
          /// 
   void     InsertValueElement(float, float);
          /// 
   void     SetValue(pFloatIntervalTab*, Boolean);
          /// 
   void     SetDefaultValue(pFloatIntervalTab*); 
          /// 
   void     SetToDefaultValue(pFloatIntervalTab*); 
          /// Renvoie {\em vrai} si et seulement si {\em val} est une des valeurs du descripteur, c'est-à-dire si {\em val} est dans un des intervalles qui définissent la valeur du descripteur. \index{méthodes!IsInValue - FloatDomainVariableDescriptor}
   bool     IsInValue(float val) const;
//@}

// INQUIRY

protected:

   pFloatIntervalTab*           mValue;
   pFloatIntervalTab*           mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est un tableau d'intervalles de flottants double précision */
//@{
           ///
class DoubleDomainVariableDescriptor : public MonoDimensionalNumericalVariableDescriptor 
{
public:

/**@name LIFECYCLE */
//@{

         /// Affecte le type DOUBLEDOMAIN et la catégorie VARIABLE.
   DoubleDomainVariableDescriptor();   // _no swig_
         /// 
   DoubleDomainVariableDescriptor(UserDescriptorId);    // _no swig_   
         /// 
   DoubleDomainVariableDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// 
   DoubleDomainVariableDescriptor(const DoubleDomainVariableDescriptor&);// _no swig_
         /// 
   ~DoubleDomainVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   DoubleDomainVariableDescriptor&      operator=(const DoubleDomainVariableDescriptor&);
//@}
                           
/**@name  [SC]Opérations sur la valeur[/SC] */
//@{
          /// 
   pDoubleIntervalTab*      GetDoubleDomainValue() const;    // _no swig_
          /// 
   pDoubleIntervalTab*      GetDoubleDomainValue(Boolean);
          /// 
   pDoubleIntervalTab*      GetDefaultValue() const;

          /// 
   void     SetToDefaultValue();   // _no swig_
          /// 
   void     SetValue(pDoubleIntervalTab*);   // _no swig_ 
          /// 
   void     AddValueElement(double, double);
          /// 
   void     InsertValueElement(double, double);
          /// 
   void     SetValue(pDoubleIntervalTab*, Boolean);
          /// 
   void     SetDefaultValue(pDoubleIntervalTab*); 
          /// 
   void     SetToDefaultValue(pDoubleIntervalTab*); 
          /// Renvoie {\em vrai} si et seulement si {\em val} est une des valeurs du descripteur, c'est-à-dire si {\em val} est dans un des intervalles qui définissent la valeur du descripteur. \index{méthodes!IsInValue - DoubleDomainVariableDescriptor}
   bool     IsInValue(double val) const;
//@}

// INQUIRY

protected:

   pDoubleIntervalTab*           mValue;
   pDoubleIntervalTab*           mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est un tableau de chaÃ®nes de caractères */
//@{
           ///
class StringTabVariableDescriptor : public VariableDescriptor
{
public:
/**@name LIFECYCLE */
//@{
         /// Affecte le type ENTITY\_TAB et la catégorie VARIABLE.
   StringTabVariableDescriptor();   // _no swig_
         /// 
   StringTabVariableDescriptor(UserDescriptorId);   // _no swig_    
         /// 
   StringTabVariableDescriptor(UserDescriptorId, char*);   // _no swig_    
         /// {\bf Important :} la valeur de la copie est la copie de la valeur (pointeurs différents sur deux tableaux des mêmes pointeurs d'entités).
   StringTabVariableDescriptor(const StringTabVariableDescriptor&);// _no swig_   
         ///
   ~StringTabVariableDescriptor();   // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   StringTabVariableDescriptor&      operator=(const StringTabVariableDescriptor&);               
//@}              

/**@name  [SC]Opérations sur la valeur[/SC] */
//@{
          /// 
   bool              IsInDomain(char*) const;
          /// 
   bool              IsOutOfDomain(char*) const;
          
   void              SetToDefaultValue();  // _no swig_  
          // 
   void              SetDefaultValue(TableauString*);  
          // 
   void              SetToDefaultValue(TableauString*);
          /// 
   void              SetValue(TableauString*);   // _no swig_ 
          /// 
   void              SetValue(TableauString*, Boolean); 
          /// Ajoute {\em str} à la liste de valeurs, seulement si {\em str} n'y est pas déjà présent.
   void              AddValueElement(char* str);   // _no swig_
          /// 
   void              RemoveValue(char*);   // _no swig_
          /// 
   void              ClearValues();
          /// 
   void              DepthDeleteValue();           // + JPR 120116
          /// 
   void              DepthDeleteDefaultValue();    // + JPR 120116

          // 
   TableauString*       GetDefaultValue() const;   
          /// 
   TableauString*       GetStringTabValue();    // _no swig_
          // 
   TableauString*       GetStringTabValue(Boolean);
//@}

// INQUIRY

protected:

   TableauString*         mValue;
   TableauString*         mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

  /**@name \ldots dont la valeur est un tableau de domaines de valeurs */
//@{
           ///
class DescValueSpecTabVariableDescriptor : public VariableDescriptor
{
public:

/**@name LIFECYCLE */
//@{
         /// Affecte le type DESCVALUESPEC\_TAB et la catégorie VARIABLE.
   DescValueSpecTabVariableDescriptor();    // _no swig_
         /// 
   DescValueSpecTabVariableDescriptor(UserDescriptorId);    // _no swig_    
         /// 
   DescValueSpecTabVariableDescriptor(UserDescriptorId, char*);    // _no swig_    
         /// {\bf Important :} la valeur de la copie est la copie de la valeur (pointeurs différents sur deux tableaux des mêmes pointeurs de [C]DescValueSpec[/C]).
   DescValueSpecTabVariableDescriptor(const DescValueSpecTabVariableDescriptor&);    // _no swig_    
         ///
   ~DescValueSpecTabVariableDescriptor();    // _no swig_
//@}

/**@name OPERATORS */
//@{
         /// 
   const
   DescValueSpecTabVariableDescriptor& operator=(const DescValueSpecTabVariableDescriptor&);
//@}
                           
/**@name  [SC]Opérations sur la valeur[/SC] */
//@{
          
          /// Renvoie toujours {\em vrai}. 
   bool              IsInDomain(DescValueSpec*) const;
          /// Renvoie toujours {\em faux}.
   bool              IsOutOfDomain(DescValueSpec*) const;
          /// 
   pDescValueSpecTab*      GetDescValueSpecTabValue();   // _no swig_  
          /// 
   pDescValueSpecTab*      GetDescValueSpecTabValue(Boolean); 
          /// 
   void             SetValue(pDescValueSpecTab*);   // _no swig_   
          /// 
   void             SetValue(pDescValueSpecTab*, Boolean); 
          /// Ajoute {\em pDVS} à la liste de valeurs, seulement si {\em pDVS} n'y est pas déjà présent.  
   void             AddValueElement(DescValueSpec* pDVS);   // _no swig_  
          ///
   void             AddValueElement(DescValueSpec* pDVS, Boolean); 
          /// 
   void             RemoveValue(DescValueSpec*);   // _no swig_   
          /// 
   void             RemoveValue(DescValueSpec*, Boolean); 
          /// 
   void             ClearValues();

          // 
   pDescValueSpecTab*      GetDefaultValue() const;

          // 
   void             SetToDefaultValue();  // _no swig_
          // 
   void             SetDefaultValue(pDescValueSpecTab*);
          // 
   void             SetToDefaultValue(pDescValueSpecTab*);
//@}

// INQUIRY

protected:

   pDescValueSpecTab*         mValue;
   pDescValueSpecTab*         mDefaultValue;

private:
}; // _end class_

//@} // fin classe

//@} // fin sous-classes typées de descripteurs variables

//@} // fin descripteurs variables

// INLINE METHODS 


// EXTERNAL REFERENCES

#endif  // _VARIABLEDESCRIPTOR_HH
