//============================================================================
// FILE: C_CoreEvent.h
//
// TYPE: C++CLASS 
// DESCRIPTION: entête des événements "moteur" de CONTROL DIESE
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_COREEVENT_HH
#define _C_COREEVENT_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "DiscreteProcess.h" 
#include "Event.h" 

#include "C_Operation.h"  

class DiscreteProcess;

/**@name Processus fondamentaux prédéfinis
Les classes présentées dans ces chapitres sont des sous-classes de {\bf Event} et de {\bf Process}. C'est normalement l'utilisateur qui crée des descendants de la classe de base, en fonction de son domaine d'application. Cependant, certains événements apparaissent dans suffisamment de domaines pour que \CDb fournisse les classes correspondantes. Il s'agit de :\\ 
\begin{description} 

\item[au titre des activités et des opérations : ]
\begin{itemize}
\item l'examen de la {\em trajectoire réactive}, composante de la {\em stratégie}, 
\item la révision de l'{\em état des activités} des plans de la stratégie (ouverture/fermeture), 
\item l'établissement de la liste des {\em instructions} à exécuter,
\item la réaction au déclenchement d'une {\em alarme},
\item la mise en oeuvre conjointe des {\em opérations} sur le système piloté,
\item l'initialisation et la poursuite d'une opération particulière. 
\end{itemize}

\item[au titre des ressources : ]
\begin{itemize}
\item l'immobilisation d'une ressource,
\item sa mobilisation.
\end{itemize} 

\item[au titre des fichiers de données : ]
\begin{itemize}
\item la lecture des enregistrements, avec fonction de mémoire et de prédiction
\end{itemize} 

\end{description} 

*/
//@{   // fermeture dans C_CoreProcess.h

/**@name Evénements prédéfinis liés aux activités
\begin{itemize} \item Directive d'inclusion : {\tt \#include C\_CoreEvent.h} \end{itemize}
*/
//@{

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Révision de l'état des activités
\label{defEventUpdateSituation}Un événement de cette classe a pour effet de provoquer, de la part du {\em controleur}, la mise à jour de la situation des activités dans tous les plans de tous les blocs activités-ressources de la stratégie (voir page \pageref{defUpdateSituation}). \index{UpdateSituationEvent} \\
*/
//@{

/**@name La classe UpdateSituationEvent */
//@{
        ///
class UpdateSituationEvent : public Event
{
public:          
          // Constructeur de base. Normalement appelé seulement dans PostUpdateSituationEvent.
   UpdateSituationEvent();
};
//@}
        /// 
bool updateSituationEvent_generateNextEvent_Body(EventMethod* pM);
//@}

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Redéfinition des instructions à exécuter
\label{defEventMakeInstructionList}Un événement de cette classe a pour effet de provoquer, de la part du {\em système opérant}, l'établissement de la liste des {\em instructions} à exécuter (voir page \pageref{defMakeInstructionList}). Il est placé dans l'agenda par l'appel à la fonction\newline {\bf PostMakeInstructionListEvent} (voir page \pageref{defTriggerControl}).\index{MakeInstructionListEvent} \\
*/
class MakeInstructionListEvent : public Event
{
public:          
          // Constructeur de base. Normalement appelé seulement dans PostMakeInstructionListEvent.
   MakeInstructionListEvent();
};

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Mise en oeuvre conjointe des opérations
\label{defEventActInstructionList}Un événement de cette classe a pour effet de provoquer, de la part du {\em système opérant}, la mise en oeuvre des {\em opérations} entrant dans la composition des instructions du jeu issu de l'occurrence de l'événement {\bf MakeInstructionListEvent} (voir page \pageref{defActInstructionList}). Il est placé dans l'agenda par l'appel à la fonction\newline {\bf C\_PostActInstructionListEvent} en fin du mécanisme d'établissement des instructions (voir page \pageref{defTriggerControl}).\index{ActInstructionListEvent} \\
*/
//@{
        ///
class ActInstructionListEvent : public Event
{
public:          
          // Constructeur de base. Normalement appelé seulement dans PostActInstructionListEvent.
   ActInstructionListEvent();
};
//@}

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Réalisation d'une opération
\label{defEventProceedOperation}Un événement de cette classe a pour effet de provoquer, de la part du {\em système opérant}, la mise en oeuvre d'une{\em opération}. Il est placé dans l'agenda par l'appel à la fonction\newline {\bf PostProceedOperationEvent} dans le mécanisme de lancement conjoint des opérations (voir page \pageref{defTriggerControl}).\\
*/
//@{
        ///
class ProceedOperationEvent : public Event
{
public:          
          // Constructeur de base. Normalement appelé seulement dans PostProceedOperationEvent.
   ProceedOperationEvent();
   ProceedOperationEvent(Operation*);  // + JPR 040223
};
//@}

//@} // fin ev. liés aux activités

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Evénements prédéfinis liés aux ressources
Les deux classes  {\bf ResourceImmobilizationEvent} et {\bf ResourceMobilizationEvent} représentent des événements dans l'environnement du système, avec effet sur le système : l'immobilisation d'une ressource, laquelle n'est alors plus allouable à aucune activité, et la mobilisation d'une ressource, laquelle devient allouable pour toute activité.\\

Cet effet sur le système est spécifié par un processus discret, instance de {\bf ResourceImmobilizationProcess} ou de {\bf ResourceMobilizationProcess} .\\

Ces deux événements sont les sujets d'une génération automatique. Pour la classe {\bf ResourceImmobilizationEvent}, l'événement généré par la fonction assignée par {\bf SetGenerateNextEvent} est de la classe {\bf ResourceMobilizationEvent} (et {\em vice-versa}). La date aléatoire de son occurrence est déterminée à partir des valeurs affectées par {\bf SetMinDelayNextEvent} et {\bf SetMaxDelayNextEvent} et de la fonction de tirage aléatoire assignée par {\bf SetRandomizeNextEventClockTime} (basée sur la loi uniforme, par défaut).\\

Tout événement d'un type ou de l'autre est dotée d'une postconséquence. Il s'agit de la programmation, à la date courante et avec une priorité maximale, d'un événememt {\bf MakeInstructionListEvent}.
*/
//@{
/**@name Immobilisation d'une ressource
\label{defEventResourceImmobilization}\index{ResourceImmobilizationEvent}
*/
//@{
        ///
class ResourceImmobilizationEvent : public Event
{
public:          
          // Constructeur de base. 
   ResourceImmobilizationEvent();
};
//@}

bool resourceImmobilizationEvent_generateNextEvent_Body(EventMethod*);
void resourceEvent_postConsequence_Body(EventMethod*);

// X///////////////X/////////////////////////////X////////////////////////////

/**@name Mobilisation d'une ressource
\label{defEventResourceMobilization}\index{ResourceMobilizationEvent} 
*/
//@{
        ///
class ResourceMobilizationEvent : public Event
{
public:          
          // Constructeur de base. 
   ResourceMobilizationEvent();
};
//@}

bool resourceMobilizationEvent_generateNextEvent_Body(EventMethod*);

// X///////////////X/////////////////////////////X////////////////////////////

//@} // fin event resource

#endif  // _C_COREEVENT_HH
