//============================================================================
// FILE: C_ResourceCapacity.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête des capacités des ressources
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
// REFERENCE : "Fondements ontologiques des systèmes pilotés"
//             R. Martin-Clouaire, J.-P. Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _C_RESOURCECAPACITY_HH
#define _C_RESOURCECAPACITY_HH

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include "Entity.h"

#include "C_CoreTypes.h"

// EXTERNAL REFERENCES

/**@name Capacité des ressources\\

Une ressource dite "à capacité" (classe [C]SingleCapacitedResource[/C] ou [C]HomogeneousAggregatedCapacitedResource[/C]) est caractérisée par un vecteur de valeurs de capacité. Chaque valeur se rapporte à un volet de ce qu'offre la ressource aux activités qui la requièrent. \\

Par exemple, un avion peut offir une capacité de 100 places assises et un volume de 50 tonnes de frêt. Les deux offres feront chacune l'objet d'une instance de [C]ResourceCapacity[/C], et ces deux instances seront les éléments d'une instance de [C]MultipleResourceCapacity[/C]. Et cette dernière sera la valeur du descripteur [C]ResourceCapacitySupply[/C] de notre classe [C]Avion[/C].\\

Un usage complémentaire de cette cette classe est fait par le moteur de simulation : représenter, dans une activité de transfert, la demande en capacité que doit satisfaire la source du transfert. Cette demande sera comparée à l'offre pour déterminer si le transfert est réalisable.\\

Les mécanismes qui manipulent les capacités sont codés dans le moteur de simulation. Le développeur d'une application, et l'utilisateur du simulateur, n'ont normalement qu'à déclarer les valeurs de capacités des ressources, notamment par les services [Y]Install<type>Capacity[/Y]. Cependant il peut exister des motifs particulier d'utiliser les services [Y]UseCapacity[/Y] et [Y]RefillCapacity[/Y] de la classe [C]SingleCapacitatedReusableResource[/C], et [Y]ConsumeCapacity[/Y] de la classe [C]SingleCapacitatedConsumableResource[/C].
*/ 
//@{

// /X///////////////X/////////////////////////////X////////////////////////////
//
// CAPACITE MULTIPLE D'UNE RESSOURCE
//
// /X///////////////X/////////////////////////////X////////////////////////////

class ResourceCapacity;

/**@name Capacité multiple d'une ressource\\
\begin{itemize} \item Directive d'inclusion : 
[P]\#include C_ResourceCapacity.h[/P]
\end{itemize}

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]MultipleResourceCapacity[/C]} & {\tiny MULTIPLE_RESOURCE_CAPACITY} & {\tiny multipleResourceCapacity} & {\tiny RESOURCE_CAPACITY} & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]DescribedResource[/C] DESCRIBED_RESOURCE describedResource} & {\tiny NULL} & {\tiny la ressource qui détient cette capacité}
\end{tabular}
*/ 
//@{
          ///
class MultipleResourceCapacity : public Entity
{
public:
          // Constructeur de base.
  MultipleResourceCapacity();
  
          // Constructeur par copie.
  MultipleResourceCapacity(const MultipleResourceCapacity&);
  
          // Destructeur.
  ~MultipleResourceCapacity();

/**@name  OPERATION */
//@{
          ///
void Install(UserDescriptorId);
          ///
void Init(UserDescriptorId);
          ///
void Use(UserDescriptorId, int);
void Use(UserDescriptorId, float);
void Use(UserDescriptorId, double);
          ///
void Consume(UserDescriptorId, int);
void Consume(UserDescriptorId, float);
void Consume(UserDescriptorId, double);

          ///
ResourceCapacity* GetSingleResourceCapacity(UserDescriptorId);

//@}
};
//@}  // fin class ResCap

// /X///////////////X/////////////////////////////X////////////////////////////
//
// CAPACITE SIMPLE D'UNE RESSOURCE
//
// /X///////////////X/////////////////////////////X////////////////////////////
/**@name Capacité simple d'une ressource\\
\begin{itemize} \item Directive d'inclusion : 
[P]\#include C_ResourceCapacity.h[/P]
\end{itemize}

\begin{tabular}{cccll}
{\tiny nom C++} & {\tiny symbole de classe} & {\tiny nom de classe} & {\tiny classe des éléments} & {\tiny classes des composants}\hline
\hline
{\tiny [C]ResourceCapacity[/C]} & {\tiny RESOURCE_CAPACITY} & {\tiny resourceCapacity} & {\tiny } & {\tiny }
\end{tabular}

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
 & {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny int} & {\tiny [C]TypeAttribute[/C] TYPE_ATTRIBUTE typeAttribute} & {\tiny } & {\tiny INT, FLOAT, etc.}\hline
{\tiny C} & {\tiny int} & {\tiny [C]DescriptorIdAttribute[/C] DESCRIPTOR_ID_ATTRIBUTE descriptorIdAttribute} & {\tiny -9} & {\tiny le symbole de classe du descripteur}\hline
{\tiny C} & {\tiny char*} & {\tiny [C]DescriptorNameAttribute[/C] DESCRIPTOR_NAME_ATTRIBUTE descriptorNameAttribute} & {\tiny ""} & {\tiny le nom de classe du descripteur}\hline
{\tiny C} & {\tiny pEntity*} & {\tiny [C]DescribedResource[/C] DESCRIBED_RESOURCE describedResource} & {\tiny NULL} & {\tiny la ressource qui détient cette capacité}
\end{tabular}

Dans le tableau ci-dessus, le descripteur est un de ceux de la ressource qui détient la capacité. C'est sur la grandeur que représente ce descripteur que va être mesurée la capacité offete ou demandée par la ressource. Pour reprendre l'exemple pris en introduction générale des capacités des ressources, ça peut-être par exemple le nombre de places assises d'un avion, ou son volume total des containers de frêt.\\

Selon que ce desripteur est à valeur entière ou flottante, on exploite des sous-classes correspondantes : [C]IntegerResourceCapacity[/C] et [C]FloatResourceCapacity[/C], qui possèdent un descripteur spécifique pour capter la valeur.\\

Pour les ressources spéciales que sont les stocks (classe [C]SingleStock[/C]), les sous-classes sont [C]IntegerStockCapacity[/C] et [C]FloatStockCapacity[/C].
*/ 
//@{
        ///
class ResourceCapacity : public Entity
{
public:
          // Constructeur de base.
  ResourceCapacity();
  
          // Constructeur par copie.
  ResourceCapacity(const ResourceCapacity&);
  
          // Destructeur.
  ~ResourceCapacity();

/* *@name  OPERATION */
// @{
// @}
};

//@}

/**@name Capacité simple à valeur entière d'une ressource

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
& {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny V} & {\tiny int} & {\tiny [C]AnonymousIntAttribute[/C] ANONYMOUS_INT_ATTRIBUTE anonymousIntAttribute} & {\tiny O} & {\tiny valeur de la capacité}
\end{tabular}

Ce descripteur (prédéfini dans \D), est géré par le moteur de simulation via les services [Y]InstallIntegerCapacity[/Y], [Y]UseCapacity[/Y], [Y]RefillCapacity[/Y] de la classe [C]SingleCapacitatedReusableResource[/C], et [Y]ConsumeCapacity[/Y] de la classe [C]SingleCapacitatedConsumableResource[/C]. Le code de l'application ne doit changer sa valeur que pour une raison exceptionnellement motivée. L'accès à sa valeur est par contre possible pour un motif propre au domaine étudié.
*/ 
//@{
        ///
class IntegerResourceCapacity : public ResourceCapacity
{
public:
          // Constructeur de base.
  IntegerResourceCapacity();
  
          // Constructeur par copie.
  IntegerResourceCapacity(const IntegerResourceCapacity&);
  
          // Destructeur.
  ~IntegerResourceCapacity();
};
//@}

/**@name Capacité simple à valeur réelle d'une ressource

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
& {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny V} & {\tiny float} & {\tiny [C]AnonymousFloatAttribute[/C] ANONYMOUS_FLOAT_ATTRIBUTE anonymousFloatAttribute} & {\tiny O.} & {\tiny valeur de la capacité}
\end{tabular}

Ce descripteur (prédéfini dans \D), est géré par le moteur de simulation via les services [Y]InstallFloatCapacity[/Y], [Y]UseCapacity[/Y], [Y]RefillCapacity[/Y] de la classe [C]SingleCapacitatedReusableResource[/C], et [Y]ConsumeCapacity[/Y] de la classe [C]SingleCapacitatedConsumableResource[/C]. Le code de l'application ne doit changer sa valeur que pour une raison exceptionnellement motivée. L'accès à sa valeur est par contre possible pour un motif propre au domaine étudié.
*/ 
//@{
        ///
class FloatResourceCapacity : public ResourceCapacity
{
public:
          // Constructeur de base.
  FloatResourceCapacity();
  
          // Constructeur par copie.
  FloatResourceCapacity(const FloatResourceCapacity&);
  
          // Destructeur.
  ~FloatResourceCapacity();
};
//@}

/**@name Les capacités d'une ressource particulière : les stocks

On rappelle ici qu'un stock est une ressource caractérisée par un niveau, dont la valeur courante est limitée inférieurement et supérieurement. Ces trois valeurs sont installées dans une instance, attachée au stock, d'une des sous-classes de [C]ResourceCapacity[/C] :
\begin{itemize}
\item [C]IntegerStockCapacity[/C] si le descripteur du stock sur lequel est mesuré la capacité est à valeur entière ;
\item [C]FloatStockCapacity[/C] si ce descripteur est à valeur réelle.
\end{itemize}
*/ 
//@{

/**@name Les capacités des stocks à valeur entière

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
& {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny int} & {\tiny [C]IntUpperLevel[/C] INT_UPPER_LEVEL intUpperLevel} & {\tiny O} & {\tiny borne supérieure de la capacité}\hline
{\tiny C} & {\tiny int} & {\tiny [C]IntLowerLevel[/C] INT_LOWER_LEVEL intLowerLevel} & {\tiny O} & {\tiny borne inférieure de la capacité}\hline
{\tiny V (1)} & {\tiny int} & {\tiny [C]IntCurrentLevel[/C] INT_CURRENT_LEVEL intCurrentLevel} & {\tiny O} & {\tiny valeur courante de la capacité}\hline
{\tiny V} & {\tiny int} & {\tiny [C]IntInCapacity[/C] INT_IN_CAPACITY intInCapacity} & {\tiny -9} & {\tiny valeur courante de la capacité en entrée}\hline
{\tiny V} & {\tiny int} & {\tiny [C]IntOutCapacity[/C] INT_OUT_CAPACITY intOutCapacity} & {\tiny -9} & {\tiny valeur courante de la capacité en sortie}
\end{tabular}

La capacité 'en entrée' est la différence entre la valeur courante de la capacité et sa borne supérieure : la ressource ne peut pas recevoir plus.\\
La capacité 'en sortie' est la différence entre la valeur courante de la capacité et sa borne inférieure : la ressource ne peut pas offrir plus.

  {\bf ** Moniteurs :}\\

\begin{tabular}{ccl}
 & {\tiny } & {\tiny événement} & {\tiny }\hline
 & {\tiny (1)} & {\tiny SetValue} & {\tiny . ajuste les capacités 'en entrée' et 'en sortie' de la ressource décrite \newline . lève l'exception [Y]IntegerStockLevelOutOfBounds[/Y] si une limite inf. ou sup. serait dépassée}
\end{tabular}
*/ 
//@{
          /// 
class IntegerStockCapacity : public ResourceCapacity
{
public:
          // Constructeur de base.
  IntegerStockCapacity();
          // Constructeur par copie.
  IntegerStockCapacity(const IntegerStockCapacity&);
          // Destructeur.
  ~IntegerStockCapacity();

/**@name [SC]Positivité de la capacité en entrée[/SC] */
//@{
          /// Renvoie {\em vrai} si la capacité en entrée (capacité à recevoir) est strictement positive.
  Boolean IsInCapacityPositive();
//@}
};
//@}

/**@name Les capacités des stocks à valeur réelle

  {\bf Descripteurs :}\\

\begin{tabular}{ccccl}
& {\tiny type} & {\tiny classe, symbole, nom} & {\tiny v. par défaut} & {\tiny }\hline
\hline
{\tiny C} & {\tiny float} & {\tiny [C]FloatUpperLevel[/C] FLOAT_UPPER_LEVEL floatUpperLevel} & {\tiny O} & {\tiny borne supérieure de la capacité}\hline
{\tiny C} & {\tiny float} & {\tiny [C]FloatLowerLevel[/C] FLOAT_LOWER_LEVEL floatLowerLevel} & {\tiny O} & {\tiny borne inférieure de la capacité}\hline
{\tiny V (1)} & {\tiny float} & {\tiny [C]FloatCurrentLevel[/C] FLOAT_CURRENT_LEVEL floatCurrentLevel} & {\tiny O} & {\tiny valeur courante de la capacité}\hline
{\tiny V} & {\tiny float} & {\tiny [C]FloatInCapacity[/C] FLOAT_IN_CAPACITY floatInCapacity} & {\tiny -9} & {\tiny valeur courante de la capacité en entrée}\hline
{\tiny V} & {\tiny float} & {\tiny [C]FloatOutCapacity[/C] FLOAT_OUT_CAPACITY floatOutCapacity} & {\tiny -9} & {\tiny valeur courante de la capacité en sortie}
\end{tabular}

La capacité 'en entrée' est la différence entre la valeur courante de la capacité et sa borne supérieure : la ressource ne peut pas recevoir plus.\\
La capacité 'en sortie' est la différence entre la valeur courante de la capacité et sa borne inférieure : la ressource ne peut pas offrir plus.

  {\bf ** Moniteurs :}\\

\begin{tabular}{ccl}
 & {\tiny } & {\tiny événement} & {\tiny }\hline
 & {\tiny (1)} & {\tiny SetValue} & {\tiny . ajuste les capacités 'en entrée' et 'en sortie' de la ressource décrite \newline . lève l'exception [Y]FloatStockLevelOutOfBounds[/Y] si une limite inf. ou sup. serait dépassée}
\end{tabular}
*/ 
//@{
          ///
class FloatStockCapacity : public ResourceCapacity
{
public:
          // Constructeur de base.
  FloatStockCapacity();
          // Constructeur par copie.
  FloatStockCapacity(const FloatStockCapacity&);
          // Destructeur.
  ~FloatStockCapacity();

/**@name [SC]Positivité de la capacité en entrée[/SC] */
//@{
          /// Renvoie {\em vrai} si la capacité en entrée (capacité à recevoir) est strictement positive.
  Boolean IsInCapacityPositive();
//@}
};

//@}

//@}


//@}  // Capacité des ressource



#endif  // _C_RESOURCECAPACITY_HH
