//============================================================================
// FILE: BasicEntity.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe des entités sans descripteurs ni méthodes
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _BASICENTITY_HH
#define _BASICENTITY_HH

// SYSTEM INCLUDES
//
//#include <>                                   //

// PROJECT INCLUDES
//
#include "CoreTypes.h"

//
// 070101 (symboliquement) 
// suppression de la gestion des listes d'entités libres
// (= les mpFreeInstanceTab des MetaClass, et ce qui va avec dans MetaClass,
// BasicEntity et CoreFunction)
// -> vitesse très augmentée
// Corollairement, recodage de la fonction globale GetFreeEntityList(Id)
// (qui exploitait les listes d'instances libres)
//

/**@name CDPFORTEXLes entités
Une entité est un granule du modèle structurel du domaine, en complémentarité avec les processus (classe [C]Process[/C]), qui en sont le modèle fonctionnel, et les événements (classe [C]Event[/C]), qui en sont le modèle dynamique. \\ 

Dans un simulateur écrit avec DIESE, une entité est décrite par ses caractéristiques propres (sous la forme de variables ou de fonctions) et par ses relations structurales avec d'autres entités. \\\\ 

{\large {\tt Relations structurales entre entités}}\\

Les caractères structuraux sont les relations qui lient l'entité à ses parties. Dans \D, ces relations sont ramenées à seulement deux : la relation de {\em composition}\index{relation de composition} et la relation {\em ensembliste}\index{relation ensembliste}. Une entité est constituée d'un certain nombre de {\em composants}\index{composant} de types différents ou, exclusivement, contient un certain nombre {\em d'éléments}\index{élément} tous du même type. Réciproquement, une entité possède une, et une seule, {\em super-entité}\index{super-entité} (c'est-à-dire une entité dont elle est un {\em composant}), et appartient à un certain nombre de {\em sur-ensembles}\index{sur-ensemble} (c'est-à-dire d'entités dont elle est {\em élément}, et qui peuvent être de différents types).\\

Pour mieux définir les notions de {\em composant} et d'{\em élément}, on peut ajouter que :
\begin{itemize}
\item Enlever ou ajouter un {\em élément} à un ensemble ne dénature pas cet ensemble. Un même entité (instance) peut appartenir à plusieurs ensembles, qui peuvent être de différents types. Un ensemble peut être vide, puis recevoir et se voir retirer des {\em éléments}, sans inhiberer ses fonctions.
\item Enlever ou ajouter un {\em composant} à une entité dénature l'entité (le tout). Une entité ne peut pas être {\em composant} de plus d'une autre entité. Une entité peut être construite en assemblant ses {\em composants} les uns après les autres. Le retrait d'un {\em composant} détruit l'entité et inhibe tout ou partie de ses fonctions.
\end{itemize}
\\

{\large {\tt Caractéristiques propres des entités}}\\

Les descripteurs (proprement dit) d'une entité correspondent aux notions classiques de variables d'état et de propriétés à valeur constante. Dans \D, les descripteurs sont eux-aussi des objets. Ce sont des instances de sous-classes de [C]Descriptor[/C]. Ces objets possèdent en particulier un attribut qui représente la {\em valeur} de la variable d'état ou de la constante. Toute entité, dans \D, possède un descripteur prédéfini dont la valeur est un tableau d'instances de [C]Descriptor[/C]. On ajoute donc un élément à ce tableau pour chaque variable d'état et propriété à valeur constante de l'entité.\\

Il est d'autre part possible de doter une sous-classe d'entités par un ensemble de caractéristiques fonctionnelles, c'est-à-dire de connaissances procédurales sur l'entité. Dans la terminologie de la conception orientée objet, une telle caractéristique est généralement appelée 'méthode'.\\
Cette adjonction de méthodes à une entité dans le domaine d'application n'est pas inéluctable. En effet, et d'une part, des services généraux tels que la manipulation des variables d'état sont définis et fournis dans la bibliothèque \D. D'autre part, certaines connaissances procédurales sur l'entité sont en fait un granule du modèle fonctionnel du système dans lequel est incluse l'entité. On les représentera donc comme des {\em processus} et on les codera dans des sous-classes de [C]Process[/C] de \Db (voir la section dédiée).\\
Attacher un descripteur fonctionnel à une entité du domaine d'application peut être réalisé de deux manières :
\begin{itemize}
\item en déclarant (dans le fichier d'interface de la classe) et en réalisant (dans le fichier de réalisation) une méthode au sens du C++ natif, d'accès public (mot-clé {\em public}) ou restreint à la classe (mot-clé {\em protected}). On sait que cette méthode est alors une propriété constante de toutes les instances de la classe. Autrement dit, aucune instance ne peut être caractérisée par un {\em corps} particulier de la méthode. Toujours en C++ natif, on peut surcharger différement dans des sous-classes le {\em corps} d'une méthode {\em virtuelle} de la classe-mère, mais leurs instances propres restent indifférenciées.
\item en exploitant la classe [C]Method[/C] prédéfinie dans \Db (voir la section dédiée) : une connaissance fonctionnelle sur l'entité fait l'objet d'une sous-classe de [C]Method[/C], et on attache à la classe d'entités, dans son constructeur, une instance de cette sous-classe de [C]Method[/C]. Comme toute entité, dans \D, possède un descripteur prédéfini dont la valeur est un tableau d'instances de [C]Method[/C], il suffit d'ajouter l'instance à ce tableau. 
\noLF
Une instance de [C]Method[/C] possède en particulier un attribut qui est le {\em corps} de la méthode (de manière analogue au fait qu'un descripteur possède un attribut qui représente sa {\em valeur}). Il est ainsi possible de distinguer deux instances de la même sous-classe d'entités par deux corps distincts de la même méthode (de manière analogue au fait que deux entités peuvent avoir deux valeurs distinctes pour le même descripteur).
\end{itemize}
\\

{\large {\tt En résumé : la conception générérale d'une entité}}\\

Toute entité est caractérisée par les attributs suivants :
\begin{itemize}
\item {\em l'indication de sa super-entité}\\
	  - éventuellement inexistante (l'indication est alors le pointeur NULL)
\item {\em le tableau de ses composants}\\
- éventuellement vide, et nécessairement vide si le tableau de ses éléments ne l'est pas
\item {\em le tableau de ses sur-ensembles}\\
- éventuellement vide
\item {\em le tableau de ses éléments}\\
- éventuellement vide, et nécessairement vide si le tableau de ses composants ne l'est pas
\end{itemize}
Ce jeu d'attributs définit la classe de base [C]BasicEntity[/C]. Certaines entités peuvent en outre posséder :\\

\begin{itemize}
\item {\em un tableau de descripteurs constants} 
\item {\em un tableau de descripteurs variables} (au sens de variables d'état)
\end{itemize}
Ce complément définit la sous-classe [C]DescriptedEntity[/C] de [C]BasicEntity[/C]. Enfin, certaines entités peuvent posséder :\\
  
\begin{itemize}
 \item {\em un tableau de méthodes}
\end{itemize}
Ce complément définit la sous-classe  [C]Entity[/C] de [C]DescriptedEntity[/C]. Les tableaux de descripteurs peuvent être vides.\\

Remarques importantes : 
\begin{itemize}
\item Cette structure est la même d'une classe à l'autre. L'utilisateur ne peut donc introduire des différences entre classes qu'en différenciant les contenus des tableaux (de composants, d'éléments, de descripteurs, de méthodes, etc.).\\

\item Une relation de {\em particularisation/généralisation}\index{particularisation}\index{généralisation} entre classes sera traduite par la classique hiérarchisation des classes (avec la fonctionnalité inhérente d'héritage). Une telle relation lie, par exemple, la classe des {\em bâtiments} et la classe des {\em maisons} (voir à la page 'Gestion de la hiérarchie des classes d'entités' la bonne manière d'installer une hiérarchie de classes).\\

\item Une relation de type {\em antécédent/image}\index{relation antécédent/image} (par exemple la relation {\em père}/{\em fils}) pourra être modélisée par un descripteur approprié, prenant ses valeurs parmi les entités. Une relation réciproque fera l'objet de deux descripteurs, chacun installé dans une des deux classes : le {\em fils} est pointé par un descripteur de la classe du {\em père} et le {\em père} est pointé par un descripteur de la classe du {\em fils}.
\end{itemize}

On décrit par ailleurs la notion de {\em démon}. Un démon est une structure permettant de spécifier des morceaux de code que DIESE doit exécuter lorsqu'on accède à la valeur d'un descripteur, lorsqu'on modifie cette valeur, ou encore lorsqu'on modifie la structure de l'entité. L'utilisateur exploite les méthodes normales d'accès et de modification, mais le résultat de ces méthodes (en terme de valeur renvoyée ou d'effets de bord) passe par le "filtre" du démon, et peut donc varier en fonction de l'état courant de l'entité ou de ses descripteurs. \\\\

{\large {\tt Les trois classes de base d'entités}}\\

Ainsi \Db propose-t-elle trois classes de base pour représenter une entité.\\

{\scriptsize
\begin{tabular}{lll}
                                      &{\small {\tt descripteurs fonctionnels} & {\small {\tt pas de\newline descripteurs fonctionnels} \\
{\small {\tt variables d'état\newline et/ou constantes}       & [C]Entity[/C] &  [C]DescriptedEntity[/C]\newline [C]Entity[/C]  \\
{\small {\tt pas de variables d'état\newline ni constantes} & [C]Entity[/C] & [C]BasicEntity[/C]\newline [C]DescriptedEntity[/C]\newline [C]Entity[/C] \\
\end{tabular}
}

Si l'entité possède des descripteurs fonctionnels (méthodes), c'est de la classe de base [C]Entity[/C] que devra descendre la classe de l'entité considérée.\\
Si l'entité ne possède pas de descripteurs fonctionnels, mais possède des descripteurs au sens de variables d'état, elle pourra descendre d'[C]Entity[/C], mais [C]DescriptedEntity[/C] suffirait comme classe de base, par mesure d'économie et de clarté.\\
Enfin, si l'entité ne possède de descripteurs d'aucun type, elle pourra descendre de [C]DescriptedEntity[/C] ou d'[C]Entity[/C], mais [C]BasicEntity[/C] suffirait comme classe de base.

Ces classes de base ne sont pas destinées à avoir des instances directes. Les entités du domaine d'application de l'utilisateur font l'objet de classes spécifiques au domaine, lesquelles doivent être créées comme des sous-classes d'une de ces trois classes de base.\\

Les trois liens en haut de la présente page décrivent chacun une des trois classes de base des entités.

*/
//@{         // fermeture dans Entity.h


/**@name CDPFORTEXLa classe des entités sans descripteurs ni méthodes 
\begin{itemize} \item Directive d'inclusion :
[P]
\#include BasicEntity.h
[/P]
\end{itemize}

	Cette classe est la classe mère appropriée pour les classes d'entités du domaine qui ne sont pas caractérisées par des variables d'état ou des attributs à valeur constante (classe [C]Descriptor[/C]), ni des descripteurs fonctionnels (méthodes, de la classe [C]Method[/C]).\\

Dans ce cas en effet, il suffit que les instances créées possèdent tous les services nécessaires à l'identification de l'entité, à la gestion de sa structure, à la gestion de sa place dans la hiérarchie des classes, services prédéfinis dans la classe [C]BasicEntity[/C].
*/
//@{
        ///
class BasicEntity 
{
public:

/**@name LIFECYCLE  */
//@{
          /// Constructeur de base, invoqué seulement par les constructeurs des sous-classes. Ajoute l'entité en cours de création à la liste des instances (indirectes) de [C]BasicEntity[/C]. Il est important de noter qu'aucune entité ne doit être créée avant que la simulation ne soit initialisée par la construction d'une instance de la classe [C]Simulation[/C], et plus particulièrement qu'une date courante lui soit affectée (0 par défaut). La connaissance de la date de création est effet nécessaire à la gestion de la sauvegarde de certaines informations sur l'entité (en particulier la valeur des descripteurs constants dans la sous-classe [C]DescriptedEntity[/C]). L'instance créée possède le type O\_BASIC\_ENTITY, valeur renvoyée par [Y]EntityType[/Y].
   BasicEntity(); // _no swig_
          /// Constructeur par copie profonde (c'est-à-dire copie des composants et des éléments). Un nom d'instance distinct de celui de l'entité copiée est attribué par la règle par défaut (un numéro unique concaténé derrière le nom de classe, voir [Y]SetInstanceName[/Y]). Les méthodes du démon éventuellement attaché à la structure de l'entité sont déclenchées. Ajoute l'entité en cours de création par copie à la liste des instances (indirectes) de [C]BasicEntity[/C].
   BasicEntity(const BasicEntity&); // _no swig_ 
   /*
     JPR 0609 (appli Moderato)
     L'initialisation du processus de progression d'une opération (cdiese)
     réalise une copy (par CopyEntity) de l'entité sur laquelle l'ot porte.
     Dans l'appli en question, l'entité est dotée d'un StructureMonitor.
     Le code de ce démon (whenAddElement) accède aux descripteurs de l'entité.
     Or, la copie commence par la structure commune BasicEntity, puis celle
     spécifique à DescriptedEntity, puis enfin celle spécifique à Entity.
     Les éléments sont ajoutés à la copie dans la première étape, alors que
     les descripteurs ne le sont qu'à la seconde. Le démon ne trouve donc pas
     les descripteurs auxquels il souhaite accéder.
     On réalise donc la copie par une version de CopyEntity qui ne lance pas
     le démon (ce qui est de fait inutile dans ce cas). Cette version invoque
     des constructeurs par copie de BasicEntity, DescriptedEntity et Entity
     qui ne lancent ou ne lancent pas le démon selon la valeur de leur second
     argument.
    */  
   BasicEntity(const BasicEntity&, bool triggerMonitor); // _no swig_   

          /// Même effet que le constructeur ci-dessus, mais les éléments ou les composants ne sont pas nécessairement copiés. Si l'argument 'partsCopied' est FALSE, ce rôle est laissé aux constructeurs par copie des sous-classes de l'application. 
   BasicEntity(const BasicEntity&, int partsCopied); // _no swig_ 
   BasicEntity(const BasicEntity&, bool triggerMonitor, int partsCopied); // _no swig_   
          /// Destructeur superficiel (les composants et les éléments ne sont pas détruits ; le démon éventuellement attaché à la structure n'est pas détruit). Enlève l'entité en cours de destruction de la liste des instances (directes) de sa propre classe, puis de la liste des instances des classes parentes, jusqu'à la classe [C]BasicEntity[/C] (méthode [Y]RemoveInstanceFromClass[/Y]).
   ~BasicEntity(); // _no swig_ 
          /// Destructeur profond. Les composants et les éléments sont détruits, puis il est fait appel au destructeur superficiel. 
   void            DeleteInDepth(); // _no swig_ 
//@}                             

// @Man: OPERATORS
// @{

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const BasicEntity& right) const {
printf("\nentrée inattendue dans BasicEntity::operator (%x) < (%x)\n", this, &right);return TRUE;};
  Boolean         operator > (const BasicEntity& right) const {
printf("\nentrée inattendue dans BasicEntity::operator (%x) > (%x)\n", this, &right);return TRUE;};
// @}     

/**@name [SC]Identification de l'entité[/SC] */
//@{   
         /// Renvoie le type de l'entité : O\_BASIC\_ENTITY, O\_DESCRIPTED\_ENTITY ou O\_ENTITY. Ne pas confondre avec le symbole de classe : les classes descendant de [C]BasicEntity[/C] ont des symboles tous différents mais un type unique.
   ObjectClassId EntityType() const;

         /// Affecte un symbole de classe au récepteur. \index{méthodes!ClassSymbol - BasicEntity}
   void        ClassSymbol(UserClassId); // _no swig_

         /// Renvoie le symbole de classe du récepteur.
   UserClassId ClassSymbol() const;
         //  
   UserClassId ClassId() const;

   /// Affecte un nom de classe au récepteur du message. Toute instance doit se voir affecter un nom de classe (typiquement par un appel à la méthode [C]AddInstanceToClass[/C] dans le constructeur de la classe, voir pages 'Gestion de la hiérarchie des classes d'entités' et 'Définition et accès à la place dans la hiérarchie de classes'). Le nom de classe est le même pour toutes les instances d'une classe. Pour une exploitation dans le fichier de spécifications de sorties, le nom ne doit comporter que des caractères ASCII\index{nom de classe (entité)} sans espace. \index{méthodes!ClassName - BasicEntity}
   void        ClassName(char*); // _no swig_

         /// Renvoie le nom de classe du récepteur.
   char*       ClassName() const;

         //
   int       InstanceNumber() const;

         /// Affecte un nom d'instance au récepteur du message, qui surcharge le nom d'instance par défaut attribué par le moteur. Ce nom par défaut est la concaténation du nom de classe et d'un nombre entier positif unique sur l'ensemble de toutes les instances de toutes les classes de \D. \index{nom d'instance (entité)}\index{nom par défaut - BasicEntity}\index{méthodes!SetInstanceName - BasicEntity}\label{nomParDefaut}
   void        SetInstanceName(char*);

         // Affecte un nom d'instance au récepteur du message, qui est la concaténation d'une chaîne de caractères quelconque et d'un entier quelconque. Cette méthode n'est utilisée que par le moteur.
    void        SetInstanceName(char*, int); // _no swig_

         /// Renvoie le nom d'instance du récepteur.
   char*       InstanceName() const;

         /// Attache à l'entité la métaclasse correspondant au symbole de classe passé en argument. Doit être invoqué comme première instruction e tout constructeur d'entité. Lève l'exception [Y]MetaClassNotYetCreated[/Y] si la métaclasse n'a pas été créée par [Y]CreateMetaClass[/Y] (dans [Y]App_InstallMetaClasses()[/Y]).\index{méthodes!SetMetaClass - BasicEntity}
   void        SetMetaClass(UserClassId id); // _no swig_ 

   // Renvoie la métaclasse attachée à l'entité.\index{méthodes!GetMetaClassInstance - BasicEntity}
   MetaClass*  GetMetaClassInstance() const; // _no swig_ 

         // Equipe l'entité pour la prise en compte d'une spécification de sortie portant sur la classe de l'entité. Cette méthode n'est invoquée que par les constructeurs des sous-classes d'[C]Entity[/C]. Autrement dit, elle n'est pas invoquée lors des trois appels successifs aux constructeurs de [C]BasicEntity[/C], [C]DescriptedEntity[/C] et [C]Entity[/C], puis elle est invoquée lors des appels successifs aux constructeurs des sous-classes d'[C]Entity[/C] jusqu'à celle de 'this'. En conséquence, il est inutile d'écrire des spécifications de sortie portant sur les trois classes de base [C]BasicEntity[/C], [C]DescriptedEntity[/C] et [C]Entity[/C]. 
void           EquipForOSAccount(); // _no swig_
//@}   

/**@name [SC]Définition de la structure de l'entité, acc\`es[/SC] */
//@{
         /// Affecte un symbole de classe unique pour les sur-ensembles du récepteur. \index{méthodes!SuperSetClassId}
   void        SuperSetClassId(UserClassId); // _no swig_

         /// Affecte deux symboles de classe pour les sur-ensembles du récepteur. \index{méthodes!SuperSetClassIds}
   void        SuperSetClassIds(UserClassId, UserClassId); // _no swig_  

         /// Ajoute un symbole de classe pour les sur-ensembles du récepteur. \index{méthodes!AddSuperSetClassId}
   void        AddSuperSetClassId(UserClassId); // _no swig_

         /// Renvoie le symbole de classe des sur-ensembles du récepteur, si l'entité n'est élément que d'ensembles de la même classe. \index{méthodes!SuperSetClassId}
   UserClassId SuperSetClassId() const;

         /// Renvoie le tableau des symboles des classes des sur-ensembles du récepteur. \index{méthodes!SuperSetClassIdTab}
UserClassIdTab SuperSetClassIdTab() const;

         /// Affecte un symbole de classe unique pour les éléments du récepteur. \index{méthodes!ElementClassId}
   void        ElementClassId(UserClassId);  // _no swig_

         /// Renvoie le symbole de classe des éléments du récepteur. \index{méthodes!ElementClassId}
     UserClassId ElementClassId() const;

         /// Ajoute un symbole à la table des symboles de classe des composants du récepteur, et donne la valeur de pointeur NULL à l'élément correspondant du tableau des composants. Le récepteur pourra alors avoir un composant, {\em et un seul}, de la classe dont le symbole vient d'être ajouté ou d'une classe descendante. \index{méthodes!AddComponentClassId}
   void        AddComponentClassId(UserClassId); // _no swig_ 

         /// Renvoie la liste des symboles de classe des composants du récepteur. \index{méthodes!ComponentClassIdTab}
   UserClassIdTab ComponentClassIdTab() const;

         /// Renvoie la liste des composants du récepteur. \index{méthodes!GetComponentTab}
   pEntityTab   GetComponentTab() const;
         /// Donne comme valeur à l'argument {\em tab} la liste des composants du récepteur.
   void         GetComponentTab(pEntityTab& tab) const;  // _no swig_ 
         /// Renvoie la liste des entités de classe {\em id} qui sont en position de composants directs ou indirects dans les composants ou les éléments du récepteur, à quelque profondeur que ce soit.
   pEntityTab   GetDepthComponentTab(UserClassId id) const;
         /// Donne comme valeur à l'argument {\em tab} la liste des entités de classe {\em id} qui sont en position de composants directs ou indirects dans les composants ou les éléments du récepteur, à quelque profondeur que ce soit.
   void         GetDepthComponentTab(UserClassId id, pEntityTab& tab) const; // _no swig_ 

         /// Renvoie le composant dont le symbole de classe est soit {\em id} soit celui d'une classe descendante de celle de symbole {\em id}. La valeur NULL est renvoyée si le composant est présent mais de valeur NULL (affectée par exemple dès la construction de l'entité alors que le composant de ce type n'est pas encore créé). Lève l'exception [Y]BadComponent\-ClassId[/Y] si le composant est complètement absent. \index{méthodes!GetBasicComponent}
   BasicEntity*      GetBasicComponent(UserClassId id);

         /// Opère comme le service ci-dessus, mais suppose que le composant est une instance d'[C]Entity[/C]. Aucune exception n'est levée si le second argumenty est FALSE.
   BasicEntity*      GetBasicComponent(UserClassId id, Boolean failIfNull); // + JPR 081118

         /// Opère comme le service ci-dessus, mais suppose que le composant est une instance d'[C]Entity[/C].
   Entity*           GetComponent(UserClassId id);

         /// Opère comme le service ci-dessus, mais suppose que le composant est une instance d'[C]Entity[/C]. Aucune exception n'est levée si le second argumenty est FALSE.
   Entity*           GetComponent(UserClassId id, Boolean failIfNull); // + JPR 081118

         /// Renvoie le composant dont le nom de classe est soit {\em nom} soit celui d'une classe descendante de celle de nom {\em nom}. La valeur NULL est renvoyée si le composant est présent mais de valeur NULL (affectée par exemple dès la construction de l'entité alors que le composant de ce type n'est pas encore créé). Lève l'exception [Y]BadComponent\-Name[/Y] si le composant est complètement absent.
   BasicEntity*      GetBasicComponent(char* nom); // _no swig_

         /// Opère comme le service ci-dessus, mais suppose que le composant est une instance d'[C]Entity[/C].
   Entity*           GetComponent(char* nom); // _no swig_

         /// Renvoie la super-entité du récepteur ou le pointeur NULL s'il n'en existe pas. \index{méthodes!GetSuperEntity}
   BasicEntity*      GetBasicSuperEntity();

         /// Opère comme le service ci-dessus, mais suppose que la super-entité est une instance d'[C]Entity[/C]. 
   Entity*           GetSuperEntity();

         /// Renvoie la liste des sur-ensembles du récepteur. \index{méthodes!GetSuperSetTab}
   pEntityTab        GetSuperSetTab() const ;

         /// Renvoie le (ou le seul) sur-ensemble du récepteur. \index{méthodes!GetSuperSet}
   BasicEntity*      GetBasicSuperSet() const ; // _no swig_

         /// Opère comme le service ci-dessus, mais suppose que le sur-ensemble est une instance d'[C]Entity[/C]. 
   Entity*           GetSuperSet() const ; // _no swig_

         /// Renvoie le sur-ensemble de rang {\em k}, supposé valide, du récepteur. \index{méthodes!GetSuperSet(k)}
   BasicEntity*      GetBasicSuperSet(int k) const; 

         /// Opère comme le service ci-dessus, mais suppose que le sur-ensemble est une instance d'[C]Entity[/C]. 
   Entity*           GetSuperSet(int k) const;
 
         /// Renvoie le nombre de sur-ensembles du récepteur. \index{méthodes!GetNumberOfSuperSets}
   int               GetNumberOfSuperSets() const;

         /// Renvoie {\em vrai} si l'entité passée en argument est dans la liste des sur-ensembles de this, et {\em faux} sinon. \index{méthodes!IsSuperSetOf}
   bool              HasInSuperSets(BasicEntity*) const;

         /// Renvoie la liste des entités de la classe {\em id} qui contiennent le récepteur du message en position de composant ou d'élément à une profondeur quelconque. \index{méthodes!GetContainerTab}
  pEntityTab    GetContainerTab(UserClassId id) const;

         /// Renvoie une liste d'entités qui contiennent le récepteur du message en position de composant ou d'élément à une profondeur quelconque. Si le second argument est TRUE, les éléments de la liste renvoyée sont seulement de la classe {\em id}. S'il est FALSE, les éléments sont de la classe {\em id} ou d'une classe descendante.
  pEntityTab    GetContainerTab(UserClassId id, Boolean flag) const;

         /// Renvoie la première entité de la classe {\em id} (ou, si le second argument est 'vrai', d'une classe mère de {\em id}), trouvée en remontant à partir de 'this', dont 'this' est composant ou élément à une profondeur quelconque. \index{méthodes!GetFirstContainer}
  BasicEntity*  GetFirstContainer(UserClassId id, Boolean flag) const ;

         /// Renvoie la liste des élements du récepteur. \index{méthodes!GetElementTab}
   pEntityTab   GetElementTab() const;

         /// Donne comme valeur à l'argument {\em tab} la liste des élements du récepteur.
   void         GetElementTab(pEntityTab& tab) const; // _no swig_  

         /// Donne l'argument {\em tab} comme valeur de la liste des élements du récepteur.
   void         SetElementTab(pEntityTab& tab); // _no swig_ // + JPR 071017

         /// Définit l'incrément du tableau des éléments.
   void         SetElementTabIncrement(int); // _no swig_ // + JPR 071017

         /// Renvoie la liste des entités de la classe {\em id} ou d'une classe qui descend de {\em id}, qui sont éléments ou en position d'élements dans les composants ou les éléments du récepteur, à quelque profondeur que ce soit.
   pEntityTab   GetDepthElementTab(UserClassId id) const;

         /// Donne comme valeur à l'argument {\em tab} la liste des entités de la classe {\em id} ou d'une classe qui descend de {\em id}, qui sont éléments ou en position d'élements dans les composants ou les éléments du récepteur, à quelque profondeur que ce soit.
   void         GetDepthElementTab(UserClassId id, pEntityTab& tab) const; // _no swig_ 

         /// Renvoie {\em vrai} si une entité de la classe dont le symbole est passé en argument est un élément de this à une profondeur quelconque {\em faux} sinon. \index{méthodes!HasAsElement}
   Boolean      HasAsElement(UserClassId id) const; 

         /// Si l'entité passée en argument est un élément de this à une profondeur quelconque, cette entité est renvoyée. Renvoie NULL sinon.
   BasicEntity*      HasAsElement(BasicEntity*) const; // _no swig_ 
 
         /// Renvoie le premier (ou le seul) élement du récepteur. \index{méthodes!GetElement}
   BasicEntity*      GetBasicElement() const; // _no swig_ 
 
         /// Opère comme le service ci-dessus, mais suppose que l'élement est une instance d'[C]Entity[/C].  
   Entity*           GetElement() const; // _no swig_  

         /// Renvoie l'élement de rang {\em k} ({\em k} de {\em 0} à {\em n-1}) du récepteur, si un tel élément existe, et le pointeur NULL sinon. \index{méthodes!GetElement(k)}
   BasicEntity*      GetBasicElement(int k) const; 

         ///  Opère comme le service ci-dessus, mais suppose que l'élement est une instance d'[C]Entity[/C].
   Entity*           GetElement(int k) const; 

         /// Renvoie le dernier élement du récepteur. \index{méthodes!GetLastElement}
   BasicEntity*      GetLastElement() const;

         /// Renvoie l'élement dont le nom d'instance est passé en argument, et NULL si on ne l'a pas trouvé.
   BasicEntity*      GetNamedElement(char*) const;

         /// Renvoie l'élement, à une profondeur quelconque, dont le nom d'instance est passé en argument, et NULL si on ne l'a pas trouvé.
   BasicEntity*      GetDepthNamedElement(char*) const;

         /// Renvoie le premier élement dont le nom de classe est passé en argument.
   BasicEntity*      GetClassNamedElement(char*) const;

         /// Renvoie le premier élement, à une profondeur quelconque, dont le nom de classe est passé en argument.
   BasicEntity*      GetDepthClassNamedElement(char*) const;

         /// Renvoie le premier élement dont le symbole de classe est passé en argument.
   BasicEntity*      GetClassNamedElement(UserClassId) const; // _no swig_

         /// Renvoie le premier élement, à une profondeur quelconque, dont le symbole de classe est passé en argument.
   BasicEntity*      GetDepthClassNamedElement(UserClassId) const; // _no swig_

         /// Renvoie le nombre d'éléments du récepteur. \index{méthodes!GetNumberOfElements}
   int               GetNumberOfElements() const;
   int               GetNumberOfElements100901() const;  // to scratch

         /// Renvoie le rang de l'élément passé en argument (0 si c'est le premier élémént), ou -9 s'il n'est pas élément. \index{méthodes!GetElementPosition}
   int          GetElementPosition(BasicEntity*) const;
//@}   

/**@name [SC]Affichage[/SC] */
//@{
         /// Affiche les informations sur la structure du récepteur (composants, éléments,\ldots). \index{méthodes!Display}
   void                Display() const; 

         /// Affiche l'adresse du tableau des éléments et, entre deux parenthèses, la liste des valeurs de pointeurs sur les éléments. \index{méthodes!DisplayElementTab}
   void                DisplayElementTab() const; // _no swig_ 

         /// Affiche entre deux parenthèses la liste des noms des éléments, sans retour à la ligne entre eux. \index{méthodes!DisplayElementNames}
   void                DisplayElementNames() const; // _no swig_

         /// Affiche les informations sur la structure du récepteur, et aussi les valeurs des descripteurs.index{méthodes!DisplayAttributes} 
   void                DisplayAttributes() const; //

         /// Affiche les informations sur la structure du récepteur, ainsi que sur ses composants et éléments. Les lignes d'information sur l'entité réceptrice (de niveau {\em 0}) sont indentées de {\em k} caractères (typiquement {\em 0}). Les lignes sur les entités de plus bas niveaux ({\em n} = {\em 1}\ldots) sont indentées de $n*4$ caractères.
   void                DisplayInDepth(int k) const;
         /// A le même effet que [Y]DisplayInDepth(0)[/Y].
   void                DisplayInDepth() const; // _no swig_
         //
   void                DisplayShortInDepth(int k) const; // _no swig_
         //
   void                DisplayShortInDepth() const; // _no swig_

//@}                            

   /**@name [SC]Définition et accès à la place dans la hiérarchie de classes[/SC]
\\

Il est possible de définir une hiérarchie de classes pour les entités. Par exemple, on veut dire que {\em C} est une sorte de {\em B}, et que {\em B} est une sorte de {\em A}. Une fois cette hiérarchie définie (voir page 'Gestion de la hiérarchie des classes d'entités'), les services ci-dessus permettent de tester les liens de filiation entre classes, et de déterminer si une entité possède les propriétés d'une classe, soit parce qu'il est instance directe de la classe, soit parce que sa propre classe a hérité ces propriétés.
*/
//@{ 

          /// \label{defAddInstanceToClass}Attribue un symbole de classe et un nom de classe à l'entité réceptrice du message, puis ajoute l'entité à la liste des instances de sa classe. Crée la liste si l'entité est la première instance créée pour cette classe. {\em Attention ! Cette méthode doit obligatoirement être invoquée dans le constructeur de chaque sous-classe utilisateur des entités} (voir page 'Gestion de la hiérarchie des classes d'entités'). Elle inclut notamment les effets des appels [Y]ClassSymbol(id)[/Y] et [Y]ClassName(nomc)[/Y]. \index{méthodes!AddInstanceToClass}
void            AddInstanceToClass(); // _no swig_
         // 
//void            ReducedAddInstanceToClass(); // _no swig_  // + JPR 061219

         // Service 'moteur'.
void            AddInstanceToClass(Boolean noEquip); // _no swig_
         // 
//void            ReducedAddInstanceToClass(Boolean noEquip); // _no swig_  // + JPR 061219

         /// Cette méthode, décrite ici pour information, est invoquée dans le destructeur de la classe [C]BasicEntity[/C] et ne doit normalement pas être invoquée dans le code utilisateur. Elle retire l'entité de la liste des instances de sa classe. Attention ! cette méthode n'est pas le destructeur défini pour la classe. C'est ce dernier qui doit être invoqué pour libérer l'espace occupé par l'entité elle-même. \index{méthodes!RemoveInstanceFromClass}
 void           RemoveInstanceFromClass(UserClassId id); // _no swig_

 //---------
 // la gestion des classes parentes est reportée dans la classe MetaClass
 //
         // Cette fonction et la suivante indiquent que les symboles passés en arguments sont les seuls symboles de classe des classes parentes, au sens de l'héritage (multiple si plus d'une classe est mentionnée). Les arguments fournis doivent être compatibles avec la déclaration d'héritage faite dans le fichier d'entêtes. Elle n'établissent pas les liens hiérarchiques réciproques entre la classe de la présente Hinstance et ses classes mères. Utiliser pour cela les méthodes [Y]SetOneLevelHierarchy[/Y] et [Y]SetTotalHierarchy[/Y] après l'une ou l'autre ce ces fonctions. Las méthodes [Y]Set*Hierarchy[/Y] sont notamment invoquées lors de l'installation des instances de [C]MetaClass[/C], en préambule de la simulation. \index{méthodes!SetParentclassIds - BasicEntity}
 //void        SetParentclassId(UserClassId); // _no swig_
         ///
 //void        SetParentclassIds(UserClassId, UserClassId); // _no swig_

         // Ajoute un symbole de classe à la liste des symboles des classes parentes, sans modifier les liens hiérarchiques réciproques entre la classe de la présente instance et ses classes mères. A utiliser normalement que pour des besoins particuliers.
 //void           AddParentclassId(UserClassId);
 //---------

         /// Renvoie la liste des symboles des classes immédiatement parentes. Le symbole de la classe même du récepteur n'est pas inclus dans la liste. \index{méthodes!ParentIds - BasicEntity}
   UserClassIdTab ParentIds() const;

         /// Renvoie la liste des symboles des classes parentes, à tous les niveaux supérieurs. Le symbole de la classe même du récepteur n'est pas inclus dans la liste. \index{méthodes!AncestorIds - BasicEntity}
   UserClassIdTab AncestorIds() const;

         /// Etablit la liste des symboles des classes parentes, à tous les niveaux supérieurs. Le symbole de la classe même du récepteur n'est pas inclus dans la liste. Le résultat devient la valeur du second argument (transmis par référence au programme appelant).
   void           AncestorIds(UserClassIdTab& result) const; // _no swig_ 

         /// Renvoie {\em vrai} si le récepteur du message est une instance de la classe dont le symbole est passé en argument, et {\em faux} sinon. \index{méthodes!IsDirectInstanceOf}
   Boolean        IsDirectInstanceOf(UserClassId) const;

         /// Renvoie {\em vrai} si le récepteur du message est une instance de la classe dont le symbole est passé en argument ou d'une de ses classes descendantes, et {\em faux} sinon. \index{méthodes!IsInstanceOf}
   Boolean        IsInstanceOf(UserClassId) const; 
 
// début code gestion OutputSpec

        //   Renvoie la structure de stockage des données résultant des diverses requêtes de sorties pour cette instance.
   EntityInstanceOS*     GetEntityInstanceOS() const;  // _no swig_            

         /// Service normalement utilisé dans le constructeur d'une classe. Détermine le fichier (plus exactement l'instance de [C]OutputFile[/C] dans lequel seront écrits les informations demandées dans les spécifications de sortie attachées à la classe en question. \index{méthodes!SetOutputFile}
   void               SetOutputFile(OutputFile* pF); // +JPR 000128 (utilisé dans Tommy)

         /// Renvoie le fichier déterminé par l'utilisation du service ci-dessus. \index{méthodes!GetOutputFile}
   OutputFile*        GetOutputFile() const; // +JPR 000128 (utilisé dans Tommy) 

// fin code gestion OutputSpec
//@}                          

/**@name [SC]Opérations sur la structure de l'entité[/SC] */
//@{    
    
         /// Affecte l'entité passée en premier argument comme super-entité du récepteur. Lève l'exception [Y]SuperEntityAlreadyEquippedWithClass[/Y]. \index{méthodes!SetSuperEntity}
   void            SetSuperEntity(BasicEntity*) ;

         /// Etablit que la liste des composants du récepteur est le tableau donné en argument. La liste ancienne est remplacée le cas échéant. \index{méthodes!SetComponentTab}
   void            SetComponentTab(pEntityTab);

         /// Ajoute l'entité à la liste des composants du récepteur du message, s'il n'y est pas déjà. Le composant doit être d'une des classes déclarées par [Y]AddComponentClassId[/Y] ou d'une de leurs classes descendantes. \index{méthodes!AddComponent}
   void            AddComponent(BasicEntity*); 
         /// Retire l'entité de la liste des composants du récepteur du message, s'il y est déjà. Le composant doit être d'une des classes déclarées par [Y]AddComponentClassId[/Y] ou d'une de leurs classes descendantes. \index{méthodes!RemoveComponent}
   void            RemoveComponent(BasicEntity*); 

         /// Dans la liste des composants du récepteur du message, remplace par l'entité {\em pE} le composant du même type que {\em pE} s'il y est déjà, et ajoute simplement {\em pE} sinon. Le composant doit être d'une des classes déclarées par [Y]AddComponentClassId[/Y] ou d'une de leurs classes descendantes. \index{méthodes!ReplaceComponent}
   void            ReplaceComponent(BasicEntity* pE); 

         /// Ajoute l'entité à la liste des ensembles dont le récepteur est élément, s'il n'y est pas déjà. Lève l'exception [Y]BadSuperSetClass[/Y]. \index{méthodes!AddSuperSet}
   void            AddSuperSet(BasicEntity*);  

         /// Seulement appelé par [Y]AddElement[/Y]. Force l'ajoute de l'entité à la liste des ensembles dont le récepteur est élément. Lève l'exception [Y]BadSuperSetClass[/Y].
   void            ForceSuperSet(BasicEntity*);
 
         /// Enlève l'entité de la liste des sur-ensembles du récepteur. Renvoie TRUE si la suppression a bien eu lieu, c'est-à-dire si le sur-ensemble était dans la liste. \index{méthodes!RemoveSuperSet}
   Boolean         RemoveSuperSet(BasicEntity*); 

         // Service 'moteur', utilisé dans ~Entity() 
   Boolean         RemoveSuperSetNoPropagate(BasicEntity*); // _no swig_

         /// Ajoute l'entité à la liste des éléments du récepteur, s'il n'y est pas déjà. Le démon éventuellement attaché à l'entité sera déclenché. Lève l'exception [Y]BadSubEntityClass[/Y]. Renvoie l'élément si l'ajout a échoué ou si l'élément est NULL. \index{méthodes!AddElement}
   BasicEntity*    AddElement(BasicEntity*) ; // _no swig_

         /// Ajoute l'entité à la liste des éléments du récepteur, sans tester s'il y est déjà. Le démon éventuellement attaché à l'entité sera déclenché. Lève l'exception [Y]BadSubEntityClass[/Y]. Renvoie l'élément si l'ajout a échoué ou si l'élément est NULL. \index{méthodes!AddElementNoCheckIfNew}
   BasicEntity*    AddElementNoCheckIfNew(BasicEntity*) ; // _no swig_ // + JPR 071017

         /// Ajoute l'entité à la liste des éléments du récepteur, s'il n'y est pas déjà. Lève l'exception [Y]BadSubEntityClass[/Y]. Le démon éventuellement attaché à l'entité ne sera déclenché que si le second argument est TRUE. Renvoie l'élément si l'ajout a échoué ou si l'élément est NULL. 
   BasicEntity*    AddElement(BasicEntity*, Boolean triggerDeamon);

         /// Appelle [Y]AddElement(élément)[/Y] pour chaque {\em élément} de l'argument. Renvoie le pointeur NULL si tous les ajouts ont réussi, ou bien le pointeur sur le premier élément dont l'ajout a échoué. \index{méthodes!AddElements}
   BasicEntity*    AddElements(pEntityTab); // _no swig_ 

         /// Appelle [Y]AddElement(élément, leBooleen)[/Y] pour chaque {\em élément} du premier argument. Renvoie le pointeur NULL si tous les ajouts ont réussi, ou bien le pointeur sur le premier élément dont l'ajout a échoué.
   BasicEntity*    AddElements(pEntityTab, Boolean triggerDeamon); 

         /// Appelle [Y]AddElement(élément)[/Y] pour chaque {\em élément} de l'argument. Si l'ajout d'un élément a échoué, ou poursuit l'ajout des suivants. \index{méthodes!AddElementsIfNew}
   void    AddElementsIfNew(pEntityTab); // _no swig_  

         /// Appelle [Y]AddElement(élément, leBooleen)[/Y] pour chaque {\em élément} du premier argument. Si l'ajout d'un élément a échoué, ou poursuit l'ajout des suivants.
   void    AddElementsIfNew(pEntityTab, Boolean triggerDeamon); 

         // Seulement appelé par AddSuperSet. Force l'ajout l'entité à la liste des éléments du récepteur. Le démon éventuellement attaché à l'entité sera déclenché. Lève l'exception [Y]BadSubEntityClass[/Y].                              [ndlr : pas de retour sur AddSuperSet -> pas de création inutile de tableau]
   void            ForceElement(BasicEntity*);

         /// Insère l'entité dans la liste des éléments du récepteur, à la position indiquée par le premier argument. Avec un indexégal à 0, l'entité devient le premier élément. Le démon éventuellement attaché à l'entité sera déclenché. Lève l'exception [Y]BadSubEntityClass[/Y]. Renvoie l'élément si l'insertion a échoué ou si l'élément est NULL. \index{méthodes!InsertElement}
   BasicEntity*    InsertElement(int index, BasicEntity*) ; // _no swig_

         /// Retire l'entité argument de la liste des éléments du récepteur. Le démon éventuellement attaché à l'entité sera déclenché immédiatement avant le retrait de l'élément. Renvoie le pointeur NULL si l'élément est NULL ou s'il était dans la liste et a été retiré, et le pointeur sur l'entité en argument sinon. \index{méthodes!RemoveElement}
   BasicEntity*    RemoveElement(BasicEntity*); // _no swig_  

         /// Retire l'entité argument de la liste des éléments du récepteur. Le démon éventuellement attaché à l'entité ne sera déclenché que si le second argument est TRUE. Même valeur retournée que ci-dessus.
   BasicEntity*    RemoveElement(BasicEntity*, Boolean triggerDeamon);

         /// Retire l'entité de rang k de la liste des éléments du récepteur. Le démon éventuellement attaché à l'entité sera déclenché immédiatement avant le retrait de l'élément. Renvoie le pointeur NULL dans tous les cas. \index{méthodes!RemoveElement}
   BasicEntity*    RemoveElement(int k); // _no swig_  

         /// Retire l'entité de rang k de la liste des éléments du récepteur. Le démon éventuellement attaché à l'entité ne sera déclenché que si le second argument est TRUE. Même valeur retournée que ci-dessus.
   BasicEntity*    RemoveElement(int k, Boolean triggerDeamon);  // _no swig_ 

         // Service 'moteur', utilisé dans ~Entity()
   BasicEntity*    RemoveElementNoPropagate(BasicEntity*); // _no swig_ 

         /// Vide la liste des éléments du récepteur. Renvoie le pointeur NULL si tous les retraits ont réussi, ou bien le pointeur sur le premier élément dont le ratrait a échoué. \index{méthodes!RemoveElements}
   BasicEntity*    RemoveElements();

         /// Dans la liste des éléments du récepteur du message, remplace l'entité {\em pE} par les entités du tableau passé en second argument. Renvoie TRUE si le remplacement a bien eu lieu. \index{méthodes!ReplaceElement}
   Boolean         ReplaceElement(BasicEntity* pE, pEntityTab);

         /// Renvoie {\em vrai} si et seulement si l'entité n'est élément ou composant d'aucune autre. \index{méthodes!IsFree}
   Boolean         IsFree() const;
//@}                          


/**@name [SC]Acc\`es aux moniteurs (démons)[/SC]
Il s'agit ici des démons sur la structure de l'entité et non des démons sur les descripteurs, puisqu'une instance de la présente classe peut ne pas posséder de descripteurs. On ne peut manipuler des démons sur les descripteurs que si l'entité hérite de la classe de base [C]DescriptedEntity[/C].
 */     
//@{ 
          /// Renvoie le démon attaché à cette entité. \index{méthodes!GetMonitor - BasicEntity}
   StructureMonitor* GetMonitor() const;

          /// Attache à l'entité le démon passé en argument. Utilisé exclusivement par la méthode [Y]AttachToEntity[/Y] de [C]StructureMonitor[/C]. \index{méthodes!SetMonitor - BasicEntity}
   void              SetMonitor(StructureMonitor*);  // _no swig_

          // Service 'moteur'. Détermine si un démon est attaché (TRUE) ou non (FALSE) à l'entité.
   void              HasMonitor(Boolean); // _no swig_
//@}   

/**@name [SC]Acc\`es à la simulation[/SC]
Une simulation est une procédure qui pointe obligatoirement sur et une seule une entité, dite l'entité simulée. On peut établir la relation inverse, c'est-à-dire pointer, à partir d'une entité simulée (au sens ci-dessus), vers la simulation qui pointe ou pointera sur elle. Par 'simulation', on entend instance de la classe [C]Simulation[/C]. Chacune des deux déclarations est à réaliser explicitement
 : l'une n'est pas prise en charge par l'autre. */     
//@{ 
          /// Attache une instance de [C]Simulation[/C] à l'entité. Cette entité est la seule qui est ou qui sera l'entité simulé de l'instance de [C]Simulation[/C] passée en argument. \index{méthodes!SetSimulation - BasicEntity}
   void          SetSimulation(Simulation* pSim); // _no swig_

          /// Renvoie l'instance de  [C]Simulation[/C] attachée à l'entité. Renvoie donc NULL si cette entité n'est pas l'entité simulé de la simulation passée en argument.\index{méthodes!GetSimulation - BasicEntity}
   Simulation*   GetSimulation() const; // _no swig_ 

          /// Renvoie l'entité simulée dont 'this' est composant ou élément à profondeur quelconque, ou NULL si this est une instance libre ou si l'entité simulée n'a pas encore été désignée ou aucune simulation n'a encore été associée à l'entité simulée. \index{méthodes!GetSimulatedContainerEntity - BasicEntity}
   BasicEntity*  GetSimulatedContainerEntity() const; 
//@}
   
// INQUIRY

protected: 
   MetaClass*             mpMetaClass;

   ObjectClassId          mThisEntityType;
   static int             mInstanceNumber; 
          int             mThisInstanceNumber;

   char*                  mThisEntityInstanceName;

   UserClassId            mElementClassId;

   UserClassIdTab         mComponentClassIdTab;
   pEntityTab             mpComponentTab;

   UserClassIdTab         mSuperSetClassIdTab;
   pEntityTab             mpSuperSetTab;
   int                    mNbSuperSets;

   pEntityTab             mpElementTab;
   int                    mNbElements;

   BasicEntity*           mpSuperEntity;
   Boolean                mHasSuperEntity;

   StructureMonitor*      mpStructureMonitorAttached;
   Boolean                mHasStructureMonitorAttached;

  // début code gestion OutputSpec
   int                    mCreationDate;
   EntityInstanceOS*      mpInstanceOutputSpec;

   Boolean                mIsCreationSaved;
   Boolean                mIsChangeComponentSaved;
   Boolean                mIsChangeElementSaved;
  // fin code gestion OutputSpec

   OutputFile*            mpOutputFile;

   Simulation*            mpSimulation;  // + JPR 090817 (5.0)

protected:

// PRIVATE OPERATIONS

// PRIVATE ACCESS

   Boolean  HasComponents() const;
   Boolean  HasSuperSets() const;
   Boolean  HasElements() const;

   friend class Simulation; //appel ComponentTab HasSuperEntity HasElements
};  // _end class_

//@} // fin classe

// INLINE METHODS

// EXTERNAL REFERENCES


//-X///////////////X/////////////////////////////X////////////////////////////

         // Chronique des modifications de structure 

/* *\label{defChangeStructureItem}\index{ChangeStructureItem}La chronique des ajouts et retraits de composants ou d'éléments est enregistrée dans une liste renvoyée par un appel à [Y]GetChangeComponentList[/Y] ou [Y]GetChangeElementList[/Y]. Chaque élément de ces listes est un pointeur sur une instance de la présente classe. 
*/

class ChangeStructureItem // _no swig_
{
 public:
         // constructeur
  ChangeStructureItem(char*, char*, int, int);
         // destructeur
  ~ChangeStructureItem();

//  OPERATORS 
         // Opérateur d'égalité. Renvoie {\em vrai} si tous les champs sont identiques.
  Boolean       operator==(const ChangeStructureItem&) const;
         // Opérateur d'inégalité. Renvoie {\em vrai} si un champ est différents.
  Boolean       operator!=(const ChangeStructureItem&) const;

// ACCESS 
         // Renvoie le nom de classe du composant ou de l'élément.
  char*         GetClassName();
         // Renvoie le nom d'instance du composant ou de l'élément.
  char*         GetInstanceName();
         // Renvoie la date d'ajout ou de retrait.
  int           GetClock();
         // Renvoie le nombre résultant de composants ou d'éléments.
  int           GetResultingNumber();



 protected:
         // Le nom de classe du composant ou de l'élément
  char*           mClassName;
         // Le nom d'instance du composant ou de l'élément
  char*           mInstanceName;
         // La date d'ajout ou de retrait
  int           mClock;
         // Le nombre résultant de composante ou d'éléments
  int           mResultingNumber;
};

// fin Chronique des modifications de structure class ChangeStructureItem

//-X///////////////X/////////////////////////////X////////////////////////////


//-X///////////////X/////////////////////////////X////////////////////////////

#endif  // _BASICENTITY_HH
