//============================================================================
// FILE: Parameter.h
//
// TYPE: C++CLASS
// DESCRIPTION: entête de la classe générique des paramètres
// 
// AUTHORS: Rellier (INRA/UBIA, Toulouse)
//=X===============X=============================X============================

#ifndef _PARAMETER_HH
#define _PARAMETER_HH

// SYSTEM INCLUDES
//
//#include <>                                    //

// PROJECT INCLUDES
//
#include "CoreTypes.h"

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name CDPFORTEXLes paramètres
\begin{itemize} \item Directive d'inclusion : 
[P]
\#include Parameter.h
[/P]
\end{itemize}\label{defParameter}

Cette classe permet de stocker en mémoire les informations écrites par l'utilisateur du simulateur dans le fichier externe de paramètres du système (voir page 'Le fichier des paramètres du système' les règles d'écriture de ce fichier). Une instance de [C]Parameter[/C] sera créée pour chaque ligne du fichier paramètres.\\

La particularité du système de paramètres de DIESE est qu'un paramètre n'est pas simplement identifié par son nom de code (c'est-à-dire le nom d'une variable admis par le langage de programmation utilisé). Un paramètre est identifié par deux chaînes de caractères, celles que l'utilisateur a utilisées dans le fichier externe, et qui deviennent des attributs de l'instance de paramètre lors de sa création.\\

Les instances sont créées par l'exécution de la fonction [Y]ParseParameterFile[/Y] (voir page 'Les fichiers'). Un simple appel à cette fonction (au plus tard avant l'appel de la méthode [Y]Run[/Y]) évite donc au développeur du simulateur de coder lui-même la création des instances de paramètres à partir du fichier.\\

Les valeurs des paramètres sont, typiquement, affectées (avant ou après l'appel de [Y]Run[/Y]) à des descripteurs d'entités, ou à certains attributs d'autres classes de DIESE, telles que [C]Simulation[/C], [C]SequentialDataFile[/C], \ldots). On accède à la valeur des paramètres par les fonctions [Y]Get...ParameterValue[/Y] décrites ci-après, et on réalise les affectations par les méthodes appropriées de DIESE (par exemple les méthodes de la famille [Y]Set...Value[/Y] pour [C]Entity[/C]).\\

Pour un paramètre donné, le contenu des deux chaînes identifiantes est libre. Cependant, le développeur du simulateur pourra leur donner un contenu signifiant. Supposons par exemple que le développeur ait décidé que la position initiale d'une voiture sera un paramètre de la simulation de son parcours. Le paramètre pourra être identifié par les chaînes "voiture" et "positionInitiale". Le développeur signale ce choix à l'utilisateur, qui devra donc écrire dans le fichier des paramètres une ligne comme suit :

\begin{verbatim}
[P]     voiture positionInitiale <- 0.5;[/P]
\end{verbatim}

On aurait aussi pu écrire :

\begin{verbatim}
[P]     INIT voiture positionInitiale <- 0.5;[/P]
\end{verbatim}
... pour ranger le paramètre dans un groupe (ici 'INIT'), soit pour améliorer la clarté du fichier, soit pour utiliser cette notion de groupe dans les spécifications de sortie (voir page 'Les spécifications de sortie/Sauvegarde sur fichier séquentiel').

L'appel à [Y]ParseParameterFile[/Y] crée une instance de [C]Parameter[/C] avec les attributs "voiture" et "positionInitiale". L'exploitation du paramètre se fait par une séquence telle que :

\begin{verbatim}
[P]  Car* p = CreateCar();
  p->SetValue(POS_INIT, GetRealParameterValue("voiture", "positionInitiale"));[/P]
\end{verbatim}

La fonction [Y]ParseParameterFile[/Y] sait reconnaître le type d'un paramètre, grâce à la forme de la valeur écrite en fin de ligne sur le fichier (voir page 'Les fichiers/Le fichier des paramètres du système'}). Une incohérence entre la fonction utilisée pour accéder à la valeur et le type du paramètre identifié par les deux chaînes de caractères fournies en arguments provoque l'arrêt du programme. Par exemple, l'appel suivant est incorrect :

\begin{verbatim}
[P]  Car* p = CreateCar();
  p->SetValue(POS_INIT, GetIntParameterValue("voiture", "positionInitiale"));[/P]
\end{verbatim}
*/
//@{
          // / La classe des paramètres
class Parameter   // _no swig_
{
public:

  /* *@name LIFECYCLE */
  // @{
          // Constructeur de base. N'est normalement appelé que dans la fonction prédéfinie de lecture des paramètres sur le(s) fichier(s) externe(s), et non dans les classes définies par l'utilisateur.
  Parameter();  // _no swig_
          // Destructeur invoqué dans la fonction prédéfinie de destruction des paramètres.
   ~Parameter();   // _no swig_
// @}

// OPERATORS     

         // Opérateurs < et > (pour la forme).
  Boolean         operator < (const Parameter&) const {return TRUE;};
  Boolean         operator > (const Parameter&) const {return TRUE;};                           

  /* *@name  ACCESS */
  // @{
         // Affecte le paramètre qui reçoit ce message à un groupe.\index{méthodes!Group - Parameter}
   void              Group(char*);  // _no swig_
         // Renvoie le groupe auquel appartient paramètre.
   char*             Group() const;  // _no swig_
         // Affecte un type à la valeur du paramètre qui reçoit ce message.\index{méthodes!ValueType - Parameter}
   void              ValueType(ValueTypeId);  // _no swig_
         // Renvoie le type de la valeur du paramètre.
   ValueTypeId       ValueType() const;  // _no swig_
         // Affecte au descripteur la première chaîne de caractères qui le définit. Elle représente typiquement, mais non nécessairement, le nom d'une classe d'entités, à laquelle se rapporte le paramètre.\index{méthodes!EntityClassName - Parameter}
   void              EntityClassName(char*);  // _no swig_
         // Renvoie la chaîne de caractères affectée par la méthode ci-dessus.
   char*             EntityClassName() const;
         // Affecte au paramètre la seconde chaîne de caractères. Elle représente typiquement, mais non nécessairement, le nom d'une classe de descripteurs, à laquelle se rapporte le paramètre.\index{méthodes!DescriptorName - Parameter}  // _no swig_
   void              DescriptorName(char*);
         // Renvoie la chaîne de caractères affectée par la méthode ci-dessus.
   char*             DescriptorName() const;   // _no swig_
// @}

    // comptage const/dest
    static int   getNbConst() {return _nbConst;}
    static int   getNbDest() {return _nbDest;}      

    // comptage instances
    //static int                   InstanceNumber(); 
    //static pParameterTab         InstanceTab();    
    static void                  AddInstance(Parameter*);
        
    static pParameterTab          mpInstanceTab;
    
    static int                    mInstanceNumber; // modif 090213 

// INQUIRY

protected:

   int static   _nbConst;
   int static   _nbDest;

   ValueTypeId       mValueTypeId;
   char*             mRelatedEntityClassName;
   char*             mRelatedDescriptorName;
   char*             mGroup;                     // + JPR 061024

private:
};  // _end class_

//-X///////////////X/////////////////////////////X////////////////////////////

/**@name Fonctions de création/suppression, accès 
\\
{\large {\tt void  CreateParameter(ValueTypeId, char*, char*, UnifiedValue)}} : \\ 

\DTab4 Cette fonction crée un paramètre dont la valeur est du type passé en premier argument.  
Cette fonction n'est normalement pas invoquée directement par le développeur d'un simulateur, puisque celui-ci aura avantage à utiliser la fonction [Y]Parse\-ParameterFile[/Y]. La valeur, celle dont le type passé en premier argument, est passée en quatrième argument. Les deuxième et troisième arguments sont des chaînes de caractères qui identifient le paramètre. Elles serviront de critères d'accès à la valeur du paramètre dans les fonctions [Y]Get...ParameterValue[/Y]. Typiquement, ces chaînes peuvent rappeler un nom de classe de descripteur et un nom de classe d'entités, pour rendre plus claires les instructions d'affectation des valeurs des paramètres aux descripteurs des entités.\index{fonctions!CreateParameter}

{\large {\tt void  CreateParameter(ValueTypeId, char*, char*, UnifiedValue)}} : \\
 
\DTab4 Même fonction, avec affectation du paramètre à un groupe.

{\large {\tt void  DeleteParameters()}} : \\ 

\DTab4 Supprime toutes les instances de paramètres.\index{fonctions!DeleteParameters}

{\large {\tt Parameter*   GetParameter(char*, char*)}} : \\

\DTab4 Renvoie le paramètre identifié par deux chaînes de caractères affectées à sa création.\index{fonctions!GetParameter}
*/ 
//@{
void  CreateParameter(ValueTypeId, char*, char*, UnifiedValue);   // _no swig_
void  CreateParameter(ValueTypeId, char*, char*, UnifiedValue, char*);   // _no swig_

void  DeleteParameters();  // _no swig_ 

Parameter*   GetParameter(char*, char*);   
//@}
 
/**@name Accès à la valeur d'un paramètre
\\
{\large {\tt int   GetIntParameterValue(char*, char*)}} : \\

\DTab4 Renvoie la valeur entière d'un paramètre, identifié par deux chaînes de caractères affectées à sa création.\\ 
Le paramètre doit être de type entier.\index{fonctions!Get...ParameterValue}

{\large {\tt double GetRealParameterValue(char*, char*)}} : \\

\DTab4 Renvoie la valeur réelle (flottante double précision) d'un paramètre, identifié par deux chaînes de caractères affectées à sa création.\\ 
Le paramètre doit être de type réel.

{\large {\tt char* GetStringParameterValue(char*, char*)}} : \\

\DTab4 Renvoie la chaîne de caractères qui est la valeur d'un paramètre, identifié par deux chaînes de caractères affectées à sa création.\\ 
Le paramètre doit être de type chaîne de caractères.

{\large {\tt Date* GetDateParameterValue(char*, char*)}} : \\

\DTab4 Renvoie l'instance de la classe [C]Date[/C] qui est la valeur d'un paramètre de type date, identifié par deux chaînes de caractères affectées à sa création.

{\large {\tt tm* GetTmParameterValue(char*, char*)}} : \\

\DTab4 Renvoie la structure C tm qui est la valeur d'un paramètre de type date, identifié par deux chaînes de caractères affectées à sa création.

{\large {\tt int GetClockParameterValue(char*, char*)}} : \\

\DTab4 Renvoie la valeur d'horloge qui correspond à la valeur d'un paramètre de type date, identifié par deux chaînes de caractères affectées à sa création.

{\large {\tt int GetJulianDayParameterValue(char*, char*)}} : \\ 

\DTab4 Renvoie le jour julien correspondant à la valeur d'un paramètre de type date, identifié par deux chaînes de caractères affectées à sa création.
*/
//@{

int   GetIntParameterValue(char*, char*);  

double GetRealParameterValue(char*, char*);  
 
char* GetStringParameterValue(char*, char*);

Date* GetDateParameterValue(char*, char*);
 
tm* GetTmParameterValue(char*, char*);

int GetClockParameterValue(char*, char*); 

int GetJulianDayParameterValue(char*, char*);
//@}
 
/**@name Etablissement de la valeur d'un paramètre 
\\
{\large {\tt void  SetIntParameterValue(char*, char*, int)}} : \\

\DTab4 Affecte une valeur entière au paramètre, identifié par deux chaînes de caractères affectées à sa création.\\  
Le paramètre doit être de type entier.\index{fonctions!Set...ParameterValue}

{\large {\tt void  SetRealParameterValue(char*, char*, double)}} : \\

\DTab4 Affecte une valeur réelle (flottante double précision) d'un paramètre, identifié par deux chaînes de caractères affectées à sa création..\\  
Le paramètre doit être de type réel.

{\large {\tt void  SetStringParameterValue(char*, char*, char*)}} : \\ 

\DTab4 Affecte une chaîne de caractères comme valeur d'un paramètre, identifié par deux chaînes de caractères affectées à sa création.\\ 
Le paramètre doit être de type chaîne de caractères. 
*/
//@{

void SetIntParameterValue(char*, char*, int);  

void SetRealParameterValue(char*, char*, double);  
 
void SetStringParameterValue(char*, char*, char*); 
//@}

//-X///////////////X/////////////////////////////X////////////////////////////

/* *@name Les sous-classes typ\'ees de param\`etres */
// @{

/* *@name La classe des paramètres à valeur entière */
// @{
        // /
class IntParameter : public Parameter  // _no swig_
{
public:

// LIFECYCLE
         // Constructeur avec attribution d'un nom de classe d'entité associé et d'un nom de descripteur associé. Affecte le type INT au paramètre.
   IntParameter(char*, char*);
         // Constructeur avec attribution d'un nom de classe d'entité associé, un nom de descripteur associé et un groupe d'appartenance. Affecte le type INT au paramètre.
   IntParameter(char*, char*, char*);

// OPERATORS

// OPERATIONS

// ACCESS 
                           
          // Renvoie la valeur entière du paramètre.\index{méthodes!GetIntValue - Parameter}
   int      GetIntValue() const; 
          // Affecte une valeur entière au paramètre.\index{méthodes!SetValue - Parameter}
   void     SetValue(int);      

// INQUIRY

protected:
   int             mValue;

private:
};  // _end class_

// @} // fin classe

//-X///////////////X/////////////////////////////X//////////////////////////// 

/* *@name La classe des paramètres à valeur date */
// @{
        // /
class DateParameter : public Parameter  // _no swig_
{
public:

// LIFECYCLE
         // Constructeur avec attribution d'un nom de classe d'entité associé et d'un nom de descripteur associé. Affecte le type VOID au paramètre.
   DateParameter(char*, char*);
         // Constructeur avec attribution d'un nom de classe d'entité associé, un nom de descripteur associé et un groupe d'appartenance. Affecte le type VOID au paramètre.
   DateParameter(char*, char*, char*); 
         // Pour désallouer la valeur date.
   ~DateParameter();

// OPERATORS

// OPERATIONS

// ACCESS 
                                       
          // Renvoie la date valeur du paramètre.\index{méthodes!GetDateValue - Parameter}
   Date*    GetDateValue() const;                           
          // Renvoie la structure tm correspondant à la valeur du paramètre.\index{méthodes!GetTmValue - Parameter}
   tm*      GetTmValue() const;  
          // Renvoie la valeur d'horloge correspondant à la valeur du paramètre.\index{méthodes!GetClockValue - Parameter}
   int      GetClockValue() const; 
          // Renvoie le jour joulien correspondant à la valeur du paramètre de type date.\index{méthodes!GetClockValue - Parameter}
   int      GetJulianDayValue() const;

          // Affecte une valeur Date au paramètre.\index{méthodes!SetValue - Parameter}
   void     SetValue(Date*);   
   
// INQUIRY

protected:
   Date*           mValue;

private:
};  // _end class_

// @} // fin classe

//-X///////////////X/////////////////////////////X//////////////////////////// 

/* *@name La classe des paramètres à valeur r\'eelle */
// @{
        // /
class RealParameter : public Parameter  // _no swig_
{
public:

// LIFECYCLE
         // Constructeur avec attribution d'un nom de classe d'entité associé et d'un nom de descripteur associé. Affecte le type DOUBLE au paramètre.
   RealParameter(char*, char*);
         // Constructeur avec attribution d'un nom de classe d'entité associé, un nom de descripteur associé et un groupe d'appartenance. Affecte le type DOUBLE au paramètre.
   RealParameter(char*, char*, char*);

// OPERATORS

// OPERATIONS 
                           
// ACCESS        
          // Renvoie la valeur réelle du paramètre.\index{méthodes!GetRealValue - Parameter}
   double   GetRealValue() const; 
          // Affecte une valeur réelle au paramètre.\index{méthodes!SetValue - Parameter}
   void     SetValue(double);      

// INQUIRY

protected:
   double            mValue;

private:
};  // _end class_

// @} // fin classe

//-X///////////////X/////////////////////////////X////////////////////////////

/* *@name La classe des paramètres à valeur 'chaine de caract\`eres' */
// @{
        // /
class StringParameter : public Parameter  // _no swig_
{
public:

// LIFECYCLE
         // Constructeur avec attribution d'un nom de classe d'entité associé et d'un nom de descripteur associé. Affecte le type STRING au paramètre.
   StringParameter(char*, char*); 
         // Constructeur avec attribution d'un nom de classe d'entité associé, un nom de descripteur associé et un groupe d'appartenance. Affecte le type STRING au paramètre.
   StringParameter(char*, char*, char*); 
         // Pour désallouer la chaîne valeur.
   ~StringParameter();

// OPERATORS

// OPERATIONS 
                           
// ACCESS      
          // Renvoie la chaîne de caractères valeur du paramètre.\index{méthodes!GetStringValue - Parameter}
   char*      GetStringValue() const; 
          // Affecte une chaîne de caractères comme valeur du paramètre.\index{méthodes!SetValue - Parameter}
   void     SetValue(char*);      

// INQUIRY

protected:
   char*             mValue;

private:
};  // _end class_

// @} // fin classe

// @} // fin classe typées

//@} // fin parametres


#endif  // _PARAMETER_HH
